/* Driver template for the LEMON parser generator.
** The author disclaims copyright to this source code.
*/
/* First off, code is include which follows the "include" declaration
** in the input file. */
#include <stdio.h>
#line 4 "mlr_dsl_parse.y"

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "../lib/mlrutil.h"
#include "../containers/mlr_dsl_ast.h"
#include "../containers/sllv.h"

// ================================================================
// AST:
// * parens, commas, semis, line endings, whitespace are all stripped away
// * variable names and literal values remain as leaf nodes of the AST
// * = + - * / ** {function names} remain as non-leaf nodes of the AST
// CST: See mlr_dsl_cst.c
//
// Note: This parser accepts many things that are invalid, e.g.
// * begin{end{}} -- begin/end not at top level
// * begin{$x=1} -- references to stream records at begin/end
// * break/continue outside of for/while/do-while
// * $x=x -- boundvars outside of for-loop variable bindings
// All of the above are enforced by the CST builder, which takes this parser's output AST as input.
// This is done (a) to keep this grammar from being overly complex, and (b) so we can get much more
// informative error messages in C than in Lemon ('syntax error').
//
// The parser hooks all build up an abstract syntax tree specifically for the CST builder.
// For clearer visuals on what the ASTs look like:
// * See mlr_dsl_cst.c
// * See reg_test/run's filter -v and put -v outputs, e.g. in reg_test/expected/out
// * Do "mlr -n put -v 'your expression goes here'"
// ================================================================


#line 41 "mlr_dsl_parse.c"
/* Next is all token values, in a form suitable for use by makeheaders.
** This section will be null unless lemon is run with the -m switch.
*/
/*
** These constants (all generated automatically by the parser generator)
** specify the various kinds of tokens (terminals) that the parser
** understands.
**
** Each symbol here is a terminal symbol in the grammar.
*/
/* Make sure the INTERFACE macro is defined.
*/
#ifndef INTERFACE
# define INTERFACE 1
#endif
/* The next thing included is series of defines which control
** various aspects of the generated parser.
**    YYCODETYPE         is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 terminals
**                       and nonterminals.  "int" is used otherwise.
**    YYNOCODE           is a number of type YYCODETYPE which corresponds
**                       to no legal terminal or nonterminal number.  This
**                       number is used to fill in empty slots of the hash
**                       table.
**    YYFALLBACK         If defined, this indicates that one or more tokens
**                       have fall-back values which should be used if the
**                       original value of the token will not parse.
**    YYACTIONTYPE       is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 rules and
**                       states combined.  "int" is used otherwise.
**    ParseTOKENTYPE     is the data type used for minor tokens given
**                       directly to the parser from the tokenizer.
**    YYMINORTYPE        is the data type used for all minor tokens.
**                       This is typically a union of many types, one of
**                       which is ParseTOKENTYPE.  The entry in the union
**                       for base tokens is called "yy0".
**    YYSTACKDEPTH       is the maximum depth of the parser's stack.
**    ParseARG_SDECL     A static variable declaration for the %extra_argument
**    ParseARG_PDECL     A parameter declaration for the %extra_argument
**    ParseARG_STORE     Code to store %extra_argument into pparser
**    ParseARG_FETCH     Code to extract %extra_argument from pparser
**    YYNSTATE           the combined number of states.
**    YYNRULE            the number of rules in the grammar
**    YYERRORSYMBOL      is the code number of the error symbol.  If not
**                       defined, then do no error processing.
*/
#define YYCODETYPE unsigned char
#define YYNOCODE 196
#define YYACTIONTYPE unsigned short int
#define ParseTOKENTYPE mlr_dsl_ast_node_t*
typedef union {
  ParseTOKENTYPE yy0;
  mlr_dsl_ast_node_t* yy121;
  int yy391;
} YYMINORTYPE;
#define YYSTACKDEPTH 800
#define ParseARG_SDECL mlr_dsl_ast_t* past;
#define ParseARG_PDECL ,mlr_dsl_ast_t* past
#define ParseARG_FETCH mlr_dsl_ast_t* past = pparser->past
#define ParseARG_STORE pparser->past = past
#define YYNSTATE 587
#define YYNRULE 307
#define YYERRORSYMBOL 96
#define YYERRSYMDT yy391
#define YY_NO_ACTION      (YYNSTATE+YYNRULE+2)
#define YY_ACCEPT_ACTION  (YYNSTATE+YYNRULE+1)
#define YY_ERROR_ACTION   (YYNSTATE+YYNRULE)

/* Next are that tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N < YYNSTATE                  Shift N.  That is, push the lookahead
**                                      token onto the stack and goto state N.
**
**   YYNSTATE <= N < YYNSTATE+YYNRULE   Reduce by rule N-YYNSTATE.
**
**   N == YYNSTATE+YYNRULE              A syntax error has occurred.
**
**   N == YYNSTATE+YYNRULE+1            The parser accepts its input.
**
**   N == YYNSTATE+YYNRULE+2            No such action.  Denotes unused
**                                      slots in the yy_action[] table.
**
** The action table is constructed as a single large table named yy_action[].
** Given state S and lookahead X, the action is computed as
**
**      yy_action[ yy_shift_ofst[S] + X ]
**
** If the index value yy_shift_ofst[S]+X is out of range or if the value
** yy_lookahead[yy_shift_ofst[S]+X] is not equal to X or if yy_shift_ofst[S]
** is equal to YY_SHIFT_USE_DFLT, it means that the action is not in the table
** and that yy_default[S] should be used instead.
**
** The formula above is for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the yy_reduce_ofst[] array is used in place of
** the yy_shift_ofst[] array and YY_REDUCE_USE_DFLT is used in place of
** YY_SHIFT_USE_DFLT.
**
** The following are the tables generated in this section:
**
**  yy_action[]        A single table containing all actions.
**  yy_lookahead[]     A table containing the lookahead for each entry in
**                     yy_action.  Used to detect hash collisions.
**  yy_shift_ofst[]    For each state, the offset into yy_action for
**                     shifting terminals.
**  yy_reduce_ofst[]   For each state, the offset into yy_action for
**                     shifting non-terminals after a reduce.
**  yy_default[]       Default action for each state.
*/
static YYACTIONTYPE yy_action[] = {
 /*     0 */   895,    1,    2,    4,    6,    7,    8,    9,   10,   11,
 /*    10 */    12,   13,   14,   15,   16,   17,   18,   19,   20,   21,
 /*    20 */    22,   23,   24,   25,   26,   27,   28,   29,   30,   31,
 /*    30 */    32,   33,   34,   35,   36,   37,   38,   39,   40,   41,
 /*    40 */    42,   43,   44,   45,   46,   47,   48,   49,   50,   51,
 /*    50 */    52,   53,   54,   55,   56,   57,   58,   59,   60,   71,
 /*    60 */   567,  587,  277,  218,   92,  226,  205,  236,  566,  132,
 /*    70 */   130,  131,  161,  162,  163,  113,  146,  156,   90,   91,
 /*    80 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*    90 */   177,  117,  126,  160,  172,  173,  174,  586,    2,    4,
 /*   100 */     6,    7,    8,    9,   10,   11,   12,   13,   14,   15,
 /*   110 */    16,   17,   18,   19,   20,   21,   22,   23,   24,   25,
 /*   120 */    26,   27,   28,   29,   30,   31,   32,   33,   34,   35,
 /*   130 */    36,   37,   38,   39,   40,   41,   42,   43,   44,   45,
 /*   140 */    46,   47,   48,   49,   50,   51,   52,   53,   54,   55,
 /*   150 */    56,   57,   58,   59,   60,   71,    3,  189,  277,  218,
 /*   160 */   190,  226,   64,  236,   66,  132,  101,  137,  199,  201,
 /*   170 */   115,  148,  158,  167,   90,   91,   96,  135,  136,  152,
 /*   180 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*   190 */   172,  173,  174,    5,    2,    4,    6,    7,    8,    9,
 /*   200 */    10,   11,   12,   13,   14,   15,   16,   17,   18,   19,
 /*   210 */    20,   21,   22,   23,   24,   25,   26,   27,   28,   29,
 /*   220 */    30,   31,   32,   33,   34,   35,   36,   37,   38,   39,
 /*   230 */    40,   41,   42,   43,   44,   45,   46,   47,   48,   49,
 /*   240 */    50,   51,   52,   53,   54,   55,   56,   57,   58,   59,
 /*   250 */    60,   71,  221,  552,  277,  218,   68,  226,  219,  236,
 /*   260 */   220,  132,  161,  162,  163,  328,  335,  339,   70,   72,
 /*   270 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*   280 */   155,  166,  177,  117,  126,  160,  172,  173,  174,   65,
 /*   290 */     2,    4,    6,    7,    8,    9,   10,   11,   12,   13,
 /*   300 */    14,   15,   16,   17,   18,   19,   20,   21,   22,   23,
 /*   310 */    24,   25,   26,   27,   28,   29,   30,   31,   32,   33,
 /*   320 */    34,   35,   36,   37,   38,   39,   40,   41,   42,   43,
 /*   330 */    44,   45,   46,   47,   48,   49,   50,   51,   52,   53,
 /*   340 */    54,   55,   56,   57,   58,   59,   60,   71,  562,   74,
 /*   350 */   277,  218,  371,  226,  383,  236,   76,  132,  511,  513,
 /*   360 */   515,  387,  151,  151,  393,  151,   90,   91,   96,  135,
 /*   370 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*   380 */   126,  160,  172,  173,  174,   69,    2,    4,    6,    7,
 /*   390 */     8,    9,   10,   11,   12,   13,   14,   15,   16,   17,
 /*   400 */    18,   19,   20,   21,   22,   23,   24,   25,   26,   27,
 /*   410 */    28,   29,   30,   31,   32,   33,   34,   35,   36,   37,
 /*   420 */    38,   39,   40,   41,   42,   43,   44,   45,   46,   47,
 /*   430 */    48,   49,   50,   51,   52,   53,   54,   55,   56,   57,
 /*   440 */    58,   59,   60,   71,  406,  410,  277,  218,  393,  226,
 /*   450 */   423,  236,   78,  132,  427,  151,  432,  393,   79,  393,
 /*   460 */    81,  151,   90,   91,   96,  135,  136,  152,  165,  141,
 /*   470 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*   480 */   174,   73,    2,    4,    6,    7,    8,    9,   10,   11,
 /*   490 */    12,   13,   14,   15,   16,   17,   18,   19,   20,   21,
 /*   500 */    22,   23,   24,   25,   26,   27,   28,   29,   30,   31,
 /*   510 */    32,   33,   34,   35,   36,   37,   38,   39,   40,   41,
 /*   520 */    42,   43,   44,   45,   46,   47,   48,   49,   50,   51,
 /*   530 */    52,   53,   54,   55,   56,   57,   58,   59,   60,   71,
 /*   540 */   446,  458,  277,  218,  481,  226,   83,  236,   85,  132,
 /*   550 */   462,  151,  151,  468,  485,  151,   86,  468,   90,   91,
 /*   560 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*   570 */   177,  117,  126,  160,  172,  173,  174,   80,    2,    4,
 /*   580 */     6,    7,    8,    9,   10,   11,   12,   13,   14,   15,
 /*   590 */    16,   17,   18,   19,   20,   21,   22,   23,   24,   25,
 /*   600 */    26,   27,   28,   29,   30,   31,   32,   33,   34,   35,
 /*   610 */    36,   37,   38,   39,   40,   41,   42,   43,   44,   45,
 /*   620 */    46,   47,   48,   49,   50,   51,   52,   53,   54,   55,
 /*   630 */    56,   57,   58,   59,   60,   71,  498,  502,  277,  218,
 /*   640 */   468,  226,   87,  236,   89,  132,  507,  151,  569,  468,
 /*   650 */    94,  583,   99,   97,   90,   91,   96,  135,  136,  152,
 /*   660 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*   670 */   172,  173,  174,   84,    2,    4,    6,    7,    8,    9,
 /*   680 */    10,   11,   12,   13,   14,   15,   16,   17,   18,   19,
 /*   690 */    20,   21,   22,   23,   24,   25,   26,   27,   28,   29,
 /*   700 */    30,   31,   32,   33,   34,   35,   36,   37,   38,   39,
 /*   710 */    40,   41,   42,   43,   44,   45,   46,   47,   48,   49,
 /*   720 */    50,   51,   52,   53,   54,   55,   56,   57,   58,   59,
 /*   730 */    60,   71,  579,  105,  277,  218,  107,  226,  109,  236,
 /*   740 */   127,  132,  134,  151,  164,  176,  151,  182,  187,  184,
 /*   750 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*   760 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  216,
 /*   770 */     2,    4,    6,    7,    8,    9,   10,   11,   12,   13,
 /*   780 */    14,   15,   16,   17,   18,   19,   20,   21,   22,   23,
 /*   790 */    24,   25,   26,   27,   28,   29,   30,   31,   32,   33,
 /*   800 */    34,   35,   36,   37,   38,   39,   40,   41,   42,   43,
 /*   810 */    44,   45,   46,   47,   48,   49,   50,   51,   52,   53,
 /*   820 */    54,   55,   56,   57,   58,   59,   60,   71,  194,  198,
 /*   830 */   277,  218,  204,  226,  208,  236,  209,  132,  566,  210,
 /*   840 */   211,  212,  214,  215,  222,  217,   90,   91,   96,  135,
 /*   850 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*   860 */   126,  160,  172,  173,  174,  550,    2,    4,    6,    7,
 /*   870 */     8,    9,   10,   11,   12,   13,   14,   15,   16,   17,
 /*   880 */    18,   19,   20,   21,   22,   23,   24,   25,   26,   27,
 /*   890 */    28,   29,   30,   31,   32,   33,   34,   35,   36,   37,
 /*   900 */    38,   39,   40,   41,   42,   43,   44,   45,   46,   47,
 /*   910 */    48,   49,   50,   51,   52,   53,   54,   55,   56,   57,
 /*   920 */    58,   59,   60,   71,  224,  225,  277,  218,  228,  226,
 /*   930 */   231,  236,  230,  132,  233,  272,  320,  274,  275,  314,
 /*   940 */   330,  331,   90,   91,   96,  135,  136,  152,  165,  141,
 /*   950 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*   960 */   174,  232,    2,    4,    6,    7,    8,    9,   10,   11,
 /*   970 */    12,   13,   14,   15,   16,   17,   18,   19,   20,   21,
 /*   980 */    22,   23,   24,   25,   26,   27,   28,   29,   30,   31,
 /*   990 */    32,   33,   34,   35,   36,   37,   38,   39,   40,   41,
 /*  1000 */    42,   43,   44,   45,   46,   47,   48,   49,   50,   51,
 /*  1010 */    52,   53,   54,   55,   56,   57,   58,   59,   60,   71,
 /*  1020 */   337,  338,  277,  218,  341,  226,  557,  236,  342,  132,
 /*  1030 */   345,  350,  354,  358,  362,  364,  368,  375,   90,   91,
 /*  1040 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  1050 */   177,  117,  126,  160,  172,  173,  174,  554,    2,    4,
 /*  1060 */     6,    7,    8,    9,   10,   11,   12,   13,   14,   15,
 /*  1070 */    16,   17,   18,   19,   20,   21,   22,   23,   24,   25,
 /*  1080 */    26,   27,   28,   29,   30,   31,   32,   33,   34,   35,
 /*  1090 */    36,   37,   38,   39,   40,   41,   42,   43,   44,   45,
 /*  1100 */    46,   47,   48,   49,   50,   51,   52,   53,   54,   55,
 /*  1110 */    56,   57,   58,   59,   60,   71,  377,  380,  277,  218,
 /*  1120 */   388,  226,  390,  236,  398,  132,  400,  403,  411,  415,
 /*  1130 */   417,  420,  428,  433,   90,   91,   96,  135,  136,  152,
 /*  1140 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  1150 */   172,  173,  174,  559,    2,    4,    6,    7,    8,    9,
 /*  1160 */    10,   11,   12,   13,   14,   15,   16,   17,   18,   19,
 /*  1170 */    20,   21,   22,   23,   24,   25,   26,   27,   28,   29,
 /*  1180 */    30,   31,   32,   33,   34,   35,   36,   37,   38,   39,
 /*  1190 */    40,   41,   42,   43,   44,   45,   46,   47,   48,   49,
 /*  1200 */    50,   51,   52,   53,   54,   55,   56,   57,   58,   59,
 /*  1210 */    60,   71,  437,  439,  277,  218,  443,  226,  450,  236,
 /*  1220 */   452,  132,  455,  463,  465,  473,  475,  478,  486,  490,
 /*  1230 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  1240 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  564,
 /*  1250 */     2,    4,    6,    7,    8,    9,   10,   11,   12,   13,
 /*  1260 */    14,   15,   16,   17,   18,   19,   20,   21,   22,   23,
 /*  1270 */    24,   25,   26,   27,   28,   29,   30,   31,   32,   33,
 /*  1280 */    34,   35,   36,   37,   38,   39,   40,   41,   42,   43,
 /*  1290 */    44,   45,   46,   47,   48,   49,   50,   51,   52,   53,
 /*  1300 */    54,   55,   56,   57,   58,   59,   60,   71,  492,  495,
 /*  1310 */   277,  218,  503,  226,  508,  236,  522,  132,  526,  530,
 /*  1320 */   538,  542,  546,  551,  553,  558,   90,   91,   96,  135,
 /*  1330 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  1340 */   126,  160,  172,  173,  174,  576,    2,    4,    6,    7,
 /*  1350 */     8,    9,   10,   11,   12,   13,   14,   15,   16,   17,
 /*  1360 */    18,   19,   20,   21,   22,   23,   24,   25,   26,   27,
 /*  1370 */    28,   29,   30,   31,   32,   33,   34,   35,   36,   37,
 /*  1380 */    38,   39,   40,   41,   42,   43,   44,   45,   46,   47,
 /*  1390 */    48,   49,   50,   51,   52,   53,   54,   55,   56,   57,
 /*  1400 */    58,   59,   60,   71,  555,  560,  277,  218,  563,  226,
 /*  1410 */   565,  236,  585,  132,  568,  570,  571,  574,  575,  572,
 /*  1420 */   577,  580,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  1430 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  1440 */   174,  581,    2,    4,    6,    7,    8,    9,   10,   11,
 /*  1450 */    12,   13,   14,   15,   16,   17,   18,   19,   20,   21,
 /*  1460 */    22,   23,   24,   25,   26,   27,   28,   29,   30,   31,
 /*  1470 */    32,   33,   34,   35,   36,   37,   38,   39,   40,   41,
 /*  1480 */    42,   43,   44,   45,   46,   47,   48,   49,   50,   51,
 /*  1490 */    52,   53,   54,   55,   56,   57,   58,   59,   60,   71,
 /*  1500 */   582,  584,  277,  218,  504,  226,  504,  236,  504,  132,
 /*  1510 */   213,  556,  504,  504,  144,  504,  504,  504,   90,   91,
 /*  1520 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  1530 */   177,  117,  126,  160,  172,  173,  174,  592,   61,   62,
 /*  1540 */    63,  111,  504,   67,   75,  185,  347,   82,  207,  504,
 /*  1550 */   504,  313,  504,  175,  227,  344,  278,  234,  151,  271,
 /*  1560 */   281,  283,  285,  287,  289,  291,  293,  295,  297,  299,
 /*  1570 */   301,  303,  305,  307,  309,  311,  504,  160,  316,  573,
 /*  1580 */   327,  161,  162,  163,  343,  360,  435,  510,  517,  518,
 /*  1590 */   532,  534,  548,   61,   62,   63,  504,  504,   67,   75,
 /*  1600 */   185,  470,   82,  207,  504,  504,  313,  504,  175,  227,
 /*  1610 */   118,  120,  234,  103,  271,  461,  504,  122,  124,  504,
 /*  1620 */   130,  131,  161,  162,  163,  169,  170,  171,  178,  179,
 /*  1630 */   180,  181,  160,  316,  504,  327,  139,  153,  195,  343,
 /*  1640 */   360,  435,  510,  517,  518,  532,  534,  548,  504,  161,
 /*  1650 */   162,  163,  504,  504,  504,  504,  504,  326,  348,  352,
 /*  1660 */   356,  504,  323,  319,  324,  118,  120,  504,  504,  504,
 /*  1670 */   504,  561,  122,  124,  504,  130,  131,  161,  162,  163,
 /*  1680 */   169,  170,  171,  178,  179,  180,  181,  192,  160,  504,
 /*  1690 */   150,  504,  504,  504,  504,  129,  504,  132,  150,  161,
 /*  1700 */   162,  163,  160,  129,  504,  132,   90,   91,   96,  135,
 /*  1710 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  1720 */   126,  160,  172,  173,  174,  188,  116,  117,  126,  160,
 /*  1730 */   172,  173,  174,  504,  504,  332,  504,  504,  150,  504,
 /*  1740 */   504,  504,  430,  129,  347,  132,  329,  504,  504,  367,
 /*  1750 */   504,  504,  504,  351,   90,   91,   96,  135,  136,  152,
 /*  1760 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  1770 */   172,  173,  174,  504,  332,  160,  361,  150,  373,  396,
 /*  1780 */   413,  504,  129,  150,  132,  336,  504,  322,  129,  504,
 /*  1790 */   132,  504,  321,   90,   91,   96,  135,  136,  152,  165,
 /*  1800 */   141,  142,  143,  155,  166,  177,  117,  126,  160,  172,
 /*  1810 */   173,  174,  128,  126,  160,  172,  173,  174,  160,  161,
 /*  1820 */   162,  163,  332,  504,  504,  150,  504,  504,  504,  504,
 /*  1830 */   129,  315,  132,  349,  504,  504,  346,  370,  504,  504,
 /*  1840 */   504,   90,   91,   96,  135,  136,  152,  165,  141,  142,
 /*  1850 */   143,  155,  166,  177,  117,  126,  160,  172,  173,  174,
 /*  1860 */   504,  504,  160,  332,  504,  504,  150,  160,  160,  385,
 /*  1870 */   347,  129,  504,  132,  353,  504,  379,  382,  394,  355,
 /*  1880 */   405,  504,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  1890 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  1900 */   174,  160,  366,  376,  504,  150,  408,  504,  160,  160,
 /*  1910 */   129,  160,  132,  402,  422,  363,  504,  504,  504,  504,
 /*  1920 */   504,   90,   91,   96,  135,  136,  152,  165,  141,  142,
 /*  1930 */   143,  155,  166,  177,  117,  126,  160,  172,  173,  174,
 /*  1940 */   399,  366,  504,  504,  150,  160,  161,  162,  163,  129,
 /*  1950 */   347,  132,  504,  504,  369,  445,  504,  457,  504,  359,
 /*  1960 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  1970 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  366,
 /*  1980 */   504,  160,  150,  161,  162,  163,  160,  129,  160,  132,
 /*  1990 */   469,  480,  372,  497,  504,  504,  578,  504,   90,   91,
 /*  2000 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  2010 */   177,  117,  126,  160,  172,  173,  174,  504,  504,  504,
 /*  2020 */   332,  160,  160,  150,  160,  504,  425,  160,  129,  504,
 /*  2030 */   132,  374,  504,  419,  504,  504,  504,  504,  504,   90,
 /*  2040 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  2050 */   166,  177,  117,  126,  160,  172,  173,  174,  504,  366,
 /*  2060 */   416,  504,  150,  460,  504,  504,  504,  129,  504,  132,
 /*  2070 */   454,  504,  378,  504,  504,  504,  504,  504,   90,   91,
 /*  2080 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  2090 */   177,  117,  126,  160,  172,  173,  174,  451,  366,  504,
 /*  2100 */   504,  150,  504,  161,  162,  163,  129,  504,  132,  504,
 /*  2110 */   504,  381,  504,  504,  504,  504,  504,   90,   91,   96,
 /*  2120 */   135,  136,  152,  165,  141,  142,  143,  155,  166,  177,
 /*  2130 */   117,  126,  160,  172,  173,  174,  366,  504,  504,  150,
 /*  2140 */   161,  162,  163,  504,  129,  504,  132,  504,  504,  384,
 /*  2150 */   504,  504,  504,  504,  504,   90,   91,   96,  135,  136,
 /*  2160 */   152,  165,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  2170 */   160,  172,  173,  174,  504,  504,  504,  392,  504,  504,
 /*  2180 */   150,  504,  504,  483,  504,  129,  504,  132,  504,  504,
 /*  2190 */   477,  504,  504,  389,  504,  504,   90,   91,   96,  135,
 /*  2200 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  2210 */   126,  160,  172,  173,  174,  504,  332,  474,  504,  150,
 /*  2220 */   504,  504,  504,  504,  129,  504,  132,  397,  504,  504,
 /*  2230 */   504,  504,  504,  504,  504,   90,   91,   96,  135,  136,
 /*  2240 */   152,  165,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  2250 */   160,  172,  173,  174,  504,  366,  504,  504,  150,  504,
 /*  2260 */   161,  162,  163,  129,  504,  132,  504,  504,  401,  504,
 /*  2270 */   504,  504,  504,  504,   90,   91,   96,  135,  136,  152,
 /*  2280 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  2290 */   172,  173,  174,  366,  504,  504,  150,  504,  504,  504,
 /*  2300 */   504,  129,  504,  132,  504,  504,  404,  504,  504,  504,
 /*  2310 */   504,  504,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  2320 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  2330 */   174,  504,  504,  504,  366,  504,  504,  150,  504,  504,
 /*  2340 */   504,  504,  129,  504,  132,  504,  504,  407,  504,  504,
 /*  2350 */   504,  504,  504,   90,   91,   96,  135,  136,  152,  165,
 /*  2360 */   141,  142,  143,  155,  166,  177,  117,  126,  160,  172,
 /*  2370 */   173,  174,  504,  392,  504,  504,  150,  500,  504,  504,
 /*  2380 */   504,  129,  504,  132,  494,  504,  504,  504,  504,  412,
 /*  2390 */   504,  504,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  2400 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  2410 */   174,  491,  366,  504,  504,  150,  504,  504,  504,  504,
 /*  2420 */   129,  504,  132,  504,  504,  418,  504,  504,  504,  504,
 /*  2430 */   504,   90,   91,   96,  135,  136,  152,  165,  141,  142,
 /*  2440 */   143,  155,  166,  177,  117,  126,  160,  172,  173,  174,
 /*  2450 */   366,  504,  504,  150,  161,  162,  163,  504,  129,  504,
 /*  2460 */   132,  504,  504,  421,  504,  504,  504,  504,  504,   90,
 /*  2470 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  2480 */   166,  177,  117,  126,  160,  172,  173,  174,  504,  504,
 /*  2490 */   504,  366,  504,  504,  150,  504,  504,  504,  504,  129,
 /*  2500 */   504,  132,  504,  504,  424,  504,  504,  504,  504,  504,
 /*  2510 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  2520 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  504,
 /*  2530 */   392,  504,  504,  150,  504,  504,  504,  504,  129,  504,
 /*  2540 */   132,  504,  504,  504,  504,  504,  429,  504,  504,   90,
 /*  2550 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  2560 */   166,  177,  117,  126,  160,  172,  173,  174,  504,  392,
 /*  2570 */   504,  504,  150,  504,  504,  505,  504,  129,  504,  132,
 /*  2580 */   504,  504,  442,  504,  504,  434,  504,  504,   90,   91,
 /*  2590 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  2600 */   177,  117,  126,  160,  172,  173,  174,  441,  504,  436,
 /*  2610 */   150,  448,  471,  488,  504,  129,  504,  132,  504,  504,
 /*  2620 */   504,  438,  504,  504,  504,  504,   90,   91,   96,  135,
 /*  2630 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  2640 */   126,  160,  172,  173,  174,  504,  504,  504,  441,  504,
 /*  2650 */   504,  150,  161,  162,  163,  504,  129,  504,  132,  504,
 /*  2660 */   504,  504,  444,  504,  504,  504,  504,   90,   91,   96,
 /*  2670 */   135,  136,  152,  165,  141,  142,  143,  155,  166,  177,
 /*  2680 */   117,  126,  160,  172,  173,  174,  504,  441,  504,  504,
 /*  2690 */   150,  504,  504,  504,  504,  129,  504,  132,  504,  504,
 /*  2700 */   504,  447,  504,  504,  504,  504,   90,   91,   96,  135,
 /*  2710 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  2720 */   126,  160,  172,  173,  174,  504,  332,  504,  504,  150,
 /*  2730 */   504,  504,  504,  504,  129,  504,  132,  449,  504,  504,
 /*  2740 */   504,  504,  504,  504,  504,   90,   91,   96,  135,  136,
 /*  2750 */   152,  165,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  2760 */   160,  172,  173,  174,  441,  504,  504,  150,  504,  504,
 /*  2770 */   504,  504,  129,  504,  132,  504,  504,  504,  453,  504,
 /*  2780 */   504,  504,  504,   90,   91,   96,  135,  136,  152,  165,
 /*  2790 */   141,  142,  143,  155,  166,  177,  117,  126,  160,  172,
 /*  2800 */   173,  174,  504,  504,  504,  441,  504,  504,  150,  504,
 /*  2810 */   504,  504,  504,  129,  504,  132,  504,  504,  504,  456,
 /*  2820 */   504,  504,  504,  504,   90,   91,   96,  135,  136,  152,
 /*  2830 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  2840 */   172,  173,  174,  504,  441,  504,  504,  150,  504,  504,
 /*  2850 */   504,  504,  129,  504,  132,  504,  504,  504,  459,  504,
 /*  2860 */   504,  504,  504,   90,   91,   96,  135,  136,  152,  165,
 /*  2870 */   141,  142,  143,  155,  166,  177,  117,  126,  160,  172,
 /*  2880 */   173,  174,  504,  467,  504,  504,  150,  504,  504,  504,
 /*  2890 */   504,  129,  504,  132,  504,  504,  504,  504,  504,  504,
 /*  2900 */   504,  464,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  2910 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  2920 */   174,  332,  504,  504,  150,  504,  504,  504,  504,  129,
 /*  2930 */   504,  132,  472,  504,  504,  504,  504,  504,  504,  504,
 /*  2940 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  2950 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  504,
 /*  2960 */   504,  504,  441,  504,  504,  150,  504,  504,  504,  504,
 /*  2970 */   129,  504,  132,  504,  504,  504,  476,  504,  504,  504,
 /*  2980 */   504,   90,   91,   96,  135,  136,  152,  165,  141,  142,
 /*  2990 */   143,  155,  166,  177,  117,  126,  160,  172,  173,  174,
 /*  3000 */   504,  441,  504,  504,  150,  504,  504,  504,  504,  129,
 /*  3010 */   504,  132,  504,  504,  504,  479,  504,  504,  504,  504,
 /*  3020 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  3030 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  504,
 /*  3040 */   441,  504,  504,  150,  504,  504,  504,  504,  129,  504,
 /*  3050 */   132,  504,  504,  504,  482,  504,  504,  504,  504,   90,
 /*  3060 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  3070 */   166,  177,  117,  126,  160,  172,  173,  174,  467,  504,
 /*  3080 */   504,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  3090 */   504,  504,  504,  504,  504,  504,  487,   90,   91,   96,
 /*  3100 */   135,  136,  152,  165,  141,  142,  143,  155,  166,  177,
 /*  3110 */   117,  126,  160,  172,  173,  174,  504,  504,  504,  441,
 /*  3120 */   504,  504,  150,  504,  504,  504,  504,  129,  504,  132,
 /*  3130 */   504,  504,  504,  493,  504,  504,  504,  504,   90,   91,
 /*  3140 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  3150 */   177,  117,  126,  160,  172,  173,  174,  504,  441,  504,
 /*  3160 */   504,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  3170 */   504,  504,  496,  504,  504,  504,  504,   90,   91,   96,
 /*  3180 */   135,  136,  152,  165,  141,  142,  143,  155,  166,  177,
 /*  3190 */   117,  126,  160,  172,  173,  174,  504,  441,  504,  504,
 /*  3200 */   150,  504,  504,  504,  504,  129,  504,  132,  504,  504,
 /*  3210 */   504,  499,  504,  504,  504,  504,   90,   91,   96,  135,
 /*  3220 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  3230 */   126,  160,  172,  173,  174,  467,  504,  504,  150,  504,
 /*  3240 */   504,  504,  504,  129,  504,  132,  504,  504,  504,  504,
 /*  3250 */   504,  504,  504,  504,   90,   91,   96,  135,  136,  152,
 /*  3260 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  3270 */   172,  173,  174,  504,  504,  504,  467,  504,  504,  150,
 /*  3280 */   504,  504,  504,  504,  129,  504,  132,  504,  504,  504,
 /*  3290 */   504,  504,  504,  504,  509,   90,   91,   96,  135,  136,
 /*  3300 */   152,  165,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  3310 */   160,  172,  173,  174,  504,  332,  504,  504,  150,  504,
 /*  3320 */   504,  504,  504,  129,  504,  132,  512,  504,  504,  504,
 /*  3330 */   504,  504,  504,  504,   90,   91,   96,  135,  136,  152,
 /*  3340 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  3350 */   172,  173,  174,  504,  332,  504,  504,  150,  504,  504,
 /*  3360 */   504,  504,  129,  504,  132,  514,  504,  504,  504,  504,
 /*  3370 */   504,  504,  504,   90,   91,   96,  135,  136,  152,  165,
 /*  3380 */   141,  142,  143,  155,  166,  177,  117,  126,  160,  172,
 /*  3390 */   173,  174,  332,  504,  504,  150,  504,  504,  504,  504,
 /*  3400 */   129,  504,  132,  521,  504,  504,  504,  504,  504,  504,
 /*  3410 */   504,   90,   91,   96,  135,  136,  152,  165,  141,  142,
 /*  3420 */   143,  155,  166,  177,  117,  126,  160,  172,  173,  174,
 /*  3430 */   504,  504,  504,  332,  504,  504,  150,  504,  504,  504,
 /*  3440 */   504,  129,  504,  132,  525,  504,  504,  504,  504,  504,
 /*  3450 */   504,  504,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  3460 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  3470 */   174,  504,  332,  504,  504,  150,  504,  504,  504,  504,
 /*  3480 */   129,  504,  132,  537,  504,  504,  504,  504,  504,  504,
 /*  3490 */   504,   90,   91,   96,  135,  136,  152,  165,  141,  142,
 /*  3500 */   143,  155,  166,  177,  117,  126,  160,  172,  173,  174,
 /*  3510 */   504,  332,  504,  504,  150,  504,  504,  504,  504,  129,
 /*  3520 */   504,  132,  541,  504,  504,  504,  504,  504,  504,  504,
 /*  3530 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  3540 */   155,  166,  177,  117,  126,  160,  172,  173,  174,   77,
 /*  3550 */   504,  504,  150,  504,  504,  504,  504,  129,  504,  132,
 /*  3560 */   504,  504,  504,  504,  504,  504,  504,  504,   90,   91,
 /*  3570 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  3580 */   177,  117,  126,  160,  172,  173,  174,  504,  504,  504,
 /*  3590 */    88,  504,  504,  150,  504,  504,  504,  504,  129,  504,
 /*  3600 */   132,  504,  504,  504,  504,  504,  504,  504,  504,   90,
 /*  3610 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  3620 */   166,  177,  117,  126,  160,  172,  173,  174,  504,  203,
 /*  3630 */   504,  504,  150,  504,  504,  504,  504,  129,  504,  132,
 /*  3640 */   504,  504,  504,  504,  504,  504,  504,  504,   90,   91,
 /*  3650 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  3660 */   177,  117,  126,  160,  172,  173,  174,  504,  197,  504,
 /*  3670 */   504,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  3680 */   504,  504,  504,  504,  504,  504,  504,   90,   91,   96,
 /*  3690 */   135,  136,  152,  165,  141,  142,  143,  155,  166,  177,
 /*  3700 */   117,  126,  160,  172,  173,  174,  193,  504,  504,  150,
 /*  3710 */   504,  504,  504,  504,  129,  504,  132,  504,  504,  504,
 /*  3720 */   504,  504,  504,  504,  504,   90,   91,   96,  135,  136,
 /*  3730 */   152,  165,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  3740 */   160,  172,  173,  174,  504,  504,  504,  183,  504,  504,
 /*  3750 */   150,  504,  504,  504,  504,  129,  504,  132,  504,  504,
 /*  3760 */   504,  504,  504,  504,  504,  504,   90,   91,   96,  135,
 /*  3770 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  3780 */   126,  160,  172,  173,  174,  504,  186,  504,  504,  150,
 /*  3790 */   504,  504,  504,  504,  129,  504,  132,  504,  504,  504,
 /*  3800 */   504,  504,  504,  504,  504,   90,   91,   96,  135,  136,
 /*  3810 */   152,  165,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  3820 */   160,  172,  173,  174,  504,  191,  504,  504,  150,  504,
 /*  3830 */   504,  504,  504,  129,  504,  132,  504,  504,  504,  504,
 /*  3840 */   504,  504,  504,  504,   90,   91,   96,  135,  136,  152,
 /*  3850 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  3860 */   172,  173,  174,  223,  504,  504,  150,  504,  504,  504,
 /*  3870 */   504,  129,  504,  132,  504,  504,  504,  504,  504,  504,
 /*  3880 */   504,  504,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  3890 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  3900 */   174,  504,  504,  504,  229,  504,  504,  150,  504,  504,
 /*  3910 */   504,  504,  129,  504,  132,  504,  504,  504,  504,  504,
 /*  3920 */   504,  504,  504,   90,   91,   96,  135,  136,  152,  165,
 /*  3930 */   141,  142,  143,  155,  166,  177,  117,  126,  160,  172,
 /*  3940 */   173,  174,  504,  235,  504,  504,  150,  504,  504,  504,
 /*  3950 */   504,  129,  504,  132,  504,  504,  504,  504,  504,  504,
 /*  3960 */   504,  504,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  3970 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  3980 */   174,  504,  238,  504,  504,  150,  504,  504,  504,  504,
 /*  3990 */   129,  504,  132,  504,  504,  504,  504,  504,  504,  504,
 /*  4000 */   504,   90,   91,   96,  135,  136,  152,  165,  141,  142,
 /*  4010 */   143,  155,  166,  177,  117,  126,  160,  172,  173,  174,
 /*  4020 */   240,  504,  504,  150,  504,  504,  504,  504,  129,  504,
 /*  4030 */   132,  504,  504,  504,  504,  504,  504,  504,  504,   90,
 /*  4040 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  4050 */   166,  177,  117,  126,  160,  172,  173,  174,  504,  504,
 /*  4060 */   504,  242,  504,  504,  150,  504,  504,  504,  504,  129,
 /*  4070 */   504,  132,  504,  504,  504,  504,  504,  504,  504,  504,
 /*  4080 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  4090 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  504,
 /*  4100 */   244,  504,  504,  150,  504,  504,  504,  504,  129,  504,
 /*  4110 */   132,  504,  504,  504,  504,  504,  504,  504,  504,   90,
 /*  4120 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  4130 */   166,  177,  117,  126,  160,  172,  173,  174,  504,  246,
 /*  4140 */   504,  504,  150,  504,  504,  504,  504,  129,  504,  132,
 /*  4150 */   504,  504,  504,  504,  504,  504,  504,  504,   90,   91,
 /*  4160 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  4170 */   177,  117,  126,  160,  172,  173,  174,  248,  504,  504,
 /*  4180 */   150,  504,  504,  504,  504,  129,  504,  132,  504,  504,
 /*  4190 */   504,  504,  504,  504,  504,  504,   90,   91,   96,  135,
 /*  4200 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  4210 */   126,  160,  172,  173,  174,  504,  504,  504,  250,  504,
 /*  4220 */   504,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  4230 */   504,  504,  504,  504,  504,  504,  504,   90,   91,   96,
 /*  4240 */   135,  136,  152,  165,  141,  142,  143,  155,  166,  177,
 /*  4250 */   117,  126,  160,  172,  173,  174,  504,  252,  504,  504,
 /*  4260 */   150,  504,  504,  504,  504,  129,  504,  132,  504,  504,
 /*  4270 */   504,  504,  504,  504,  504,  504,   90,   91,   96,  135,
 /*  4280 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  4290 */   126,  160,  172,  173,  174,  504,  254,  504,  504,  150,
 /*  4300 */   504,  504,  504,  504,  129,  504,  132,  504,  504,  504,
 /*  4310 */   504,  504,  504,  504,  504,   90,   91,   96,  135,  136,
 /*  4320 */   152,  165,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  4330 */   160,  172,  173,  174,  256,  504,  504,  150,  504,  504,
 /*  4340 */   504,  504,  129,  504,  132,  504,  504,  504,  504,  504,
 /*  4350 */   504,  504,  504,   90,   91,   96,  135,  136,  152,  165,
 /*  4360 */   141,  142,  143,  155,  166,  177,  117,  126,  160,  172,
 /*  4370 */   173,  174,  504,  504,  504,  258,  504,  504,  150,  504,
 /*  4380 */   504,  504,  504,  129,  504,  132,  504,  504,  504,  504,
 /*  4390 */   504,  504,  504,  504,   90,   91,   96,  135,  136,  152,
 /*  4400 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  4410 */   172,  173,  174,  504,  260,  504,  504,  150,  504,  504,
 /*  4420 */   504,  504,  129,  504,  132,  504,  504,  504,  504,  504,
 /*  4430 */   504,  504,  504,   90,   91,   96,  135,  136,  152,  165,
 /*  4440 */   141,  142,  143,  155,  166,  177,  117,  126,  160,  172,
 /*  4450 */   173,  174,  504,  262,  504,  504,  150,  504,  504,  504,
 /*  4460 */   504,  129,  504,  132,  504,  504,  504,  504,  504,  504,
 /*  4470 */   504,  504,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  4480 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  4490 */   174,  264,  504,  504,  150,  504,  504,  504,  504,  129,
 /*  4500 */   504,  132,  504,  504,  504,  504,  504,  504,  504,  504,
 /*  4510 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  4520 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  504,
 /*  4530 */   504,  504,  266,  504,  504,  150,  504,  504,  504,  504,
 /*  4540 */   129,  504,  132,  504,  504,  504,  504,  504,  504,  504,
 /*  4550 */   504,   90,   91,   96,  135,  136,  152,  165,  141,  142,
 /*  4560 */   143,  155,  166,  177,  117,  126,  160,  172,  173,  174,
 /*  4570 */   504,  268,  504,  504,  150,  504,  504,  504,  504,  129,
 /*  4580 */   504,  132,  504,  504,  504,  504,  504,  504,  504,  504,
 /*  4590 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  4600 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  504,
 /*  4610 */   270,  504,  504,  150,  504,  504,  504,  504,  129,  504,
 /*  4620 */   132,  504,  504,  504,  504,  504,  504,  504,  504,   90,
 /*  4630 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  4640 */   166,  177,  117,  126,  160,  172,  173,  174,  273,  504,
 /*  4650 */   504,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  4660 */   504,  504,  504,  504,  504,  504,  504,   90,   91,   96,
 /*  4670 */   135,  136,  152,  165,  141,  142,  143,  155,  166,  177,
 /*  4680 */   117,  126,  160,  172,  173,  174,  504,  504,  504,  276,
 /*  4690 */   504,  504,  150,  504,  504,  504,  504,  129,  504,  132,
 /*  4700 */   504,  504,  504,  504,  504,  504,  504,  504,   90,   91,
 /*  4710 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  4720 */   177,  117,  126,  160,  172,  173,  174,  504,  279,  504,
 /*  4730 */   504,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  4740 */   504,  504,  504,  504,  504,  504,  504,   90,   91,   96,
 /*  4750 */   135,  136,  152,  165,  141,  142,  143,  155,  166,  177,
 /*  4760 */   117,  126,  160,  172,  173,  174,  504,  282,  504,  504,
 /*  4770 */   150,  504,  504,  504,  504,  129,  504,  132,  504,  504,
 /*  4780 */   504,  504,  504,  504,  504,  504,   90,   91,   96,  135,
 /*  4790 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  4800 */   126,  160,  172,  173,  174,  284,  504,  504,  150,  504,
 /*  4810 */   504,  504,  504,  129,  504,  132,  504,  504,  504,  504,
 /*  4820 */   504,  504,  504,  504,   90,   91,   96,  135,  136,  152,
 /*  4830 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  4840 */   172,  173,  174,  504,  504,  504,  286,  504,  504,  150,
 /*  4850 */   504,  504,  504,  504,  129,  504,  132,  504,  504,  504,
 /*  4860 */   504,  504,  504,  504,  504,   90,   91,   96,  135,  136,
 /*  4870 */   152,  165,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  4880 */   160,  172,  173,  174,  504,  288,  504,  504,  150,  504,
 /*  4890 */   504,  504,  504,  129,  504,  132,  504,  504,  504,  504,
 /*  4900 */   504,  504,  504,  504,   90,   91,   96,  135,  136,  152,
 /*  4910 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  4920 */   172,  173,  174,  504,  290,  504,  504,  150,  504,  504,
 /*  4930 */   504,  504,  129,  504,  132,  504,  504,  504,  504,  504,
 /*  4940 */   504,  504,  504,   90,   91,   96,  135,  136,  152,  165,
 /*  4950 */   141,  142,  143,  155,  166,  177,  117,  126,  160,  172,
 /*  4960 */   173,  174,  292,  504,  504,  150,  504,  504,  504,  504,
 /*  4970 */   129,  504,  132,  504,  504,  504,  504,  504,  504,  504,
 /*  4980 */   504,   90,   91,   96,  135,  136,  152,  165,  141,  142,
 /*  4990 */   143,  155,  166,  177,  117,  126,  160,  172,  173,  174,
 /*  5000 */   504,  504,  504,  294,  504,  504,  150,  504,  504,  504,
 /*  5010 */   504,  129,  504,  132,  504,  504,  504,  504,  504,  504,
 /*  5020 */   504,  504,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  5030 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  5040 */   174,  504,  296,  504,  504,  150,  504,  504,  504,  504,
 /*  5050 */   129,  504,  132,  504,  504,  504,  504,  504,  504,  504,
 /*  5060 */   504,   90,   91,   96,  135,  136,  152,  165,  141,  142,
 /*  5070 */   143,  155,  166,  177,  117,  126,  160,  172,  173,  174,
 /*  5080 */   504,  298,  504,  504,  150,  504,  504,  504,  504,  129,
 /*  5090 */   504,  132,  504,  504,  504,  504,  504,  504,  504,  504,
 /*  5100 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  5110 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  300,
 /*  5120 */   504,  504,  150,  504,  504,  504,  504,  129,  504,  132,
 /*  5130 */   504,  504,  504,  504,  504,  504,  504,  504,   90,   91,
 /*  5140 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  5150 */   177,  117,  126,  160,  172,  173,  174,  504,  504,  504,
 /*  5160 */   302,  504,  504,  150,  504,  504,  504,  504,  129,  504,
 /*  5170 */   132,  504,  504,  504,  504,  504,  504,  504,  504,   90,
 /*  5180 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  5190 */   166,  177,  117,  126,  160,  172,  173,  174,  504,  304,
 /*  5200 */   504,  504,  150,  504,  504,  504,  504,  129,  504,  132,
 /*  5210 */   504,  504,  504,  504,  504,  504,  504,  504,   90,   91,
 /*  5220 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  5230 */   177,  117,  126,  160,  172,  173,  174,  504,  306,  504,
 /*  5240 */   504,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  5250 */   504,  504,  504,  504,  504,  504,  504,   90,   91,   96,
 /*  5260 */   135,  136,  152,  165,  141,  142,  143,  155,  166,  177,
 /*  5270 */   117,  126,  160,  172,  173,  174,  308,  504,  504,  150,
 /*  5280 */   504,  504,  504,  504,  129,  504,  132,  504,  504,  504,
 /*  5290 */   504,  504,  504,  504,  504,   90,   91,   96,  135,  136,
 /*  5300 */   152,  165,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  5310 */   160,  172,  173,  174,  504,  504,  504,  310,  504,  504,
 /*  5320 */   150,  504,  504,  504,  504,  129,  504,  132,  504,  504,
 /*  5330 */   504,  504,  504,  504,  504,  504,   90,   91,   96,  135,
 /*  5340 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  5350 */   126,  160,  172,  173,  174,  504,  312,  504,  504,  150,
 /*  5360 */   504,  504,  504,  504,  129,  504,  132,  504,  504,  504,
 /*  5370 */   504,  504,  504,  504,  504,   90,   91,   96,  135,  136,
 /*  5380 */   152,  165,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  5390 */   160,  172,  173,  174,  504,  340,  504,  504,  150,  504,
 /*  5400 */   504,  504,  504,  129,  504,  132,  504,  504,  504,  504,
 /*  5410 */   504,  504,  504,  504,   90,   91,   96,  135,  136,  152,
 /*  5420 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  5430 */   172,  173,  174,  357,  504,  504,  150,  504,  504,  504,
 /*  5440 */   504,  129,  504,  132,  504,  504,  504,  504,  504,  504,
 /*  5450 */   504,  504,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  5460 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  5470 */   174,  504,  504,  504,  365,  504,  504,  150,  504,  504,
 /*  5480 */   504,  504,  129,  504,  132,  504,  504,  504,  504,  504,
 /*  5490 */   504,  504,  504,   90,   91,   96,  135,  136,  152,  165,
 /*  5500 */   141,  142,  143,  155,  166,  177,  117,  126,  160,  172,
 /*  5510 */   173,  174,  504,  391,  504,  504,  150,  504,  504,  504,
 /*  5520 */   504,  129,  504,  132,  504,  504,  504,  504,  504,  504,
 /*  5530 */   504,  504,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  5540 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  5550 */   174,  504,  414,  504,  504,  150,  504,  504,  504,  504,
 /*  5560 */   129,  504,  132,  504,  504,  504,  504,  504,  504,  504,
 /*  5570 */   504,   90,   91,   96,  135,  136,  152,  165,  141,  142,
 /*  5580 */   143,  155,  166,  177,  117,  126,  160,  172,  173,  174,
 /*  5590 */   440,  504,  504,  150,  504,  504,  504,  504,  129,  504,
 /*  5600 */   132,  504,  504,  504,  504,  504,  504,  504,  504,   90,
 /*  5610 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  5620 */   166,  177,  117,  126,  160,  172,  173,  174,  504,  504,
 /*  5630 */   504,  466,  504,  504,  150,  504,  504,  504,  504,  129,
 /*  5640 */   504,  132,  504,  504,  504,  504,  504,  504,  504,  504,
 /*  5650 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  5660 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  504,
 /*  5670 */   489,  504,  504,  150,  504,  504,  504,  504,  129,  504,
 /*  5680 */   132,  504,  504,  504,  504,  504,  504,  504,  504,   90,
 /*  5690 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  5700 */   166,  177,  117,  126,  160,  172,  173,  174,  504,  516,
 /*  5710 */   504,  504,  150,  504,  504,  504,  504,  129,  504,  132,
 /*  5720 */   504,  504,  504,  504,  504,  504,  504,  504,   90,   91,
 /*  5730 */    96,  135,  136,  152,  165,  141,  142,  143,  155,  166,
 /*  5740 */   177,  117,  126,  160,  172,  173,  174,  519,  504,  504,
 /*  5750 */   150,  504,  504,  504,  504,  129,  504,  132,  504,  504,
 /*  5760 */   504,  504,  504,  504,  504,  504,   90,   91,   96,  135,
 /*  5770 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  5780 */   126,  160,  172,  173,  174,  504,  504,  504,  523,  504,
 /*  5790 */   504,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  5800 */   504,  504,  504,  504,  504,  504,  504,   90,   91,   96,
 /*  5810 */   135,  136,  152,  165,  141,  142,  143,  155,  166,  177,
 /*  5820 */   117,  126,  160,  172,  173,  174,  504,  527,  504,  504,
 /*  5830 */   150,  504,  504,  504,  504,  129,  504,  132,  504,  504,
 /*  5840 */   504,  504,  504,  504,  504,  504,   90,   91,   96,  135,
 /*  5850 */   136,  152,  165,  141,  142,  143,  155,  166,  177,  117,
 /*  5860 */   126,  160,  172,  173,  174,  504,  529,  504,  504,  150,
 /*  5870 */   504,  504,  504,  504,  129,  504,  132,  504,  504,  504,
 /*  5880 */   504,  504,  504,  504,  504,   90,   91,   96,  135,  136,
 /*  5890 */   152,  165,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  5900 */   160,  172,  173,  174,  531,  504,  504,  150,  504,  504,
 /*  5910 */   504,  504,  129,  504,  132,  504,  504,  504,  504,  504,
 /*  5920 */   504,  504,  504,   90,   91,   96,  135,  136,  152,  165,
 /*  5930 */   141,  142,  143,  155,  166,  177,  117,  126,  160,  172,
 /*  5940 */   173,  174,  504,  504,  504,  533,  504,  504,  150,  504,
 /*  5950 */   504,  504,  504,  129,  504,  132,  504,  504,  504,  504,
 /*  5960 */   504,  504,  504,  504,   90,   91,   96,  135,  136,  152,
 /*  5970 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  5980 */   172,  173,  174,  504,  535,  504,  504,  150,  504,  504,
 /*  5990 */   504,  504,  129,  504,  132,  504,  504,  504,  504,  504,
 /*  6000 */   504,  504,  504,   90,   91,   96,  135,  136,  152,  165,
 /*  6010 */   141,  142,  143,  155,  166,  177,  117,  126,  160,  172,
 /*  6020 */   173,  174,  504,  539,  504,  504,  150,  504,  504,  504,
 /*  6030 */   504,  129,  504,  132,  504,  504,  504,  504,  504,  504,
 /*  6040 */   504,  504,   90,   91,   96,  135,  136,  152,  165,  141,
 /*  6050 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  6060 */   174,  543,  504,  504,  150,  504,  504,  504,  504,  129,
 /*  6070 */   504,  132,  504,  504,  504,  504,  504,  504,  504,  504,
 /*  6080 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  6090 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  504,
 /*  6100 */   504,  504,  545,  504,  504,  150,  504,  504,  504,  504,
 /*  6110 */   129,  504,  132,  504,  504,  504,  504,  504,  504,  504,
 /*  6120 */   504,   90,   91,   96,  135,  136,  152,  165,  141,  142,
 /*  6130 */   143,  155,  166,  177,  117,  126,  160,  172,  173,  174,
 /*  6140 */   504,  547,  504,  504,  150,  504,  504,  504,  504,  129,
 /*  6150 */   504,  132,  504,  504,  504,  504,  504,  504,  504,  504,
 /*  6160 */    90,   91,   96,  135,  136,  152,  165,  141,  142,  143,
 /*  6170 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  504,
 /*  6180 */   549,  504,  504,  150,  504,  504,  504,  504,  129,  504,
 /*  6190 */   132,  504,  504,  504,  504,  504,  504,  504,  504,   90,
 /*  6200 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  6210 */   166,  177,  117,  126,  160,  172,  173,  174,  150,  504,
 /*  6220 */   504,  504,  504,  129,  504,  132,  504,  504,  504,  504,
 /*  6230 */   504,  504,  504,  504,   93,   91,   96,  135,  136,  152,
 /*  6240 */   165,  141,  142,  143,  155,  166,  177,  117,  126,  160,
 /*  6250 */   172,  173,  174,  150,  185,  504,  504,  504,  129,  504,
 /*  6260 */   132,  504,  175,  504,  504,  504,  504,  504,  133,   95,
 /*  6270 */    91,   96,  135,  136,  152,  165,  141,  142,  143,  155,
 /*  6280 */   166,  177,  117,  126,  160,  172,  173,  174,  185,  504,
 /*  6290 */   520,  524,  528,  504,  280,  504,  175,  504,  504,  237,
 /*  6300 */   504,  504,  133,  239,  241,  243,  245,  247,  249,  251,
 /*  6310 */   253,  255,  257,  259,  261,  263,  265,  267,  269,  118,
 /*  6320 */   120,  504,  504,  504,  504,  504,  122,  124,  470,  130,
 /*  6330 */   131,  161,  162,  163,  169,  170,  171,  178,  179,  180,
 /*  6340 */   181,  185,  484,  504,  504,  504,  504,  504,  504,  175,
 /*  6350 */   395,  504,  504,  118,  120,  133,  504,  504,  504,  160,
 /*  6360 */   122,  124,  386,  130,  131,  161,  162,  163,  169,  170,
 /*  6370 */   171,  178,  179,  180,  181,  504,  504,  536,  540,  544,
 /*  6380 */   504,  160,  504,  504,  504,  504,  504,  185,  504,  504,
 /*  6390 */   504,  504,  504,  504,  504,  175,  504,  395,  395,  395,
 /*  6400 */   504,  133,  504,  504,  504,  504,  118,  120,  504,  409,
 /*  6410 */   426,  431,  504,  122,  124,  504,  130,  131,  161,  162,
 /*  6420 */   163,  169,  170,  171,  178,  179,  180,  181,  160,  160,
 /*  6430 */   160,  504,  504,  504,  504,  333,  334,  504,  504,  504,
 /*  6440 */   504,  504,  504,  504,  470,  504,  504,  470,  504,  504,
 /*  6450 */   504,  504,  118,  120,  504,  504,  504,  504,  501,  122,
 /*  6460 */   124,  506,  130,  131,  161,  162,  163,  169,  170,  171,
 /*  6470 */   178,  179,  180,  181,  185,  160,  504,  504,  160,  150,
 /*  6480 */   504,  504,  175,  504,  129,  504,  132,  504,  133,  150,
 /*  6490 */   504,  504,  504,  504,  129,  504,  132,  206,  135,  136,
 /*  6500 */   152,  165,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  6510 */   160,  172,  173,  174,  110,  155,  166,  177,  117,  126,
 /*  6520 */   160,  172,  173,  174,  504,  504,  504,  504,  504,  504,
 /*  6530 */   504,  504,  504,  504,  504,  325,  318,  504,  504,  118,
 /*  6540 */   120,  504,  504,  133,  504,  504,  122,  124,  504,  130,
 /*  6550 */   131,  161,  162,  163,  169,  170,  171,  178,  179,  180,
 /*  6560 */   181,  150,  504,  317,  185,  504,  129,  504,  132,  504,
 /*  6570 */   504,  504,  175,  504,  504,  504,  504,  504,  133,  504,
 /*  6580 */    98,  136,  152,  165,  141,  142,  143,  155,  166,  177,
 /*  6590 */   117,  126,  160,  172,  173,  174,  150,  504,  504,  504,
 /*  6600 */   504,  129,  504,  132,  130,  131,  161,  162,  163,  504,
 /*  6610 */   504,  504,  504,  504,  504,  504,  100,  152,  165,  141,
 /*  6620 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  6630 */   174,  150,  504,  504,  504,  504,  129,  504,  132,  130,
 /*  6640 */   131,  161,  162,  163,  169,  170,  171,  178,  179,  180,
 /*  6650 */   181,  504,  102,  165,  141,  142,  143,  155,  166,  177,
 /*  6660 */   117,  126,  160,  172,  173,  174,  150,  504,  504,  504,
 /*  6670 */   504,  129,  504,  132,  504,  504,  504,  504,  504,  504,
 /*  6680 */   504,  504,  504,  504,  504,  504,  504,  138,  165,  141,
 /*  6690 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  6700 */   174,  504,  504,  504,  504,  504,  150,  504,  504,  504,
 /*  6710 */   504,  129,  504,  132,  504,  504,  504,  504,  504,  504,
 /*  6720 */   504,  504,  504,  504,  504,  504,  504,  200,  165,  141,
 /*  6730 */   142,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  6740 */   174,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  6750 */   504,  504,  504,  504,  150,  504,  504,  504,  504,  129,
 /*  6760 */   504,  132,  202,  165,  141,  142,  143,  155,  166,  177,
 /*  6770 */   117,  126,  160,  172,  173,  174,  104,  141,  142,  143,
 /*  6780 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  150,
 /*  6790 */   504,  504,  504,  504,  129,  504,  132,  504,  504,  504,
 /*  6800 */   504,  504,  504,  504,  504,  504,  504,  504,  504,  504,
 /*  6810 */   504,  140,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  6820 */   160,  172,  173,  174,  150,  504,  504,  504,  504,  129,
 /*  6830 */   504,  132,  504,  504,  504,  504,  504,  504,  504,  504,
 /*  6840 */   504,  504,  504,  504,  504,  504,  154,  141,  142,  143,
 /*  6850 */   155,  166,  177,  117,  126,  160,  172,  173,  174,  150,
 /*  6860 */   504,  504,  504,  504,  129,  504,  132,  504,  504,  504,
 /*  6870 */   504,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  6880 */   504,  196,  141,  142,  143,  155,  166,  177,  117,  126,
 /*  6890 */   160,  172,  173,  174,  106,  142,  143,  155,  166,  177,
 /*  6900 */   117,  126,  160,  172,  173,  174,  150,  504,  504,  504,
 /*  6910 */   504,  129,  504,  132,  504,  504,  504,  504,  504,  150,
 /*  6920 */   504,  504,  504,  504,  129,  504,  132,  504,  504,  504,
 /*  6930 */   108,  143,  155,  166,  177,  117,  126,  160,  172,  173,
 /*  6940 */   174,  504,  504,  504,  504,  112,  166,  177,  117,  126,
 /*  6950 */   160,  172,  173,  174,  150,  504,  504,  504,  504,  129,
 /*  6960 */   504,  132,  504,  504,  504,  504,  504,  504,  504,  504,
 /*  6970 */   504,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  6980 */   145,  166,  177,  117,  126,  160,  172,  173,  174,  150,
 /*  6990 */   504,  504,  504,  504,  129,  504,  132,  504,  114,  177,
 /*  7000 */   117,  126,  160,  172,  173,  174,  150,  504,  504,  504,
 /*  7010 */   504,  129,  504,  132,  504,  504,  147,  177,  117,  126,
 /*  7020 */   160,  172,  173,  174,  150,  504,  504,  504,  504,  129,
 /*  7030 */   504,  132,  504,  157,  177,  117,  126,  160,  172,  173,
 /*  7040 */   174,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  7050 */   504,  504,  119,  117,  126,  160,  172,  173,  174,  150,
 /*  7060 */   504,  504,  504,  504,  129,  504,  132,  504,  504,  121,
 /*  7070 */   117,  126,  160,  172,  173,  174,  504,  504,  504,  150,
 /*  7080 */   504,  504,  504,  504,  129,  504,  132,  123,  117,  126,
 /*  7090 */   160,  172,  173,  174,  150,  504,  504,  504,  504,  129,
 /*  7100 */   504,  132,  504,  504,  504,  504,  504,  125,  117,  126,
 /*  7110 */   160,  172,  173,  174,  150,  504,  504,  504,  504,  129,
 /*  7120 */   504,  132,  149,  117,  126,  160,  172,  173,  174,  504,
 /*  7130 */   504,  150,  504,  504,  504,  504,  129,  504,  132,  504,
 /*  7140 */   504,  504,  159,  117,  126,  160,  172,  173,  174,  504,
 /*  7150 */   504,  504,  504,  504,  504,  504,  504,  504,  504,  168,
 /*  7160 */   117,  126,  160,  172,  173,  174,
};
static YYCODETYPE yy_lookahead[] = {
 /*     0 */    97,   98,   99,  100,  101,  102,  103,  104,  105,  106,
 /*    10 */   107,  108,  109,  110,  111,  112,  113,  114,  115,  116,
 /*    20 */   117,  118,  119,  120,  121,  122,  123,  124,  125,  126,
 /*    30 */   127,  128,  129,  130,  131,  132,  133,  134,  135,  136,
 /*    40 */   137,  138,  139,  140,  141,  142,  143,  144,  145,  146,
 /*    50 */   147,  148,  149,  150,  151,  152,  153,  154,  155,  156,
 /*    60 */     9,    0,  159,  160,   59,  162,   61,  164,   17,  166,
 /*    70 */    84,   85,   86,   87,   88,   74,   75,   76,  175,  176,
 /*    80 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*    90 */   187,  188,  189,  190,  191,  192,  193,   98,   99,  100,
 /*   100 */   101,  102,  103,  104,  105,  106,  107,  108,  109,  110,
 /*   110 */   111,  112,  113,  114,  115,  116,  117,  118,  119,  120,
 /*   120 */   121,  122,  123,  124,  125,  126,  127,  128,  129,  130,
 /*   130 */   131,  132,  133,  134,  135,  136,  137,  138,  139,  140,
 /*   140 */   141,  142,  143,  144,  145,  146,  147,  148,  149,  150,
 /*   150 */   151,  152,  153,  154,  155,  156,    1,   10,  159,  160,
 /*   160 */    13,  162,    5,  164,    6,  166,   64,   65,   66,   67,
 /*   170 */    77,   78,   79,   80,  175,  176,  177,  178,  179,  180,
 /*   180 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*   190 */   191,  192,  193,   98,   99,  100,  101,  102,  103,  104,
 /*   200 */   105,  106,  107,  108,  109,  110,  111,  112,  113,  114,
 /*   210 */   115,  116,  117,  118,  119,  120,  121,  122,  123,  124,
 /*   220 */   125,  126,  127,  128,  129,  130,  131,  132,  133,  134,
 /*   230 */   135,  136,  137,  138,  139,  140,  141,  142,  143,  144,
 /*   240 */   145,  146,  147,  148,  149,  150,  151,  152,  153,  154,
 /*   250 */   155,  156,   19,   20,  159,  160,    5,  162,  161,  164,
 /*   260 */   163,  166,   86,   87,   88,   45,   46,   47,    6,    5,
 /*   270 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*   280 */   185,  186,  187,  188,  189,  190,  191,  192,  193,   98,
 /*   290 */    99,  100,  101,  102,  103,  104,  105,  106,  107,  108,
 /*   300 */   109,  110,  111,  112,  113,  114,  115,  116,  117,  118,
 /*   310 */   119,  120,  121,  122,  123,  124,  125,  126,  127,  128,
 /*   320 */   129,  130,  131,  132,  133,  134,  135,  136,  137,  138,
 /*   330 */   139,  140,  141,  142,  143,  144,  145,  146,  147,  148,
 /*   340 */   149,  150,  151,  152,  153,  154,  155,  156,   10,    6,
 /*   350 */   159,  160,   13,  162,   13,  164,    9,  166,   45,   46,
 /*   360 */    47,   10,   24,   24,   13,   24,  175,  176,  177,  178,
 /*   370 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*   380 */   189,  190,  191,  192,  193,   98,   99,  100,  101,  102,
 /*   390 */   103,  104,  105,  106,  107,  108,  109,  110,  111,  112,
 /*   400 */   113,  114,  115,  116,  117,  118,  119,  120,  121,  122,
 /*   410 */   123,  124,  125,  126,  127,  128,  129,  130,  131,  132,
 /*   420 */   133,  134,  135,  136,  137,  138,  139,  140,  141,  142,
 /*   430 */   143,  144,  145,  146,  147,  148,  149,  150,  151,  152,
 /*   440 */   153,  154,  155,  156,   13,   10,  159,  160,   13,  162,
 /*   450 */    13,  164,   10,  166,   10,   24,   10,   13,    5,   13,
 /*   460 */     6,   24,  175,  176,  177,  178,  179,  180,  181,  182,
 /*   470 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*   480 */   193,   98,   99,  100,  101,  102,  103,  104,  105,  106,
 /*   490 */   107,  108,  109,  110,  111,  112,  113,  114,  115,  116,
 /*   500 */   117,  118,  119,  120,  121,  122,  123,  124,  125,  126,
 /*   510 */   127,  128,  129,  130,  131,  132,  133,  134,  135,  136,
 /*   520 */   137,  138,  139,  140,  141,  142,  143,  144,  145,  146,
 /*   530 */   147,  148,  149,  150,  151,  152,  153,  154,  155,  156,
 /*   540 */    13,   13,  159,  160,   13,  162,    5,  164,    6,  166,
 /*   550 */    10,   24,   24,   13,   10,   24,    8,   13,  175,  176,
 /*   560 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*   570 */   187,  188,  189,  190,  191,  192,  193,   98,   99,  100,
 /*   580 */   101,  102,  103,  104,  105,  106,  107,  108,  109,  110,
 /*   590 */   111,  112,  113,  114,  115,  116,  117,  118,  119,  120,
 /*   600 */   121,  122,  123,  124,  125,  126,  127,  128,  129,  130,
 /*   610 */   131,  132,  133,  134,  135,  136,  137,  138,  139,  140,
 /*   620 */   141,  142,  143,  144,  145,  146,  147,  148,  149,  150,
 /*   630 */   151,  152,  153,  154,  155,  156,   13,   10,  159,  160,
 /*   640 */    13,  162,    9,  164,   10,  166,   10,   24,   10,   13,
 /*   650 */    60,   13,   63,   62,  175,  176,  177,  178,  179,  180,
 /*   660 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*   670 */   191,  192,  193,   98,   99,  100,  101,  102,  103,  104,
 /*   680 */   105,  106,  107,  108,  109,  110,  111,  112,  113,  114,
 /*   690 */   115,  116,  117,  118,  119,  120,  121,  122,  123,  124,
 /*   700 */   125,  126,  127,  128,  129,  130,  131,  132,  133,  134,
 /*   710 */   135,  136,  137,  138,  139,  140,  141,  142,  143,  144,
 /*   720 */   145,  146,  147,  148,  149,  150,  151,  152,  153,  154,
 /*   730 */   155,  156,   10,   47,  159,  160,   71,  162,   72,  164,
 /*   740 */    83,  166,   24,   24,   24,    9,   24,   24,   10,   25,
 /*   750 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*   760 */   185,  186,  187,  188,  189,  190,  191,  192,  193,   98,
 /*   770 */    99,  100,  101,  102,  103,  104,  105,  106,  107,  108,
 /*   780 */   109,  110,  111,  112,  113,  114,  115,  116,  117,  118,
 /*   790 */   119,  120,  121,  122,  123,  124,  125,  126,  127,  128,
 /*   800 */   129,  130,  131,  132,  133,  134,  135,  136,  137,  138,
 /*   810 */   139,  140,  141,  142,  143,  144,  145,  146,  147,  148,
 /*   820 */   149,  150,  151,  152,  153,  154,  155,  156,   25,   25,
 /*   830 */   159,  160,   25,  162,    9,  164,  157,  166,   17,   13,
 /*   840 */   157,   14,   10,    5,    9,    6,  175,  176,  177,  178,
 /*   850 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*   860 */   189,  190,  191,  192,  193,   98,   99,  100,  101,  102,
 /*   870 */   103,  104,  105,  106,  107,  108,  109,  110,  111,  112,
 /*   880 */   113,  114,  115,  116,  117,  118,  119,  120,  121,  122,
 /*   890 */   123,  124,  125,  126,  127,  128,  129,  130,  131,  132,
 /*   900 */   133,  134,  135,  136,  137,  138,  139,  140,  141,  142,
 /*   910 */   143,  144,  145,  146,  147,  148,  149,  150,  151,  152,
 /*   920 */   153,  154,  155,  156,   10,    5,  159,  160,    9,  162,
 /*   930 */     5,  164,   10,  166,    6,   24,   13,   25,   22,   22,
 /*   940 */    13,   15,  175,  176,  177,  178,  179,  180,  181,  182,
 /*   950 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*   960 */   193,   98,   99,  100,  101,  102,  103,  104,  105,  106,
 /*   970 */   107,  108,  109,  110,  111,  112,  113,  114,  115,  116,
 /*   980 */   117,  118,  119,  120,  121,  122,  123,  124,  125,  126,
 /*   990 */   127,  128,  129,  130,  131,  132,  133,  134,  135,  136,
 /*  1000 */   137,  138,  139,  140,  141,  142,  143,  144,  145,  146,
 /*  1010 */   147,  148,  149,  150,  151,  152,  153,  154,  155,  156,
 /*  1020 */    13,   15,  159,  160,   13,  162,   10,  164,   15,  166,
 /*  1030 */    13,   13,   13,   13,   13,   13,   13,   13,  175,  176,
 /*  1040 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  1050 */   187,  188,  189,  190,  191,  192,  193,   98,   99,  100,
 /*  1060 */   101,  102,  103,  104,  105,  106,  107,  108,  109,  110,
 /*  1070 */   111,  112,  113,  114,  115,  116,  117,  118,  119,  120,
 /*  1080 */   121,  122,  123,  124,  125,  126,  127,  128,  129,  130,
 /*  1090 */   131,  132,  133,  134,  135,  136,  137,  138,  139,  140,
 /*  1100 */   141,  142,  143,  144,  145,  146,  147,  148,  149,  150,
 /*  1110 */   151,  152,  153,  154,  155,  156,   13,   13,  159,  160,
 /*  1120 */    13,  162,   13,  164,   13,  166,   13,   13,   13,   13,
 /*  1130 */    13,   13,   13,   13,  175,  176,  177,  178,  179,  180,
 /*  1140 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  1150 */   191,  192,  193,   98,   99,  100,  101,  102,  103,  104,
 /*  1160 */   105,  106,  107,  108,  109,  110,  111,  112,  113,  114,
 /*  1170 */   115,  116,  117,  118,  119,  120,  121,  122,  123,  124,
 /*  1180 */   125,  126,  127,  128,  129,  130,  131,  132,  133,  134,
 /*  1190 */   135,  136,  137,  138,  139,  140,  141,  142,  143,  144,
 /*  1200 */   145,  146,  147,  148,  149,  150,  151,  152,  153,  154,
 /*  1210 */   155,  156,   13,   13,  159,  160,   13,  162,   13,  164,
 /*  1220 */    13,  166,   13,   13,   13,   13,   13,   13,   13,   13,
 /*  1230 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  1240 */   185,  186,  187,  188,  189,  190,  191,  192,  193,   98,
 /*  1250 */    99,  100,  101,  102,  103,  104,  105,  106,  107,  108,
 /*  1260 */   109,  110,  111,  112,  113,  114,  115,  116,  117,  118,
 /*  1270 */   119,  120,  121,  122,  123,  124,  125,  126,  127,  128,
 /*  1280 */   129,  130,  131,  132,  133,  134,  135,  136,  137,  138,
 /*  1290 */   139,  140,  141,  142,  143,  144,  145,  146,  147,  148,
 /*  1300 */   149,  150,  151,  152,  153,  154,  155,  156,   13,   13,
 /*  1310 */   159,  160,   13,  162,   13,  164,   13,  166,   13,   13,
 /*  1320 */    13,   13,   13,    6,    5,    5,  175,  176,  177,  178,
 /*  1330 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  1340 */   189,  190,  191,  192,  193,   98,   99,  100,  101,  102,
 /*  1350 */   103,  104,  105,  106,  107,  108,  109,  110,  111,  112,
 /*  1360 */   113,  114,  115,  116,  117,  118,  119,  120,  121,  122,
 /*  1370 */   123,  124,  125,  126,  127,  128,  129,  130,  131,  132,
 /*  1380 */   133,  134,  135,  136,  137,  138,  139,  140,  141,  142,
 /*  1390 */   143,  144,  145,  146,  147,  148,  149,  150,  151,  152,
 /*  1400 */   153,  154,  155,  156,    6,    6,  159,  160,    5,  162,
 /*  1410 */     6,  164,   17,  166,  158,   13,  157,   10,    5,   14,
 /*  1420 */     6,    5,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  1430 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  1440 */   193,   98,   99,  100,  101,  102,  103,  104,  105,  106,
 /*  1450 */   107,  108,  109,  110,  111,  112,  113,  114,  115,  116,
 /*  1460 */   117,  118,  119,  120,  121,  122,  123,  124,  125,  126,
 /*  1470 */   127,  128,  129,  130,  131,  132,  133,  134,  135,  136,
 /*  1480 */   137,  138,  139,  140,  141,  142,  143,  144,  145,  146,
 /*  1490 */   147,  148,  149,  150,  151,  152,  153,  154,  155,  156,
 /*  1500 */     6,   17,  159,  160,  195,  162,  195,  164,  195,  166,
 /*  1510 */    15,   16,  195,  195,   46,  195,  195,  195,  175,  176,
 /*  1520 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  1530 */   187,  188,  189,  190,  191,  192,  193,    1,    2,    3,
 /*  1540 */     4,   73,  195,    7,    8,    9,  159,   11,   12,  195,
 /*  1550 */   195,   15,  195,   17,   18,  168,   22,   21,   24,   23,
 /*  1560 */    26,   27,   28,   29,   30,   31,   32,   33,   34,   35,
 /*  1570 */    36,   37,   38,   39,   40,   41,  195,  190,   42,   16,
 /*  1580 */    44,   86,   87,   88,   48,   49,   50,   51,   52,   53,
 /*  1590 */    54,   55,   56,    2,    3,    4,  195,  195,    7,    8,
 /*  1600 */     9,  159,   11,   12,  195,  195,   15,  195,   17,   18,
 /*  1610 */    74,   75,   21,   45,   23,  173,  195,   81,   82,  195,
 /*  1620 */    84,   85,   86,   87,   88,   89,   90,   91,   92,   93,
 /*  1630 */    94,   95,  190,   42,  195,   44,   68,   69,   70,   48,
 /*  1640 */    49,   50,   51,   52,   53,   54,   55,   56,  195,   86,
 /*  1650 */    87,   88,  195,  195,  195,  195,  195,  159,   45,   46,
 /*  1660 */    47,  195,  164,  165,  166,   74,   75,  195,  195,  195,
 /*  1670 */   195,  159,   81,   82,  195,   84,   85,   86,   87,   88,
 /*  1680 */    89,   90,   91,   92,   93,   94,   95,  156,  190,  195,
 /*  1690 */   159,  195,  195,  195,  195,  164,  195,  166,  159,   86,
 /*  1700 */    87,   88,  190,  164,  195,  166,  175,  176,  177,  178,
 /*  1710 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  1720 */   189,  190,  191,  192,  193,  194,  187,  188,  189,  190,
 /*  1730 */   191,  192,  193,  195,  195,  156,  195,  195,  159,  195,
 /*  1740 */   195,  195,    9,  164,  159,  166,  167,  195,  195,   16,
 /*  1750 */   195,  195,  195,  168,  175,  176,  177,  178,  179,  180,
 /*  1760 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  1770 */   191,  192,  193,  195,  156,  190,   43,  159,   45,   46,
 /*  1780 */    47,  195,  164,  159,  166,  167,  195,  159,  164,  195,
 /*  1790 */   166,  195,  164,  175,  176,  177,  178,  179,  180,  181,
 /*  1800 */   182,  183,  184,  185,  186,  187,  188,  189,  190,  191,
 /*  1810 */   192,  193,  188,  189,  190,  191,  192,  193,  190,   86,
 /*  1820 */    87,   88,  156,  195,  195,  159,  195,  195,  195,  195,
 /*  1830 */   164,  159,  166,  167,  195,  195,  159,  159,  195,  195,
 /*  1840 */   195,  175,  176,  177,  178,  179,  180,  181,  182,  183,
 /*  1850 */   184,  185,  186,  187,  188,  189,  190,  191,  192,  193,
 /*  1860 */   195,  195,  190,  156,  195,  195,  159,  190,  190,    9,
 /*  1870 */   159,  164,  195,  166,  167,  195,   16,  159,  159,  168,
 /*  1880 */   159,  195,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  1890 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  1900 */   193,  190,  156,   43,  195,  159,    9,  195,  190,  190,
 /*  1910 */   164,  190,  166,   16,  159,  169,  195,  195,  195,  195,
 /*  1920 */   195,  175,  176,  177,  178,  179,  180,  181,  182,  183,
 /*  1930 */   184,  185,  186,  187,  188,  189,  190,  191,  192,  193,
 /*  1940 */    43,  156,  195,  195,  159,  190,   86,   87,   88,  164,
 /*  1950 */   159,  166,  195,  195,  169,  159,  195,  159,  195,  168,
 /*  1960 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  1970 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  156,
 /*  1980 */   195,  190,  159,   86,   87,   88,  190,  164,  190,  166,
 /*  1990 */   159,  159,  169,  159,  195,  195,  159,  195,  175,  176,
 /*  2000 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  2010 */   187,  188,  189,  190,  191,  192,  193,  195,  195,  195,
 /*  2020 */   156,  190,  190,  159,  190,  195,    9,  190,  164,  195,
 /*  2030 */   166,  167,  195,   16,  195,  195,  195,  195,  195,  175,
 /*  2040 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  2050 */   186,  187,  188,  189,  190,  191,  192,  193,  195,  156,
 /*  2060 */    43,  195,  159,    9,  195,  195,  195,  164,  195,  166,
 /*  2070 */    16,  195,  169,  195,  195,  195,  195,  195,  175,  176,
 /*  2080 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  2090 */   187,  188,  189,  190,  191,  192,  193,   43,  156,  195,
 /*  2100 */   195,  159,  195,   86,   87,   88,  164,  195,  166,  195,
 /*  2110 */   195,  169,  195,  195,  195,  195,  195,  175,  176,  177,
 /*  2120 */   178,  179,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  2130 */   188,  189,  190,  191,  192,  193,  156,  195,  195,  159,
 /*  2140 */    86,   87,   88,  195,  164,  195,  166,  195,  195,  169,
 /*  2150 */   195,  195,  195,  195,  195,  175,  176,  177,  178,  179,
 /*  2160 */   180,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  2170 */   190,  191,  192,  193,  195,  195,  195,  156,  195,  195,
 /*  2180 */   159,  195,  195,    9,  195,  164,  195,  166,  195,  195,
 /*  2190 */    16,  195,  195,  172,  195,  195,  175,  176,  177,  178,
 /*  2200 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  2210 */   189,  190,  191,  192,  193,  195,  156,   43,  195,  159,
 /*  2220 */   195,  195,  195,  195,  164,  195,  166,  167,  195,  195,
 /*  2230 */   195,  195,  195,  195,  195,  175,  176,  177,  178,  179,
 /*  2240 */   180,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  2250 */   190,  191,  192,  193,  195,  156,  195,  195,  159,  195,
 /*  2260 */    86,   87,   88,  164,  195,  166,  195,  195,  169,  195,
 /*  2270 */   195,  195,  195,  195,  175,  176,  177,  178,  179,  180,
 /*  2280 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  2290 */   191,  192,  193,  156,  195,  195,  159,  195,  195,  195,
 /*  2300 */   195,  164,  195,  166,  195,  195,  169,  195,  195,  195,
 /*  2310 */   195,  195,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  2320 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  2330 */   193,  195,  195,  195,  156,  195,  195,  159,  195,  195,
 /*  2340 */   195,  195,  164,  195,  166,  195,  195,  169,  195,  195,
 /*  2350 */   195,  195,  195,  175,  176,  177,  178,  179,  180,  181,
 /*  2360 */   182,  183,  184,  185,  186,  187,  188,  189,  190,  191,
 /*  2370 */   192,  193,  195,  156,  195,  195,  159,    9,  195,  195,
 /*  2380 */   195,  164,  195,  166,   16,  195,  195,  195,  195,  172,
 /*  2390 */   195,  195,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  2400 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  2410 */   193,   43,  156,  195,  195,  159,  195,  195,  195,  195,
 /*  2420 */   164,  195,  166,  195,  195,  169,  195,  195,  195,  195,
 /*  2430 */   195,  175,  176,  177,  178,  179,  180,  181,  182,  183,
 /*  2440 */   184,  185,  186,  187,  188,  189,  190,  191,  192,  193,
 /*  2450 */   156,  195,  195,  159,   86,   87,   88,  195,  164,  195,
 /*  2460 */   166,  195,  195,  169,  195,  195,  195,  195,  195,  175,
 /*  2470 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  2480 */   186,  187,  188,  189,  190,  191,  192,  193,  195,  195,
 /*  2490 */   195,  156,  195,  195,  159,  195,  195,  195,  195,  164,
 /*  2500 */   195,  166,  195,  195,  169,  195,  195,  195,  195,  195,
 /*  2510 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  2520 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  195,
 /*  2530 */   156,  195,  195,  159,  195,  195,  195,  195,  164,  195,
 /*  2540 */   166,  195,  195,  195,  195,  195,  172,  195,  195,  175,
 /*  2550 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  2560 */   186,  187,  188,  189,  190,  191,  192,  193,  195,  156,
 /*  2570 */   195,  195,  159,  195,  195,    9,  195,  164,  195,  166,
 /*  2580 */   195,  195,   16,  195,  195,  172,  195,  195,  175,  176,
 /*  2590 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  2600 */   187,  188,  189,  190,  191,  192,  193,  156,  195,   43,
 /*  2610 */   159,   45,   46,   47,  195,  164,  195,  166,  195,  195,
 /*  2620 */   195,  170,  195,  195,  195,  195,  175,  176,  177,  178,
 /*  2630 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  2640 */   189,  190,  191,  192,  193,  195,  195,  195,  156,  195,
 /*  2650 */   195,  159,   86,   87,   88,  195,  164,  195,  166,  195,
 /*  2660 */   195,  195,  170,  195,  195,  195,  195,  175,  176,  177,
 /*  2670 */   178,  179,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  2680 */   188,  189,  190,  191,  192,  193,  195,  156,  195,  195,
 /*  2690 */   159,  195,  195,  195,  195,  164,  195,  166,  195,  195,
 /*  2700 */   195,  170,  195,  195,  195,  195,  175,  176,  177,  178,
 /*  2710 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  2720 */   189,  190,  191,  192,  193,  195,  156,  195,  195,  159,
 /*  2730 */   195,  195,  195,  195,  164,  195,  166,  167,  195,  195,
 /*  2740 */   195,  195,  195,  195,  195,  175,  176,  177,  178,  179,
 /*  2750 */   180,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  2760 */   190,  191,  192,  193,  156,  195,  195,  159,  195,  195,
 /*  2770 */   195,  195,  164,  195,  166,  195,  195,  195,  170,  195,
 /*  2780 */   195,  195,  195,  175,  176,  177,  178,  179,  180,  181,
 /*  2790 */   182,  183,  184,  185,  186,  187,  188,  189,  190,  191,
 /*  2800 */   192,  193,  195,  195,  195,  156,  195,  195,  159,  195,
 /*  2810 */   195,  195,  195,  164,  195,  166,  195,  195,  195,  170,
 /*  2820 */   195,  195,  195,  195,  175,  176,  177,  178,  179,  180,
 /*  2830 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  2840 */   191,  192,  193,  195,  156,  195,  195,  159,  195,  195,
 /*  2850 */   195,  195,  164,  195,  166,  195,  195,  195,  170,  195,
 /*  2860 */   195,  195,  195,  175,  176,  177,  178,  179,  180,  181,
 /*  2870 */   182,  183,  184,  185,  186,  187,  188,  189,  190,  191,
 /*  2880 */   192,  193,  195,  156,  195,  195,  159,  195,  195,  195,
 /*  2890 */   195,  164,  195,  166,  195,  195,  195,  195,  195,  195,
 /*  2900 */   195,  174,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  2910 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  2920 */   193,  156,  195,  195,  159,  195,  195,  195,  195,  164,
 /*  2930 */   195,  166,  167,  195,  195,  195,  195,  195,  195,  195,
 /*  2940 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  2950 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  195,
 /*  2960 */   195,  195,  156,  195,  195,  159,  195,  195,  195,  195,
 /*  2970 */   164,  195,  166,  195,  195,  195,  170,  195,  195,  195,
 /*  2980 */   195,  175,  176,  177,  178,  179,  180,  181,  182,  183,
 /*  2990 */   184,  185,  186,  187,  188,  189,  190,  191,  192,  193,
 /*  3000 */   195,  156,  195,  195,  159,  195,  195,  195,  195,  164,
 /*  3010 */   195,  166,  195,  195,  195,  170,  195,  195,  195,  195,
 /*  3020 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  3030 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  195,
 /*  3040 */   156,  195,  195,  159,  195,  195,  195,  195,  164,  195,
 /*  3050 */   166,  195,  195,  195,  170,  195,  195,  195,  195,  175,
 /*  3060 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  3070 */   186,  187,  188,  189,  190,  191,  192,  193,  156,  195,
 /*  3080 */   195,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  3090 */   195,  195,  195,  195,  195,  195,  174,  175,  176,  177,
 /*  3100 */   178,  179,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  3110 */   188,  189,  190,  191,  192,  193,  195,  195,  195,  156,
 /*  3120 */   195,  195,  159,  195,  195,  195,  195,  164,  195,  166,
 /*  3130 */   195,  195,  195,  170,  195,  195,  195,  195,  175,  176,
 /*  3140 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  3150 */   187,  188,  189,  190,  191,  192,  193,  195,  156,  195,
 /*  3160 */   195,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  3170 */   195,  195,  170,  195,  195,  195,  195,  175,  176,  177,
 /*  3180 */   178,  179,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  3190 */   188,  189,  190,  191,  192,  193,  195,  156,  195,  195,
 /*  3200 */   159,  195,  195,  195,  195,  164,  195,  166,  195,  195,
 /*  3210 */   195,  170,  195,  195,  195,  195,  175,  176,  177,  178,
 /*  3220 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  3230 */   189,  190,  191,  192,  193,  156,  195,  195,  159,  195,
 /*  3240 */   195,  195,  195,  164,  195,  166,  195,  195,  195,  195,
 /*  3250 */   195,  195,  195,  174,  175,  176,  177,  178,  179,  180,
 /*  3260 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  3270 */   191,  192,  193,  195,  195,  195,  156,  195,  195,  159,
 /*  3280 */   195,  195,  195,  195,  164,  195,  166,  195,  195,  195,
 /*  3290 */   195,  195,  195,  195,  174,  175,  176,  177,  178,  179,
 /*  3300 */   180,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  3310 */   190,  191,  192,  193,  195,  156,  195,  195,  159,  195,
 /*  3320 */   195,  195,  195,  164,  195,  166,  167,  195,  195,  195,
 /*  3330 */   195,  195,  195,  195,  175,  176,  177,  178,  179,  180,
 /*  3340 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  3350 */   191,  192,  193,  195,  156,  195,  195,  159,  195,  195,
 /*  3360 */   195,  195,  164,  195,  166,  167,  195,  195,  195,  195,
 /*  3370 */   195,  195,  195,  175,  176,  177,  178,  179,  180,  181,
 /*  3380 */   182,  183,  184,  185,  186,  187,  188,  189,  190,  191,
 /*  3390 */   192,  193,  156,  195,  195,  159,  195,  195,  195,  195,
 /*  3400 */   164,  195,  166,  167,  195,  195,  195,  195,  195,  195,
 /*  3410 */   195,  175,  176,  177,  178,  179,  180,  181,  182,  183,
 /*  3420 */   184,  185,  186,  187,  188,  189,  190,  191,  192,  193,
 /*  3430 */   195,  195,  195,  156,  195,  195,  159,  195,  195,  195,
 /*  3440 */   195,  164,  195,  166,  167,  195,  195,  195,  195,  195,
 /*  3450 */   195,  195,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  3460 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  3470 */   193,  195,  156,  195,  195,  159,  195,  195,  195,  195,
 /*  3480 */   164,  195,  166,  167,  195,  195,  195,  195,  195,  195,
 /*  3490 */   195,  175,  176,  177,  178,  179,  180,  181,  182,  183,
 /*  3500 */   184,  185,  186,  187,  188,  189,  190,  191,  192,  193,
 /*  3510 */   195,  156,  195,  195,  159,  195,  195,  195,  195,  164,
 /*  3520 */   195,  166,  167,  195,  195,  195,  195,  195,  195,  195,
 /*  3530 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  3540 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  156,
 /*  3550 */   195,  195,  159,  195,  195,  195,  195,  164,  195,  166,
 /*  3560 */   195,  195,  195,  195,  195,  195,  195,  195,  175,  176,
 /*  3570 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  3580 */   187,  188,  189,  190,  191,  192,  193,  195,  195,  195,
 /*  3590 */   156,  195,  195,  159,  195,  195,  195,  195,  164,  195,
 /*  3600 */   166,  195,  195,  195,  195,  195,  195,  195,  195,  175,
 /*  3610 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  3620 */   186,  187,  188,  189,  190,  191,  192,  193,  195,  156,
 /*  3630 */   195,  195,  159,  195,  195,  195,  195,  164,  195,  166,
 /*  3640 */   195,  195,  195,  195,  195,  195,  195,  195,  175,  176,
 /*  3650 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  3660 */   187,  188,  189,  190,  191,  192,  193,  195,  156,  195,
 /*  3670 */   195,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  3680 */   195,  195,  195,  195,  195,  195,  195,  175,  176,  177,
 /*  3690 */   178,  179,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  3700 */   188,  189,  190,  191,  192,  193,  156,  195,  195,  159,
 /*  3710 */   195,  195,  195,  195,  164,  195,  166,  195,  195,  195,
 /*  3720 */   195,  195,  195,  195,  195,  175,  176,  177,  178,  179,
 /*  3730 */   180,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  3740 */   190,  191,  192,  193,  195,  195,  195,  156,  195,  195,
 /*  3750 */   159,  195,  195,  195,  195,  164,  195,  166,  195,  195,
 /*  3760 */   195,  195,  195,  195,  195,  195,  175,  176,  177,  178,
 /*  3770 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  3780 */   189,  190,  191,  192,  193,  195,  156,  195,  195,  159,
 /*  3790 */   195,  195,  195,  195,  164,  195,  166,  195,  195,  195,
 /*  3800 */   195,  195,  195,  195,  195,  175,  176,  177,  178,  179,
 /*  3810 */   180,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  3820 */   190,  191,  192,  193,  195,  156,  195,  195,  159,  195,
 /*  3830 */   195,  195,  195,  164,  195,  166,  195,  195,  195,  195,
 /*  3840 */   195,  195,  195,  195,  175,  176,  177,  178,  179,  180,
 /*  3850 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  3860 */   191,  192,  193,  156,  195,  195,  159,  195,  195,  195,
 /*  3870 */   195,  164,  195,  166,  195,  195,  195,  195,  195,  195,
 /*  3880 */   195,  195,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  3890 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  3900 */   193,  195,  195,  195,  156,  195,  195,  159,  195,  195,
 /*  3910 */   195,  195,  164,  195,  166,  195,  195,  195,  195,  195,
 /*  3920 */   195,  195,  195,  175,  176,  177,  178,  179,  180,  181,
 /*  3930 */   182,  183,  184,  185,  186,  187,  188,  189,  190,  191,
 /*  3940 */   192,  193,  195,  156,  195,  195,  159,  195,  195,  195,
 /*  3950 */   195,  164,  195,  166,  195,  195,  195,  195,  195,  195,
 /*  3960 */   195,  195,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  3970 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  3980 */   193,  195,  156,  195,  195,  159,  195,  195,  195,  195,
 /*  3990 */   164,  195,  166,  195,  195,  195,  195,  195,  195,  195,
 /*  4000 */   195,  175,  176,  177,  178,  179,  180,  181,  182,  183,
 /*  4010 */   184,  185,  186,  187,  188,  189,  190,  191,  192,  193,
 /*  4020 */   156,  195,  195,  159,  195,  195,  195,  195,  164,  195,
 /*  4030 */   166,  195,  195,  195,  195,  195,  195,  195,  195,  175,
 /*  4040 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  4050 */   186,  187,  188,  189,  190,  191,  192,  193,  195,  195,
 /*  4060 */   195,  156,  195,  195,  159,  195,  195,  195,  195,  164,
 /*  4070 */   195,  166,  195,  195,  195,  195,  195,  195,  195,  195,
 /*  4080 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  4090 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  195,
 /*  4100 */   156,  195,  195,  159,  195,  195,  195,  195,  164,  195,
 /*  4110 */   166,  195,  195,  195,  195,  195,  195,  195,  195,  175,
 /*  4120 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  4130 */   186,  187,  188,  189,  190,  191,  192,  193,  195,  156,
 /*  4140 */   195,  195,  159,  195,  195,  195,  195,  164,  195,  166,
 /*  4150 */   195,  195,  195,  195,  195,  195,  195,  195,  175,  176,
 /*  4160 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  4170 */   187,  188,  189,  190,  191,  192,  193,  156,  195,  195,
 /*  4180 */   159,  195,  195,  195,  195,  164,  195,  166,  195,  195,
 /*  4190 */   195,  195,  195,  195,  195,  195,  175,  176,  177,  178,
 /*  4200 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  4210 */   189,  190,  191,  192,  193,  195,  195,  195,  156,  195,
 /*  4220 */   195,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  4230 */   195,  195,  195,  195,  195,  195,  195,  175,  176,  177,
 /*  4240 */   178,  179,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  4250 */   188,  189,  190,  191,  192,  193,  195,  156,  195,  195,
 /*  4260 */   159,  195,  195,  195,  195,  164,  195,  166,  195,  195,
 /*  4270 */   195,  195,  195,  195,  195,  195,  175,  176,  177,  178,
 /*  4280 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  4290 */   189,  190,  191,  192,  193,  195,  156,  195,  195,  159,
 /*  4300 */   195,  195,  195,  195,  164,  195,  166,  195,  195,  195,
 /*  4310 */   195,  195,  195,  195,  195,  175,  176,  177,  178,  179,
 /*  4320 */   180,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  4330 */   190,  191,  192,  193,  156,  195,  195,  159,  195,  195,
 /*  4340 */   195,  195,  164,  195,  166,  195,  195,  195,  195,  195,
 /*  4350 */   195,  195,  195,  175,  176,  177,  178,  179,  180,  181,
 /*  4360 */   182,  183,  184,  185,  186,  187,  188,  189,  190,  191,
 /*  4370 */   192,  193,  195,  195,  195,  156,  195,  195,  159,  195,
 /*  4380 */   195,  195,  195,  164,  195,  166,  195,  195,  195,  195,
 /*  4390 */   195,  195,  195,  195,  175,  176,  177,  178,  179,  180,
 /*  4400 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  4410 */   191,  192,  193,  195,  156,  195,  195,  159,  195,  195,
 /*  4420 */   195,  195,  164,  195,  166,  195,  195,  195,  195,  195,
 /*  4430 */   195,  195,  195,  175,  176,  177,  178,  179,  180,  181,
 /*  4440 */   182,  183,  184,  185,  186,  187,  188,  189,  190,  191,
 /*  4450 */   192,  193,  195,  156,  195,  195,  159,  195,  195,  195,
 /*  4460 */   195,  164,  195,  166,  195,  195,  195,  195,  195,  195,
 /*  4470 */   195,  195,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  4480 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  4490 */   193,  156,  195,  195,  159,  195,  195,  195,  195,  164,
 /*  4500 */   195,  166,  195,  195,  195,  195,  195,  195,  195,  195,
 /*  4510 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  4520 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  195,
 /*  4530 */   195,  195,  156,  195,  195,  159,  195,  195,  195,  195,
 /*  4540 */   164,  195,  166,  195,  195,  195,  195,  195,  195,  195,
 /*  4550 */   195,  175,  176,  177,  178,  179,  180,  181,  182,  183,
 /*  4560 */   184,  185,  186,  187,  188,  189,  190,  191,  192,  193,
 /*  4570 */   195,  156,  195,  195,  159,  195,  195,  195,  195,  164,
 /*  4580 */   195,  166,  195,  195,  195,  195,  195,  195,  195,  195,
 /*  4590 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  4600 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  195,
 /*  4610 */   156,  195,  195,  159,  195,  195,  195,  195,  164,  195,
 /*  4620 */   166,  195,  195,  195,  195,  195,  195,  195,  195,  175,
 /*  4630 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  4640 */   186,  187,  188,  189,  190,  191,  192,  193,  156,  195,
 /*  4650 */   195,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  4660 */   195,  195,  195,  195,  195,  195,  195,  175,  176,  177,
 /*  4670 */   178,  179,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  4680 */   188,  189,  190,  191,  192,  193,  195,  195,  195,  156,
 /*  4690 */   195,  195,  159,  195,  195,  195,  195,  164,  195,  166,
 /*  4700 */   195,  195,  195,  195,  195,  195,  195,  195,  175,  176,
 /*  4710 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  4720 */   187,  188,  189,  190,  191,  192,  193,  195,  156,  195,
 /*  4730 */   195,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  4740 */   195,  195,  195,  195,  195,  195,  195,  175,  176,  177,
 /*  4750 */   178,  179,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  4760 */   188,  189,  190,  191,  192,  193,  195,  156,  195,  195,
 /*  4770 */   159,  195,  195,  195,  195,  164,  195,  166,  195,  195,
 /*  4780 */   195,  195,  195,  195,  195,  195,  175,  176,  177,  178,
 /*  4790 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  4800 */   189,  190,  191,  192,  193,  156,  195,  195,  159,  195,
 /*  4810 */   195,  195,  195,  164,  195,  166,  195,  195,  195,  195,
 /*  4820 */   195,  195,  195,  195,  175,  176,  177,  178,  179,  180,
 /*  4830 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  4840 */   191,  192,  193,  195,  195,  195,  156,  195,  195,  159,
 /*  4850 */   195,  195,  195,  195,  164,  195,  166,  195,  195,  195,
 /*  4860 */   195,  195,  195,  195,  195,  175,  176,  177,  178,  179,
 /*  4870 */   180,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  4880 */   190,  191,  192,  193,  195,  156,  195,  195,  159,  195,
 /*  4890 */   195,  195,  195,  164,  195,  166,  195,  195,  195,  195,
 /*  4900 */   195,  195,  195,  195,  175,  176,  177,  178,  179,  180,
 /*  4910 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  4920 */   191,  192,  193,  195,  156,  195,  195,  159,  195,  195,
 /*  4930 */   195,  195,  164,  195,  166,  195,  195,  195,  195,  195,
 /*  4940 */   195,  195,  195,  175,  176,  177,  178,  179,  180,  181,
 /*  4950 */   182,  183,  184,  185,  186,  187,  188,  189,  190,  191,
 /*  4960 */   192,  193,  156,  195,  195,  159,  195,  195,  195,  195,
 /*  4970 */   164,  195,  166,  195,  195,  195,  195,  195,  195,  195,
 /*  4980 */   195,  175,  176,  177,  178,  179,  180,  181,  182,  183,
 /*  4990 */   184,  185,  186,  187,  188,  189,  190,  191,  192,  193,
 /*  5000 */   195,  195,  195,  156,  195,  195,  159,  195,  195,  195,
 /*  5010 */   195,  164,  195,  166,  195,  195,  195,  195,  195,  195,
 /*  5020 */   195,  195,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  5030 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  5040 */   193,  195,  156,  195,  195,  159,  195,  195,  195,  195,
 /*  5050 */   164,  195,  166,  195,  195,  195,  195,  195,  195,  195,
 /*  5060 */   195,  175,  176,  177,  178,  179,  180,  181,  182,  183,
 /*  5070 */   184,  185,  186,  187,  188,  189,  190,  191,  192,  193,
 /*  5080 */   195,  156,  195,  195,  159,  195,  195,  195,  195,  164,
 /*  5090 */   195,  166,  195,  195,  195,  195,  195,  195,  195,  195,
 /*  5100 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  5110 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  156,
 /*  5120 */   195,  195,  159,  195,  195,  195,  195,  164,  195,  166,
 /*  5130 */   195,  195,  195,  195,  195,  195,  195,  195,  175,  176,
 /*  5140 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  5150 */   187,  188,  189,  190,  191,  192,  193,  195,  195,  195,
 /*  5160 */   156,  195,  195,  159,  195,  195,  195,  195,  164,  195,
 /*  5170 */   166,  195,  195,  195,  195,  195,  195,  195,  195,  175,
 /*  5180 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  5190 */   186,  187,  188,  189,  190,  191,  192,  193,  195,  156,
 /*  5200 */   195,  195,  159,  195,  195,  195,  195,  164,  195,  166,
 /*  5210 */   195,  195,  195,  195,  195,  195,  195,  195,  175,  176,
 /*  5220 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  5230 */   187,  188,  189,  190,  191,  192,  193,  195,  156,  195,
 /*  5240 */   195,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  5250 */   195,  195,  195,  195,  195,  195,  195,  175,  176,  177,
 /*  5260 */   178,  179,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  5270 */   188,  189,  190,  191,  192,  193,  156,  195,  195,  159,
 /*  5280 */   195,  195,  195,  195,  164,  195,  166,  195,  195,  195,
 /*  5290 */   195,  195,  195,  195,  195,  175,  176,  177,  178,  179,
 /*  5300 */   180,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  5310 */   190,  191,  192,  193,  195,  195,  195,  156,  195,  195,
 /*  5320 */   159,  195,  195,  195,  195,  164,  195,  166,  195,  195,
 /*  5330 */   195,  195,  195,  195,  195,  195,  175,  176,  177,  178,
 /*  5340 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  5350 */   189,  190,  191,  192,  193,  195,  156,  195,  195,  159,
 /*  5360 */   195,  195,  195,  195,  164,  195,  166,  195,  195,  195,
 /*  5370 */   195,  195,  195,  195,  195,  175,  176,  177,  178,  179,
 /*  5380 */   180,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  5390 */   190,  191,  192,  193,  195,  156,  195,  195,  159,  195,
 /*  5400 */   195,  195,  195,  164,  195,  166,  195,  195,  195,  195,
 /*  5410 */   195,  195,  195,  195,  175,  176,  177,  178,  179,  180,
 /*  5420 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  5430 */   191,  192,  193,  156,  195,  195,  159,  195,  195,  195,
 /*  5440 */   195,  164,  195,  166,  195,  195,  195,  195,  195,  195,
 /*  5450 */   195,  195,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  5460 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  5470 */   193,  195,  195,  195,  156,  195,  195,  159,  195,  195,
 /*  5480 */   195,  195,  164,  195,  166,  195,  195,  195,  195,  195,
 /*  5490 */   195,  195,  195,  175,  176,  177,  178,  179,  180,  181,
 /*  5500 */   182,  183,  184,  185,  186,  187,  188,  189,  190,  191,
 /*  5510 */   192,  193,  195,  156,  195,  195,  159,  195,  195,  195,
 /*  5520 */   195,  164,  195,  166,  195,  195,  195,  195,  195,  195,
 /*  5530 */   195,  195,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  5540 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  5550 */   193,  195,  156,  195,  195,  159,  195,  195,  195,  195,
 /*  5560 */   164,  195,  166,  195,  195,  195,  195,  195,  195,  195,
 /*  5570 */   195,  175,  176,  177,  178,  179,  180,  181,  182,  183,
 /*  5580 */   184,  185,  186,  187,  188,  189,  190,  191,  192,  193,
 /*  5590 */   156,  195,  195,  159,  195,  195,  195,  195,  164,  195,
 /*  5600 */   166,  195,  195,  195,  195,  195,  195,  195,  195,  175,
 /*  5610 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  5620 */   186,  187,  188,  189,  190,  191,  192,  193,  195,  195,
 /*  5630 */   195,  156,  195,  195,  159,  195,  195,  195,  195,  164,
 /*  5640 */   195,  166,  195,  195,  195,  195,  195,  195,  195,  195,
 /*  5650 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  5660 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  195,
 /*  5670 */   156,  195,  195,  159,  195,  195,  195,  195,  164,  195,
 /*  5680 */   166,  195,  195,  195,  195,  195,  195,  195,  195,  175,
 /*  5690 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  5700 */   186,  187,  188,  189,  190,  191,  192,  193,  195,  156,
 /*  5710 */   195,  195,  159,  195,  195,  195,  195,  164,  195,  166,
 /*  5720 */   195,  195,  195,  195,  195,  195,  195,  195,  175,  176,
 /*  5730 */   177,  178,  179,  180,  181,  182,  183,  184,  185,  186,
 /*  5740 */   187,  188,  189,  190,  191,  192,  193,  156,  195,  195,
 /*  5750 */   159,  195,  195,  195,  195,  164,  195,  166,  195,  195,
 /*  5760 */   195,  195,  195,  195,  195,  195,  175,  176,  177,  178,
 /*  5770 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  5780 */   189,  190,  191,  192,  193,  195,  195,  195,  156,  195,
 /*  5790 */   195,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  5800 */   195,  195,  195,  195,  195,  195,  195,  175,  176,  177,
 /*  5810 */   178,  179,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  5820 */   188,  189,  190,  191,  192,  193,  195,  156,  195,  195,
 /*  5830 */   159,  195,  195,  195,  195,  164,  195,  166,  195,  195,
 /*  5840 */   195,  195,  195,  195,  195,  195,  175,  176,  177,  178,
 /*  5850 */   179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
 /*  5860 */   189,  190,  191,  192,  193,  195,  156,  195,  195,  159,
 /*  5870 */   195,  195,  195,  195,  164,  195,  166,  195,  195,  195,
 /*  5880 */   195,  195,  195,  195,  195,  175,  176,  177,  178,  179,
 /*  5890 */   180,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  5900 */   190,  191,  192,  193,  156,  195,  195,  159,  195,  195,
 /*  5910 */   195,  195,  164,  195,  166,  195,  195,  195,  195,  195,
 /*  5920 */   195,  195,  195,  175,  176,  177,  178,  179,  180,  181,
 /*  5930 */   182,  183,  184,  185,  186,  187,  188,  189,  190,  191,
 /*  5940 */   192,  193,  195,  195,  195,  156,  195,  195,  159,  195,
 /*  5950 */   195,  195,  195,  164,  195,  166,  195,  195,  195,  195,
 /*  5960 */   195,  195,  195,  195,  175,  176,  177,  178,  179,  180,
 /*  5970 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  5980 */   191,  192,  193,  195,  156,  195,  195,  159,  195,  195,
 /*  5990 */   195,  195,  164,  195,  166,  195,  195,  195,  195,  195,
 /*  6000 */   195,  195,  195,  175,  176,  177,  178,  179,  180,  181,
 /*  6010 */   182,  183,  184,  185,  186,  187,  188,  189,  190,  191,
 /*  6020 */   192,  193,  195,  156,  195,  195,  159,  195,  195,  195,
 /*  6030 */   195,  164,  195,  166,  195,  195,  195,  195,  195,  195,
 /*  6040 */   195,  195,  175,  176,  177,  178,  179,  180,  181,  182,
 /*  6050 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  6060 */   193,  156,  195,  195,  159,  195,  195,  195,  195,  164,
 /*  6070 */   195,  166,  195,  195,  195,  195,  195,  195,  195,  195,
 /*  6080 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  6090 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  195,
 /*  6100 */   195,  195,  156,  195,  195,  159,  195,  195,  195,  195,
 /*  6110 */   164,  195,  166,  195,  195,  195,  195,  195,  195,  195,
 /*  6120 */   195,  175,  176,  177,  178,  179,  180,  181,  182,  183,
 /*  6130 */   184,  185,  186,  187,  188,  189,  190,  191,  192,  193,
 /*  6140 */   195,  156,  195,  195,  159,  195,  195,  195,  195,  164,
 /*  6150 */   195,  166,  195,  195,  195,  195,  195,  195,  195,  195,
 /*  6160 */   175,  176,  177,  178,  179,  180,  181,  182,  183,  184,
 /*  6170 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  195,
 /*  6180 */   156,  195,  195,  159,  195,  195,  195,  195,  164,  195,
 /*  6190 */   166,  195,  195,  195,  195,  195,  195,  195,  195,  175,
 /*  6200 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  6210 */   186,  187,  188,  189,  190,  191,  192,  193,  159,  195,
 /*  6220 */   195,  195,  195,  164,  195,  166,  195,  195,  195,  195,
 /*  6230 */   195,  195,  195,  195,  175,  176,  177,  178,  179,  180,
 /*  6240 */   181,  182,  183,  184,  185,  186,  187,  188,  189,  190,
 /*  6250 */   191,  192,  193,  159,    9,  195,  195,  195,  164,  195,
 /*  6260 */   166,  195,   17,  195,  195,  195,  195,  195,   23,  175,
 /*  6270 */   176,  177,  178,  179,  180,  181,  182,  183,  184,  185,
 /*  6280 */   186,  187,  188,  189,  190,  191,  192,  193,    9,  195,
 /*  6290 */    45,   46,   47,  195,   15,  195,   17,  195,  195,   22,
 /*  6300 */   195,  195,   23,   26,   27,   28,   29,   30,   31,   32,
 /*  6310 */    33,   34,   35,   36,   37,   38,   39,   40,   41,   74,
 /*  6320 */    75,  195,  195,  195,  195,  195,   81,   82,  159,   84,
 /*  6330 */    85,   86,   87,   88,   89,   90,   91,   92,   93,   94,
 /*  6340 */    95,    9,  173,  195,  195,  195,  195,  195,  195,   17,
 /*  6350 */   159,  195,  195,   74,   75,   23,  195,  195,  195,  190,
 /*  6360 */    81,   82,  171,   84,   85,   86,   87,   88,   89,   90,
 /*  6370 */    91,   92,   93,   94,   95,  195,  195,   45,   46,   47,
 /*  6380 */   195,  190,  195,  195,  195,  195,  195,    9,  195,  195,
 /*  6390 */   195,  195,  195,  195,  195,   17,  195,  159,  159,  159,
 /*  6400 */   195,   23,  195,  195,  195,  195,   74,   75,  195,  171,
 /*  6410 */   171,  171,  195,   81,   82,  195,   84,   85,   86,   87,
 /*  6420 */    88,   89,   90,   91,   92,   93,   94,   95,  190,  190,
 /*  6430 */   190,  195,  195,  195,  195,   57,   58,  195,  195,  195,
 /*  6440 */   195,  195,  195,  195,  159,  195,  195,  159,  195,  195,
 /*  6450 */   195,  195,   74,   75,  195,  195,  195,  195,  173,   81,
 /*  6460 */    82,  173,   84,   85,   86,   87,   88,   89,   90,   91,
 /*  6470 */    92,   93,   94,   95,    9,  190,  195,  195,  190,  159,
 /*  6480 */   195,  195,   17,  195,  164,  195,  166,  195,   23,  159,
 /*  6490 */   195,  195,  195,  195,  164,  195,  166,  177,  178,  179,
 /*  6500 */   180,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  6510 */   190,  191,  192,  193,  184,  185,  186,  187,  188,  189,
 /*  6520 */   190,  191,  192,  193,  195,  195,  195,  195,  195,  195,
 /*  6530 */   195,  195,  195,  195,  195,   15,   16,  195,  195,   74,
 /*  6540 */    75,  195,  195,   23,  195,  195,   81,   82,  195,   84,
 /*  6550 */    85,   86,   87,   88,   89,   90,   91,   92,   93,   94,
 /*  6560 */    95,  159,  195,   43,    9,  195,  164,  195,  166,  195,
 /*  6570 */   195,  195,   17,  195,  195,  195,  195,  195,   23,  195,
 /*  6580 */   178,  179,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  6590 */   188,  189,  190,  191,  192,  193,  159,  195,  195,  195,
 /*  6600 */   195,  164,  195,  166,   84,   85,   86,   87,   88,  195,
 /*  6610 */   195,  195,  195,  195,  195,  195,  179,  180,  181,  182,
 /*  6620 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  6630 */   193,  159,  195,  195,  195,  195,  164,  195,  166,   84,
 /*  6640 */    85,   86,   87,   88,   89,   90,   91,   92,   93,   94,
 /*  6650 */    95,  195,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  6660 */   188,  189,  190,  191,  192,  193,  159,  195,  195,  195,
 /*  6670 */   195,  164,  195,  166,  195,  195,  195,  195,  195,  195,
 /*  6680 */   195,  195,  195,  195,  195,  195,  195,  180,  181,  182,
 /*  6690 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  6700 */   193,  195,  195,  195,  195,  195,  159,  195,  195,  195,
 /*  6710 */   195,  164,  195,  166,  195,  195,  195,  195,  195,  195,
 /*  6720 */   195,  195,  195,  195,  195,  195,  195,  180,  181,  182,
 /*  6730 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  6740 */   193,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  6750 */   195,  195,  195,  195,  159,  195,  195,  195,  195,  164,
 /*  6760 */   195,  166,  180,  181,  182,  183,  184,  185,  186,  187,
 /*  6770 */   188,  189,  190,  191,  192,  193,  181,  182,  183,  184,
 /*  6780 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  159,
 /*  6790 */   195,  195,  195,  195,  164,  195,  166,  195,  195,  195,
 /*  6800 */   195,  195,  195,  195,  195,  195,  195,  195,  195,  195,
 /*  6810 */   195,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  6820 */   190,  191,  192,  193,  159,  195,  195,  195,  195,  164,
 /*  6830 */   195,  166,  195,  195,  195,  195,  195,  195,  195,  195,
 /*  6840 */   195,  195,  195,  195,  195,  195,  181,  182,  183,  184,
 /*  6850 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  159,
 /*  6860 */   195,  195,  195,  195,  164,  195,  166,  195,  195,  195,
 /*  6870 */   195,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  6880 */   195,  181,  182,  183,  184,  185,  186,  187,  188,  189,
 /*  6890 */   190,  191,  192,  193,  182,  183,  184,  185,  186,  187,
 /*  6900 */   188,  189,  190,  191,  192,  193,  159,  195,  195,  195,
 /*  6910 */   195,  164,  195,  166,  195,  195,  195,  195,  195,  159,
 /*  6920 */   195,  195,  195,  195,  164,  195,  166,  195,  195,  195,
 /*  6930 */   183,  184,  185,  186,  187,  188,  189,  190,  191,  192,
 /*  6940 */   193,  195,  195,  195,  195,  185,  186,  187,  188,  189,
 /*  6950 */   190,  191,  192,  193,  159,  195,  195,  195,  195,  164,
 /*  6960 */   195,  166,  195,  195,  195,  195,  195,  195,  195,  195,
 /*  6970 */   195,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  6980 */   185,  186,  187,  188,  189,  190,  191,  192,  193,  159,
 /*  6990 */   195,  195,  195,  195,  164,  195,  166,  195,  186,  187,
 /*  7000 */   188,  189,  190,  191,  192,  193,  159,  195,  195,  195,
 /*  7010 */   195,  164,  195,  166,  195,  195,  186,  187,  188,  189,
 /*  7020 */   190,  191,  192,  193,  159,  195,  195,  195,  195,  164,
 /*  7030 */   195,  166,  195,  186,  187,  188,  189,  190,  191,  192,
 /*  7040 */   193,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  7050 */   195,  195,  187,  188,  189,  190,  191,  192,  193,  159,
 /*  7060 */   195,  195,  195,  195,  164,  195,  166,  195,  195,  187,
 /*  7070 */   188,  189,  190,  191,  192,  193,  195,  195,  195,  159,
 /*  7080 */   195,  195,  195,  195,  164,  195,  166,  187,  188,  189,
 /*  7090 */   190,  191,  192,  193,  159,  195,  195,  195,  195,  164,
 /*  7100 */   195,  166,  195,  195,  195,  195,  195,  187,  188,  189,
 /*  7110 */   190,  191,  192,  193,  159,  195,  195,  195,  195,  164,
 /*  7120 */   195,  166,  187,  188,  189,  190,  191,  192,  193,  195,
 /*  7130 */   195,  159,  195,  195,  195,  195,  164,  195,  166,  195,
 /*  7140 */   195,  195,  187,  188,  189,  190,  191,  192,  193,  195,
 /*  7150 */   195,  195,  195,  195,  195,  195,  195,  195,  195,  187,
 /*  7160 */   188,  189,  190,  191,  192,  193,
};
#define YY_SHIFT_USE_DFLT (-15)
static short yy_shift_ofst[] = {
 /*     0 */  1591,   61,  155, 1591, 1536,  -15,  -15,  -15,  -15,  -15,
 /*    10 */   -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,
 /*    20 */   -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,
 /*    30 */   -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,
 /*    40 */   -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,
 /*    50 */   -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,  -15,
 /*    60 */   -15,  -15,  -15,  157, 1591,  158,  -15,  251, 1591,  262,
 /*    70 */   -15,  264, 1591,  343,  -15,  347, 6465,  442,  453, 1591,
 /*    80 */   454,  -15,  541, 1591,  542,  548,  633, 6465,  634,  -15,
 /*    90 */   -15,    5, 6465,  590, 6465,  -15,  591, 6465,  589, 6465,
 /*   100 */   102, 6465, 1568, 6465,  686, 6465,  665, 6465,  666, 6465,
 /*   110 */  1468, 6465,    1, 6465,   93, 6465,  -15,  -15, 6465,  -15,
 /*   120 */  6465,  -15, 6465,  -15, 6465,  -15,  657, 6555,  -15,  -15,
 /*   130 */   -15,  -15,  -15,  718, 6465,  589,  102, 6465, 1568, 6465,
 /*   140 */   686,  665,  666, 1468, 6465,    1, 6465,   93, 6465,  -15,
 /*   150 */   719, 6465, 1568, 6465,  686,    1, 6465,   93, 6465,  -15,
 /*   160 */   -15,  -15,  -15,  720, 6465,  686,   93, 6465,  -15,  -15,
 /*   170 */   -15,  -15,  -15,  -15,  -15,  736, 6465,  -15,  -15,  -15,
 /*   180 */   -15,  723, 6465,  724,  -15, 6465,  738,  -15,  147,  -15,
 /*   190 */  6465,  -15,  -15,  803,  -15, 6465,  686,  804,  -15, 6465,
 /*   200 */  1568, 6465, 1568,  807,  -15, 6465,  591,  825,   51,  826,
 /*   210 */   821,  827, 1495,  832,  838, 1591,  839,  -15,  233,  -15,
 /*   220 */   -15,  835, 6465,  914,  920, 1591,  -15,  919, 6465,  922,
 /*   230 */   925, 1591,  928,  -15, 6465,  -15, 6277, 6465,  -15, 6465,
 /*   240 */   -15, 6465,  -15, 6465,  -15, 6465,  -15, 6465,  -15, 6465,
 /*   250 */   -15, 6465,  -15, 6465,  -15, 6465,  -15, 6465,  -15, 6465,
 /*   260 */   -15, 6465,  -15, 6465,  -15, 6465,  -15, 6465,  -15, 6465,
 /*   270 */   -15,  911, 6465,  912,  916, 6465,  -15, 1534, 6279,  -15,
 /*   280 */   -15, 6465,  -15, 6465,  -15, 6465,  -15, 6465,  -15, 6465,
 /*   290 */   -15, 6465,  -15, 6465,  -15, 6465,  -15, 6465,  -15, 6465,
 /*   300 */   -15, 6465,  -15, 6465,  -15, 6465,  -15, 6465,  -15, 6465,
 /*   310 */   -15, 6465,  -15,  917,  176,  719, 6520,  -15,  -15,  923,
 /*   320 */   -14,  -15,  719,  -15,  -15,  -15,  719,  220, 6378,  927,
 /*   330 */   926,  -15,  -15,  -15,  -15, 6378, 1007, 1006,  -15, 6465,
 /*   340 */  1011, 1013,  -15, 1613, 1017,  176,  719,  719, 6378, 1018,
 /*   350 */   176, 1017, 6378, 1019,  176, 1017, 6465, 1020,  176, 1017,
 /*   360 */  1733, 1021, 6465, 1022, 6465,  -15,  -15, 1023, 6465, 1022,
 /*   370 */   339, 6465, 1022, 6378, 1024, 1860, 1103, 6465, 1022, 1104,
 /*   380 */  6465, 1022,  341, 6465, 1022,  176,  351, 1107, 6465, 1109,
 /*   390 */  6465,  -15,  -15,  176,  719,  719, 6378, 1111, 1897, 1113,
 /*   400 */  6465, 1022, 1114, 6465, 1022,  431, 6465, 1022,  176,  435,
 /*   410 */  1115, 6465, 1109, 6465, 1116, 2017, 1117, 6465, 1022, 1118,
 /*   420 */  6465, 1022,  437, 6465, 1022,  176,  444, 1119, 6465, 1109,
 /*   430 */   176,  446, 1120, 6465, 1109, 2566, 1199, 6465, 1200, 6465,
 /*   440 */   -15,  -15, 1203, 6465, 1200,  527, 6465, 1200, 6378, 1205,
 /*   450 */  2054, 1207, 6465, 1200, 1209, 6465, 1200,  528, 6465, 1200,
 /*   460 */   176,  540, 1210, 6465, 1211, 6465,  -15,  -15,  176,  719,
 /*   470 */   719, 6378, 1212, 2174, 1213, 6465, 1200, 1214, 6465, 1200,
 /*   480 */   531, 6465, 1200,  176,  544, 1215, 6465, 1211, 6465, 1216,
 /*   490 */  2368, 1295, 6465, 1200, 1296, 6465, 1200,  623, 6465, 1200,
 /*   500 */   176,  627, 1299, 6465, 1211,  176,  636, 1301, 6465, 1211,
 /*   510 */   313, 6378,  -15, 6378,  -15, 6465,  -15,  -15, 6245,  -15,
 /*   520 */  6378, 1303, 6465,  -15, 6378, 1305, 6465,  -15, 6465, 1306,
 /*   530 */  6465,  -15, 6465,  -15, 6332,  -15, 6378, 1307, 6465,  -15,
 /*   540 */  6378, 1308, 6465,  -15, 6465, 1309, 6465,  -15, 6465,  -15,
 /*   550 */  1317,  -15, 1319, 1591, 1398,  -15, 1016, 1320, 1591, 1399,
 /*   560 */   -15,  338, 1403, 1591, 1404,  -15,  -15, 1395,  638, 1402,
 /*   570 */   821, 1405, 1563, 1407, 1413, 1591, 1414,  -15,  722, 1416,
 /*   580 */  1591, 1494,  -15, 1484,  -15,  -15,  -15,
};
#define YY_REDUCE_USE_DFLT (-98)
static short yy_reduce_ofst[] = {
 /*     0 */   -97,  -98,  -98,   -1,   95,  -98,  -98,  -98,  -98,  -98,
 /*    10 */   -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,
 /*    20 */   -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,
 /*    30 */   -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,
 /*    40 */   -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,
 /*    50 */   -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,  -98,
 /*    60 */   -98,  -98,  -98,  -98,  191,  -98,  -98,  -98,  287,  -98,
 /*    70 */   -98,  -98,  383,  -98,  -98,  -98, 3393,  -98,  -98,  479,
 /*    80 */   -98,  -98,  -98,  575,  -98,  -98,  -98, 3434,  -98,  -98,
 /*    90 */   -98,  -98, 6059,  -98, 6094,  -98,  -98, 6402,  -98, 6437,
 /*   100 */   -98, 6472,  -98, 6595,  -98, 6712,  -98, 6747,  -98, 6330,
 /*   110 */   -98, 6760,  -98, 6812,  -98, 1539,  -98,  -98, 6865,  -98,
 /*   120 */  6882,  -98, 6900,  -98, 6920,  -98,  -98, 1624,  -98,  -98,
 /*   130 */   -98,  -98,  -98,  -98, 3473,  -98,  -98, 6507,  -98, 6630,
 /*   140 */   -98,  -98,  -98,  -98, 6795,  -98, 6830,  -98, 6935,  -98,
 /*   150 */   -98, 3512,  -98, 6665,  -98,  -98, 6847,  -98, 6955,  -98,
 /*   160 */   -98,  -98,  -98,  -98, 3550,  -98,  -98, 6972,  -98,  -98,
 /*   170 */   -98,  -98,  -98,  -98,  -98,  -98, 1531,  -98,  -98,  -98,
 /*   180 */   -98,  -98, 3591,  -98,  -98, 3630,  -98,  -98,  -98,  -98,
 /*   190 */  3669,  -98,  -98,  -98,  -98, 6700,  -98,  -98,  -98, 6547,
 /*   200 */   -98, 6582,  -98,  -98,  -98, 6320,  -98,  -98,  679,  -98,
 /*   210 */   683,  -98, 1512,  -98,  -98,  671,  -98,  -98,   97,  -98,
 /*   220 */   -98,  -98, 3707,  -98,  -98,  767,  -98,  -98, 3748,  -98,
 /*   230 */   -98,  863,  -98,  -98, 3787,  -98,  -98, 3826,  -98, 3864,
 /*   240 */   -98, 3905,  -98, 3944,  -98, 3983,  -98, 4021,  -98, 4062,
 /*   250 */   -98, 4101,  -98, 4140,  -98, 4178,  -98, 4219,  -98, 4258,
 /*   260 */   -98, 4297,  -98, 4335,  -98, 4376,  -98, 4415,  -98, 4454,
 /*   270 */   -98,  -98, 4492,  -98,  -98, 4533,  -98,  -98, 4572,  -98,
 /*   280 */   -98, 4611,  -98, 4649,  -98, 4690,  -98, 4729,  -98, 4768,
 /*   290 */   -98, 4806,  -98, 4847,  -98, 4886,  -98, 4925,  -98, 4963,
 /*   300 */   -98, 5004,  -98, 5043,  -98, 5082,  -98, 5120,  -98, 5161,
 /*   310 */   -98, 5200,  -98,  -98, 1672,  -98, 1498,  -98,  -98,  -98,
 /*   320 */  1628,  -98,  -98,  -98,  -98,  -98,  -98,  -98, 1579,  -98,
 /*   330 */   -98,  -98,  -98,  -98,  -98, 1618,  -98,  -98,  -98, 5239,
 /*   340 */   -98,  -98,  -98, 1387,  -98, 1677,  -98,  -98, 1666,  -98,
 /*   350 */  1585,  -98, 1707,  -98, 1711,  -98, 5277,  -98, 1791,  -98,
 /*   360 */  1678,  -98, 1746,  -98, 5318,  -98,  -98,  -98, 1785,  -98,
 /*   370 */   -98, 1823,  -98, 1864,  -98, 1718,  -98, 1903,  -98,  -98,
 /*   380 */  1942,  -98,  -98, 1980,  -98, 6191,  -98,  -98, 2021,  -98,
 /*   390 */  5357,  -98,  -98, 1719,  -98,  -98, 2060,  -98, 1721,  -98,
 /*   400 */  2099,  -98,  -98, 2137,  -98,  -98, 2178,  -98, 6238,  -98,
 /*   410 */   -98, 2217,  -98, 5396,  -98, 1755,  -98, 2256,  -98,  -98,
 /*   420 */  2294,  -98,  -98, 2335,  -98, 6239,  -98,  -98, 2374,  -98,
 /*   430 */  6240,  -98,  -98, 2413,  -98, 1796,  -98, 2451,  -98, 5434,
 /*   440 */   -98,  -98,  -98, 2492,  -98,  -98, 2531,  -98, 2570,  -98,
 /*   450 */  1798,  -98, 2608,  -98,  -98, 2649,  -98,  -98, 2688,  -98,
 /*   460 */  1442,  -98,  -98, 2727,  -98, 5475,  -98,  -98, 1831,  -98,
 /*   470 */   -98, 2765,  -98, 1832,  -98, 2806,  -98,  -98, 2845,  -98,
 /*   480 */   -98, 2884,  -98, 6169,  -98,  -98, 2922,  -98, 5514,  -98,
 /*   490 */  1834,  -98, 2963,  -98,  -98, 3002,  -98,  -98, 3041,  -98,
 /*   500 */  6285,  -98,  -98, 3079,  -98, 6288,  -98,  -98, 3120,  -98,
 /*   510 */   -98, 3159,  -98, 3198,  -98, 5553,  -98,  -98, 5591,  -98,
 /*   520 */  3236,  -98, 5632,  -98, 3277,  -98, 5671,  -98, 5710,  -98,
 /*   530 */  5748,  -98, 5789,  -98, 5828,  -98, 3316,  -98, 5867,  -98,
 /*   540 */  3355,  -98, 5905,  -98, 5946,  -98, 5985,  -98, 6024,  -98,
 /*   550 */   -98,  -98,  -98,  959,  -98,  -98,  -98,  -98, 1055,  -98,
 /*   560 */   -98,  -98,  -98, 1151,  -98,  -98,  -98, 1256,  -98,  -98,
 /*   570 */  1259,  -98, 1837,  -98,  -98, 1247,  -98,  -98,  -98,  -98,
 /*   580 */  1343,  -98,  -98,  -98,  -98,  -98,  -98,
};
static YYACTIONTYPE yy_default[] = {
 /*     0 */   592,  894,  588,  592,  589,  591,  593,  594,  595,  596,
 /*    10 */   597,  598,  599,  600,  601,  602,  603,  604,  605,  606,
 /*    20 */   607,  608,  609,  610,  611,  612,  613,  614,  615,  616,
 /*    30 */   617,  618,  619,  620,  621,  622,  623,  624,  625,  626,
 /*    40 */   627,  628,  629,  630,  631,  632,  633,  634,  635,  636,
 /*    50 */   637,  638,  639,  640,  641,  642,  643,  644,  645,  646,
 /*    60 */   647,  648,  649,  894,  592,  894,  650,  894,  592,  894,
 /*    70 */   651,  670,  592,  894,  652,  894,  894,  894,  894,  592,
 /*    80 */   894,  653,  894,  592,  894,  894,  894,  894,  894,  654,
 /*    90 */   823,  825,  894,  894,  894,  824,  826,  894,  829,  894,
 /*   100 */   831,  894,  833,  894,  838,  894,  843,  894,  845,  894,
 /*   110 */   847,  894,  849,  894,  852,  894,  856,  860,  894,  861,
 /*   120 */   894,  862,  894,  863,  894,  864,  865,  894,  866,  867,
 /*   130 */   868,  869,  870,  894,  894,  828,  830,  894,  834,  894,
 /*   140 */   839,  842,  844,  846,  894,  850,  894,  853,  894,  857,
 /*   150 */   872,  894,  832,  894,  840,  848,  894,  854,  894,  858,
 /*   160 */   873,  875,  876,  894,  894,  837,  851,  894,  859,  878,
 /*   170 */   879,  880,  881,  882,  883,  884,  891,  855,  885,  886,
 /*   180 */   887,  894,  894,  894,  888,  894,  894,  889,  894,  890,
 /*   190 */   894,  893,  892,  894,  877,  894,  841,  894,  874,  894,
 /*   200 */   835,  894,  836,  894,  871,  894,  827,  894,  894,  894,
 /*   210 */   894,  894,  894,  894,  894,  592,  894,  655,  663,  664,
 /*   220 */   666,  894,  894,  894,  894,  592,  665,  894,  894,  894,
 /*   230 */   894,  592,  894,  667,  894,  671,  867,  894,  672,  894,
 /*   240 */   677,  894,  678,  894,  679,  894,  680,  894,  681,  894,
 /*   250 */   682,  894,  683,  894,  684,  894,  685,  894,  686,  894,
 /*   260 */   687,  894,  688,  894,  689,  894,  690,  894,  691,  894,
 /*   270 */   692,  894,  894,  894,  871,  894,  673,  872,  894,  674,
 /*   280 */   675,  894,  693,  894,  694,  894,  695,  894,  696,  894,
 /*   290 */   697,  894,  698,  894,  699,  894,  700,  894,  701,  894,
 /*   300 */   702,  894,  703,  894,  704,  894,  705,  894,  706,  894,
 /*   310 */   707,  894,  708,  894,  894,  676,  712,  709,  710,  711,
 /*   320 */   894,  717,  718,  713,  714,  715,  716,  894,  894,  894,
 /*   330 */   894,  719,  820,  821,  822,  894,  894,  894,  720,  894,
 /*   340 */   894,  894,  721,  723,  722,  894,  725,  724,  894,  894,
 /*   350 */   723,  726,  894,  894,  723,  727,  894,  894,  723,  728,
 /*   360 */   894,  729,  894,  731,  894,  754,  753,  730,  894,  732,
 /*   370 */   733,  894,  734,  894,  894,  894,  735,  894,  736,  737,
 /*   380 */   894,  738,  739,  894,  740,  894,  894,  783,  894,  784,
 /*   390 */   894,  792,  791,  894,  790,  789,  894,  894,  894,  741,
 /*   400 */   894,  742,  743,  894,  744,  745,  894,  746,  894,  894,
 /*   410 */   785,  894,  786,  894,  894,  894,  747,  894,  748,  749,
 /*   420 */   894,  750,  751,  894,  752,  894,  894,  787,  894,  788,
 /*   430 */   894,  894,  781,  894,  782,  894,  755,  894,  757,  894,
 /*   440 */   780,  779,  756,  894,  758,  759,  894,  760,  894,  894,
 /*   450 */   894,  761,  894,  762,  763,  894,  764,  765,  894,  766,
 /*   460 */   894,  894,  795,  894,  796,  894,  804,  803,  894,  802,
 /*   470 */   801,  894,  894,  894,  767,  894,  768,  769,  894,  770,
 /*   480 */   771,  894,  772,  894,  894,  797,  894,  798,  894,  894,
 /*   490 */   894,  773,  894,  774,  775,  894,  776,  777,  894,  778,
 /*   500 */   894,  894,  799,  894,  800,  894,  894,  793,  894,  794,
 /*   510 */   805,  894,  807,  894,  808,  894,  809,  806,  894,  810,
 /*   520 */   894,  894,  894,  812,  894,  894,  894,  813,  894,  894,
 /*   530 */   894,  814,  894,  811,  894,  815,  894,  894,  894,  817,
 /*   540 */   894,  894,  894,  818,  894,  894,  894,  819,  894,  816,
 /*   550 */   894,  668,  894,  592,  894,  669,  894,  894,  592,  894,
 /*   560 */   656,  894,  894,  592,  894,  658,  660,  894,  894,  894,
 /*   570 */   894,  894,  894,  894,  894,  592,  894,  657,  894,  894,
 /*   580 */   592,  894,  659,  894,  662,  661,  590,
};
#define YY_SZ_ACTTAB (sizeof(yy_action)/sizeof(yy_action[0]))

/* The next table maps tokens into fallback tokens.  If a construct
** like the following:
**
**      %fallback ID X Y Z.
**
** appears in the grammar, then ID becomes a fallback token for X, Y,
** and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
** but it does not parse, the type of the token is changed to ID and
** the parse is retried before an error is thrown.
*/
#ifdef YYFALLBACK
static const YYCODETYPE yyFallback[] = {
};
#endif /* YYFALLBACK */

/* The following structure represents a single element of the
** parser's stack.  Information stored includes:
**
**   +  The state number for the parser at this level of the stack.
**
**   +  The value of the token stored at this level of the stack.
**      (In other words, the "major" token.)
**
**   +  The semantic value stored at this level of the stack.  This is
**      the information used by the action routines in the grammar.
**      It is sometimes called the "minor" token.
*/
struct yyStackEntry {
	int stateno;       /* The state-number */
	int major;         /* The major token value.  This is the code
										 ** number for the token at this stack level */
	YYMINORTYPE minor; /* The user-supplied minor token value.  This
										 ** is the value of the token  */
};
typedef struct yyStackEntry yyStackEntry;

/* The state of the parser is completely contained in an instance of
** the following structure. */
struct _lemon_parser_t {
	int yyidx;                    /* Index of top element in stack */
	int yyerrcnt;                 /* Shifts left before out of the error */
	ParseARG_SDECL                /* A place to hold %extra_argument */
	yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
};
typedef struct _lemon_parser_t lemon_parser_t;

#ifndef NDEBUG
#include <stdio.h>
static FILE *yyTraceFILE   = 0;
static char *yyTracePrompt = 0;
#endif /* NDEBUG */

#ifndef NDEBUG
/*
** Turn parser tracing on by giving a stream to which to write the trace
** and a prompt to preface each trace message.  Tracing is turned off
** by making either argument NULL
**
** Inputs:
** <ul>
** <li> A FILE* to which trace output should be written.
**      If NULL, then tracing is turned off.
** <li> A prefix string written at the beginning of every
**      line of trace output.  If NULL, then tracing is
**      turned off.
** </ul>
**
** Outputs:
** None.
*/
void mlr_dsl_ParseTrace(FILE *TraceFILE, char *zTracePrompt) {
	yyTraceFILE = TraceFILE;
	yyTracePrompt = zTracePrompt;
	if (yyTraceFILE==0)
		yyTracePrompt = 0;
	else if (yyTracePrompt==0)
		yyTraceFILE = 0;
}
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing shifts, the names of all terminals and nonterminals
** are required.  The following table supplies these names */
static const char *yyTokenName[] = {
  "$",             "MD_TOKEN_SEMICOLON",  "MD_TOKEN_BREAK",  "MD_TOKEN_CONTINUE",
  "MD_TOKEN_BEGIN",  "MD_TOKEN_LBRACE",  "MD_TOKEN_RBRACE",  "MD_TOKEN_END",
  "MD_TOKEN_WHILE",  "MD_TOKEN_LPAREN",  "MD_TOKEN_RPAREN",  "MD_TOKEN_DO", 
  "MD_TOKEN_FOR",  "MD_TOKEN_COMMA",  "MD_TOKEN_IN",   "MD_TOKEN_FULL_SREC",
  "MD_TOKEN_FULL_OOSVAR",  "MD_TOKEN_NON_SIGIL_NAME",  "MD_TOKEN_IF",   "MD_TOKEN_ELIF",
  "MD_TOKEN_ELSE",  "MD_TOKEN_FILTER",  "MD_TOKEN_ASSIGN",  "MD_TOKEN_DOLLAR_SIGN",
  "MD_TOKEN_LEFT_BRACKET",  "MD_TOKEN_RIGHT_BRACKET",  "MD_TOKEN_LOGICAL_OR_EQUALS",  "MD_TOKEN_LOGICAL_XOR_EQUALS",
  "MD_TOKEN_LOGICAL_AND_EQUALS",  "MD_TOKEN_BITWISE_OR_EQUALS",  "MD_TOKEN_BITWISE_XOR_EQUALS",  "MD_TOKEN_BITWISE_AND_EQUALS",
  "MD_TOKEN_BITWISE_LSH_EQUALS",  "MD_TOKEN_BITWISE_RSH_EQUALS",  "MD_TOKEN_PLUS_EQUALS",  "MD_TOKEN_MINUS_EQUALS",
  "MD_TOKEN_DOT_EQUALS",  "MD_TOKEN_TIMES_EQUALS",  "MD_TOKEN_DIVIDE_EQUALS",  "MD_TOKEN_INT_DIVIDE_EQUALS",
  "MD_TOKEN_MOD_EQUALS",  "MD_TOKEN_POW_EQUALS",  "MD_TOKEN_UNSET",  "MD_TOKEN_ALL",
  "MD_TOKEN_TEE",  "MD_TOKEN_GT",   "MD_TOKEN_BITWISE_RSH",  "MD_TOKEN_BITWISE_OR",
  "MD_TOKEN_EMITF",  "MD_TOKEN_EMITP",  "MD_TOKEN_EMIT",  "MD_TOKEN_DUMP",
  "MD_TOKEN_EDUMP",  "MD_TOKEN_PRINT",  "MD_TOKEN_EPRINT",  "MD_TOKEN_PRINTN",
  "MD_TOKEN_EPRINTN",  "MD_TOKEN_STDOUT",  "MD_TOKEN_STDERR",  "MD_TOKEN_QUESTION_MARK",
  "MD_TOKEN_COLON",  "MD_TOKEN_LOGICAL_OR",  "MD_TOKEN_LOGICAL_XOR",  "MD_TOKEN_LOGICAL_AND",
  "MD_TOKEN_MATCHES",  "MD_TOKEN_DOES_NOT_MATCH",  "MD_TOKEN_EQ",   "MD_TOKEN_NE", 
  "MD_TOKEN_GE",   "MD_TOKEN_LT",   "MD_TOKEN_LE",   "MD_TOKEN_BITWISE_XOR",
  "MD_TOKEN_BITWISE_AND",  "MD_TOKEN_BITWISE_LSH",  "MD_TOKEN_PLUS",  "MD_TOKEN_MINUS",
  "MD_TOKEN_DOT",  "MD_TOKEN_TIMES",  "MD_TOKEN_DIVIDE",  "MD_TOKEN_INT_DIVIDE",
  "MD_TOKEN_MOD",  "MD_TOKEN_LOGICAL_NOT",  "MD_TOKEN_BITWISE_NOT",  "MD_TOKEN_POW",
  "MD_TOKEN_FIELD_NAME",  "MD_TOKEN_BRACED_FIELD_NAME",  "MD_TOKEN_UNBRACED_OOSVAR_NAME",  "MD_TOKEN_BRACED_OOSVAR_NAME",
  "MD_TOKEN_AT_SIGN",  "MD_TOKEN_NUMBER",  "MD_TOKEN_TRUE",  "MD_TOKEN_FALSE",
  "MD_TOKEN_STRING",  "MD_TOKEN_REGEXI",  "MD_TOKEN_CONTEXT_VARIABLE",  "MD_TOKEN_ENV",
  "error",         "md_body",       "md_statement_list",  "md_statement_not_braced_end",
  "md_statement_braced_end",  "md_begin_block",  "md_end_block",  "md_cond_block",
  "md_while_block",  "md_for_loop_full_srec",  "md_for_loop_full_oosvar",  "md_for_loop_oosvar",
  "md_if_chain",   "md_srec_assignment",  "md_srec_indirect_assignment",  "md_oosvar_from_full_srec_assignment",
  "md_full_srec_from_oosvar_assignment",  "md_do_while_block",  "md_bare_boolean",  "md_oosvar_assignment",
  "md_filter",     "md_unset",      "md_tee_write",  "md_tee_append",
  "md_tee_pipe",   "md_emitf",      "md_emitf_write",  "md_emitf_append",
  "md_emitf_pipe",  "md_emitp",      "md_emitp_write",  "md_emitp_append",
  "md_emitp_pipe",  "md_emit",       "md_emit_write",  "md_emit_append",
  "md_emit_pipe",  "md_emitp_lashed",  "md_emitp_lashed_write",  "md_emitp_lashed_append",
  "md_emitp_lashed_pipe",  "md_emit_lashed",  "md_emit_lashed_write",  "md_emit_lashed_append",
  "md_emit_lashed_pipe",  "md_dump",       "md_dump_write",  "md_dump_append",
  "md_dump_pipe",  "md_edump",      "md_print",      "md_eprint",   
  "md_print_write",  "md_print_append",  "md_print_pipe",  "md_printn",   
  "md_eprintn",    "md_printn_write",  "md_printn_append",  "md_printn_pipe",
  "md_rhs",        "md_for_loop_index",  "md_for_oosvar_keylist",  "md_oosvar_keylist",
  "md_if_elif_star",  "md_else_block",  "md_if_block",   "md_elif_block",
  "md_field_name",  "md_unset_args",  "md_indirect_field_name",  "md_output_file",
  "md_emitf_args",  "md_emitp_namelist",  "md_emit_namelist",  "md_emitp_lashed_keylists",
  "md_emitp_lashed_namelist",  "md_emit_lashed_keylists",  "md_emit_lashed_namelist",  "md_ternary",  
  "md_logical_or_term",  "md_logical_xor_term",  "md_logical_and_term",  "md_eqne_term",
  "md_cmp_term",   "md_bitwise_or_term",  "md_bitwise_xor_term",  "md_bitwise_and_term",
  "md_bitwise_shift_term",  "md_addsubdot_term",  "md_muldiv_term",  "md_unary_bitwise_op_term",
  "md_pow_term",   "md_atom_or_fcn",  "md_oosvar_basename",  "md_string",   
  "md_regexi",     "md_bound_variable",  "md_fcn_args", 
};
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing reduce actions, the names of all rules are required.
*/
static const char *yyRuleName[] = {
 /*   0 */ "md_body ::= md_statement_list",
 /*   1 */ "md_statement_list ::= md_statement_not_braced_end",
 /*   2 */ "md_statement_list ::= md_statement_braced_end",
 /*   3 */ "md_statement_list ::= md_statement_not_braced_end MD_TOKEN_SEMICOLON md_statement_list",
 /*   4 */ "md_statement_list ::= md_statement_braced_end md_statement_list",
 /*   5 */ "md_statement_not_braced_end ::=",
 /*   6 */ "md_statement_braced_end ::= md_begin_block",
 /*   7 */ "md_statement_braced_end ::= md_end_block",
 /*   8 */ "md_statement_braced_end ::= md_cond_block",
 /*   9 */ "md_statement_braced_end ::= md_while_block",
 /*  10 */ "md_statement_braced_end ::= md_for_loop_full_srec",
 /*  11 */ "md_statement_braced_end ::= md_for_loop_full_oosvar",
 /*  12 */ "md_statement_braced_end ::= md_for_loop_oosvar",
 /*  13 */ "md_statement_braced_end ::= md_if_chain",
 /*  14 */ "md_statement_not_braced_end ::= md_srec_assignment",
 /*  15 */ "md_statement_not_braced_end ::= md_srec_indirect_assignment",
 /*  16 */ "md_statement_not_braced_end ::= md_oosvar_from_full_srec_assignment",
 /*  17 */ "md_statement_not_braced_end ::= md_full_srec_from_oosvar_assignment",
 /*  18 */ "md_statement_not_braced_end ::= md_do_while_block",
 /*  19 */ "md_statement_not_braced_end ::= md_bare_boolean",
 /*  20 */ "md_statement_not_braced_end ::= md_oosvar_assignment",
 /*  21 */ "md_statement_not_braced_end ::= md_filter",
 /*  22 */ "md_statement_not_braced_end ::= md_unset",
 /*  23 */ "md_statement_not_braced_end ::= md_tee_write",
 /*  24 */ "md_statement_not_braced_end ::= md_tee_append",
 /*  25 */ "md_statement_not_braced_end ::= md_tee_pipe",
 /*  26 */ "md_statement_not_braced_end ::= md_emitf",
 /*  27 */ "md_statement_not_braced_end ::= md_emitf_write",
 /*  28 */ "md_statement_not_braced_end ::= md_emitf_append",
 /*  29 */ "md_statement_not_braced_end ::= md_emitf_pipe",
 /*  30 */ "md_statement_not_braced_end ::= md_emitp",
 /*  31 */ "md_statement_not_braced_end ::= md_emitp_write",
 /*  32 */ "md_statement_not_braced_end ::= md_emitp_append",
 /*  33 */ "md_statement_not_braced_end ::= md_emitp_pipe",
 /*  34 */ "md_statement_not_braced_end ::= md_emit",
 /*  35 */ "md_statement_not_braced_end ::= md_emit_write",
 /*  36 */ "md_statement_not_braced_end ::= md_emit_append",
 /*  37 */ "md_statement_not_braced_end ::= md_emit_pipe",
 /*  38 */ "md_statement_not_braced_end ::= md_emitp_lashed",
 /*  39 */ "md_statement_not_braced_end ::= md_emitp_lashed_write",
 /*  40 */ "md_statement_not_braced_end ::= md_emitp_lashed_append",
 /*  41 */ "md_statement_not_braced_end ::= md_emitp_lashed_pipe",
 /*  42 */ "md_statement_not_braced_end ::= md_emit_lashed",
 /*  43 */ "md_statement_not_braced_end ::= md_emit_lashed_write",
 /*  44 */ "md_statement_not_braced_end ::= md_emit_lashed_append",
 /*  45 */ "md_statement_not_braced_end ::= md_emit_lashed_pipe",
 /*  46 */ "md_statement_not_braced_end ::= md_dump",
 /*  47 */ "md_statement_not_braced_end ::= md_dump_write",
 /*  48 */ "md_statement_not_braced_end ::= md_dump_append",
 /*  49 */ "md_statement_not_braced_end ::= md_dump_pipe",
 /*  50 */ "md_statement_not_braced_end ::= md_edump",
 /*  51 */ "md_statement_not_braced_end ::= md_print",
 /*  52 */ "md_statement_not_braced_end ::= md_eprint",
 /*  53 */ "md_statement_not_braced_end ::= md_print_write",
 /*  54 */ "md_statement_not_braced_end ::= md_print_append",
 /*  55 */ "md_statement_not_braced_end ::= md_print_pipe",
 /*  56 */ "md_statement_not_braced_end ::= md_printn",
 /*  57 */ "md_statement_not_braced_end ::= md_eprintn",
 /*  58 */ "md_statement_not_braced_end ::= md_printn_write",
 /*  59 */ "md_statement_not_braced_end ::= md_printn_append",
 /*  60 */ "md_statement_not_braced_end ::= md_printn_pipe",
 /*  61 */ "md_statement_not_braced_end ::= MD_TOKEN_BREAK",
 /*  62 */ "md_statement_not_braced_end ::= MD_TOKEN_CONTINUE",
 /*  63 */ "md_begin_block ::= MD_TOKEN_BEGIN MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE",
 /*  64 */ "md_end_block ::= MD_TOKEN_END MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE",
 /*  65 */ "md_cond_block ::= md_rhs MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE",
 /*  66 */ "md_while_block ::= MD_TOKEN_WHILE MD_TOKEN_LPAREN md_rhs MD_TOKEN_RPAREN MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE",
 /*  67 */ "md_do_while_block ::= MD_TOKEN_DO MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE MD_TOKEN_WHILE MD_TOKEN_LPAREN md_rhs MD_TOKEN_RPAREN",
 /*  68 */ "md_for_loop_full_srec ::= MD_TOKEN_FOR MD_TOKEN_LPAREN md_for_loop_index MD_TOKEN_COMMA md_for_loop_index MD_TOKEN_IN MD_TOKEN_FULL_SREC MD_TOKEN_RPAREN MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE",
 /*  69 */ "md_for_loop_full_oosvar ::= MD_TOKEN_FOR MD_TOKEN_LPAREN md_for_loop_index MD_TOKEN_COMMA md_for_loop_index MD_TOKEN_IN MD_TOKEN_FULL_OOSVAR MD_TOKEN_RPAREN MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE",
 /*  70 */ "md_for_loop_full_oosvar ::= MD_TOKEN_FOR MD_TOKEN_LPAREN MD_TOKEN_LPAREN md_for_oosvar_keylist MD_TOKEN_RPAREN MD_TOKEN_COMMA md_for_loop_index MD_TOKEN_IN MD_TOKEN_FULL_OOSVAR MD_TOKEN_RPAREN MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE",
 /*  71 */ "md_for_loop_oosvar ::= MD_TOKEN_FOR MD_TOKEN_LPAREN md_for_loop_index MD_TOKEN_COMMA md_for_loop_index MD_TOKEN_IN md_oosvar_keylist MD_TOKEN_RPAREN MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE",
 /*  72 */ "md_for_loop_oosvar ::= MD_TOKEN_FOR MD_TOKEN_LPAREN MD_TOKEN_LPAREN md_for_oosvar_keylist MD_TOKEN_RPAREN MD_TOKEN_COMMA md_for_loop_index MD_TOKEN_IN md_oosvar_keylist MD_TOKEN_RPAREN MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE",
 /*  73 */ "md_for_loop_index ::= MD_TOKEN_NON_SIGIL_NAME",
 /*  74 */ "md_for_oosvar_keylist ::= MD_TOKEN_NON_SIGIL_NAME",
 /*  75 */ "md_for_oosvar_keylist ::= md_for_oosvar_keylist MD_TOKEN_COMMA MD_TOKEN_NON_SIGIL_NAME",
 /*  76 */ "md_if_chain ::= md_if_elif_star",
 /*  77 */ "md_if_chain ::= md_if_elif_star md_else_block",
 /*  78 */ "md_if_elif_star ::= md_if_block",
 /*  79 */ "md_if_elif_star ::= md_if_elif_star md_elif_block",
 /*  80 */ "md_if_block ::= MD_TOKEN_IF MD_TOKEN_LPAREN md_rhs MD_TOKEN_RPAREN MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE",
 /*  81 */ "md_elif_block ::= MD_TOKEN_ELIF MD_TOKEN_LPAREN md_rhs MD_TOKEN_RPAREN MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE",
 /*  82 */ "md_else_block ::= MD_TOKEN_ELSE MD_TOKEN_LBRACE md_statement_list MD_TOKEN_RBRACE",
 /*  83 */ "md_bare_boolean ::= md_rhs",
 /*  84 */ "md_filter ::= MD_TOKEN_FILTER md_rhs",
 /*  85 */ "md_srec_assignment ::= md_field_name MD_TOKEN_ASSIGN md_rhs",
 /*  86 */ "md_srec_indirect_assignment ::= MD_TOKEN_DOLLAR_SIGN MD_TOKEN_LEFT_BRACKET md_rhs MD_TOKEN_RIGHT_BRACKET MD_TOKEN_ASSIGN md_rhs",
 /*  87 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_ASSIGN md_rhs",
 /*  88 */ "md_oosvar_from_full_srec_assignment ::= md_oosvar_keylist MD_TOKEN_ASSIGN MD_TOKEN_FULL_SREC",
 /*  89 */ "md_full_srec_from_oosvar_assignment ::= MD_TOKEN_FULL_SREC MD_TOKEN_ASSIGN md_oosvar_keylist",
 /*  90 */ "md_srec_assignment ::= md_field_name MD_TOKEN_LOGICAL_OR_EQUALS md_rhs",
 /*  91 */ "md_srec_assignment ::= md_field_name MD_TOKEN_LOGICAL_XOR_EQUALS md_rhs",
 /*  92 */ "md_srec_assignment ::= md_field_name MD_TOKEN_LOGICAL_AND_EQUALS md_rhs",
 /*  93 */ "md_srec_assignment ::= md_field_name MD_TOKEN_BITWISE_OR_EQUALS md_rhs",
 /*  94 */ "md_srec_assignment ::= md_field_name MD_TOKEN_BITWISE_XOR_EQUALS md_rhs",
 /*  95 */ "md_srec_assignment ::= md_field_name MD_TOKEN_BITWISE_AND_EQUALS md_rhs",
 /*  96 */ "md_srec_assignment ::= md_field_name MD_TOKEN_BITWISE_LSH_EQUALS md_rhs",
 /*  97 */ "md_srec_assignment ::= md_field_name MD_TOKEN_BITWISE_RSH_EQUALS md_rhs",
 /*  98 */ "md_srec_assignment ::= md_field_name MD_TOKEN_PLUS_EQUALS md_rhs",
 /*  99 */ "md_srec_assignment ::= md_field_name MD_TOKEN_MINUS_EQUALS md_rhs",
 /* 100 */ "md_srec_assignment ::= md_field_name MD_TOKEN_DOT_EQUALS md_rhs",
 /* 101 */ "md_srec_assignment ::= md_field_name MD_TOKEN_TIMES_EQUALS md_rhs",
 /* 102 */ "md_srec_assignment ::= md_field_name MD_TOKEN_DIVIDE_EQUALS md_rhs",
 /* 103 */ "md_srec_assignment ::= md_field_name MD_TOKEN_INT_DIVIDE_EQUALS md_rhs",
 /* 104 */ "md_srec_assignment ::= md_field_name MD_TOKEN_MOD_EQUALS md_rhs",
 /* 105 */ "md_srec_assignment ::= md_field_name MD_TOKEN_POW_EQUALS md_rhs",
 /* 106 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_LOGICAL_OR_EQUALS md_rhs",
 /* 107 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_LOGICAL_XOR_EQUALS md_rhs",
 /* 108 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_LOGICAL_AND_EQUALS md_rhs",
 /* 109 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_BITWISE_OR_EQUALS md_rhs",
 /* 110 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_BITWISE_XOR_EQUALS md_rhs",
 /* 111 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_BITWISE_AND_EQUALS md_rhs",
 /* 112 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_BITWISE_LSH_EQUALS md_rhs",
 /* 113 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_BITWISE_RSH_EQUALS md_rhs",
 /* 114 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_PLUS_EQUALS md_rhs",
 /* 115 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_MINUS_EQUALS md_rhs",
 /* 116 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_DOT_EQUALS md_rhs",
 /* 117 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_TIMES_EQUALS md_rhs",
 /* 118 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_DIVIDE_EQUALS md_rhs",
 /* 119 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_INT_DIVIDE_EQUALS md_rhs",
 /* 120 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_MOD_EQUALS md_rhs",
 /* 121 */ "md_oosvar_assignment ::= md_oosvar_keylist MD_TOKEN_POW_EQUALS md_rhs",
 /* 122 */ "md_unset ::= MD_TOKEN_UNSET MD_TOKEN_ALL",
 /* 123 */ "md_unset ::= MD_TOKEN_UNSET MD_TOKEN_FULL_OOSVAR",
 /* 124 */ "md_unset ::= MD_TOKEN_UNSET md_unset_args",
 /* 125 */ "md_unset_args ::=",
 /* 126 */ "md_unset_args ::= md_field_name",
 /* 127 */ "md_unset_args ::= md_indirect_field_name",
 /* 128 */ "md_unset_args ::= MD_TOKEN_FULL_SREC",
 /* 129 */ "md_unset_args ::= md_oosvar_keylist",
 /* 130 */ "md_unset_args ::= md_unset_args MD_TOKEN_COMMA md_field_name",
 /* 131 */ "md_unset_args ::= md_unset_args MD_TOKEN_COMMA md_oosvar_keylist",
 /* 132 */ "md_tee_write ::= MD_TOKEN_TEE MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_FULL_SREC",
 /* 133 */ "md_tee_append ::= MD_TOKEN_TEE MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_FULL_SREC",
 /* 134 */ "md_tee_pipe ::= MD_TOKEN_TEE MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_FULL_SREC",
 /* 135 */ "md_emitf ::= MD_TOKEN_EMITF md_emitf_args",
 /* 136 */ "md_emitf_args ::=",
 /* 137 */ "md_emitf_args ::= md_oosvar_keylist",
 /* 138 */ "md_emitf_args ::= md_emitf_args MD_TOKEN_COMMA md_oosvar_keylist",
 /* 139 */ "md_emitf_write ::= MD_TOKEN_EMITF MD_TOKEN_GT md_output_file MD_TOKEN_COMMA md_emitf_args",
 /* 140 */ "md_emitf_append ::= MD_TOKEN_EMITF MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA md_emitf_args",
 /* 141 */ "md_emitf_pipe ::= MD_TOKEN_EMITF MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA md_emitf_args",
 /* 142 */ "md_emitp ::= MD_TOKEN_EMITP MD_TOKEN_ALL",
 /* 143 */ "md_emitp ::= MD_TOKEN_EMITP MD_TOKEN_FULL_OOSVAR",
 /* 144 */ "md_emitp ::= MD_TOKEN_EMITP MD_TOKEN_ALL MD_TOKEN_COMMA md_emitp_namelist",
 /* 145 */ "md_emitp ::= MD_TOKEN_EMITP MD_TOKEN_FULL_OOSVAR MD_TOKEN_COMMA md_emitp_namelist",
 /* 146 */ "md_emitp ::= MD_TOKEN_EMITP md_oosvar_keylist",
 /* 147 */ "md_emitp ::= MD_TOKEN_EMITP md_oosvar_keylist MD_TOKEN_COMMA md_emitp_namelist",
 /* 148 */ "md_emitp_write ::= MD_TOKEN_EMITP MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_ALL",
 /* 149 */ "md_emitp_write ::= MD_TOKEN_EMITP MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_ALL MD_TOKEN_COMMA md_emitp_namelist",
 /* 150 */ "md_emitp_write ::= MD_TOKEN_EMITP MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_FULL_OOSVAR",
 /* 151 */ "md_emitp_write ::= MD_TOKEN_EMITP MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_FULL_OOSVAR MD_TOKEN_COMMA md_emitp_namelist",
 /* 152 */ "md_emitp_write ::= MD_TOKEN_EMITP MD_TOKEN_GT md_output_file MD_TOKEN_COMMA md_oosvar_keylist",
 /* 153 */ "md_emitp_write ::= MD_TOKEN_EMITP MD_TOKEN_GT md_output_file MD_TOKEN_COMMA md_oosvar_keylist MD_TOKEN_COMMA md_emitp_namelist",
 /* 154 */ "md_emitp_append ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_ALL",
 /* 155 */ "md_emitp_append ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_ALL MD_TOKEN_COMMA md_emitp_namelist",
 /* 156 */ "md_emitp_append ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_FULL_OOSVAR",
 /* 157 */ "md_emitp_append ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_FULL_OOSVAR MD_TOKEN_COMMA md_emitp_namelist",
 /* 158 */ "md_emitp_append ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA md_oosvar_keylist",
 /* 159 */ "md_emitp_append ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA md_oosvar_keylist MD_TOKEN_COMMA md_emitp_namelist",
 /* 160 */ "md_emitp_pipe ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_ALL",
 /* 161 */ "md_emitp_pipe ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_ALL MD_TOKEN_COMMA md_emitp_namelist",
 /* 162 */ "md_emitp_pipe ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_FULL_OOSVAR",
 /* 163 */ "md_emitp_pipe ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_FULL_OOSVAR MD_TOKEN_COMMA md_emitp_namelist",
 /* 164 */ "md_emitp_pipe ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA md_oosvar_keylist",
 /* 165 */ "md_emitp_pipe ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA md_oosvar_keylist MD_TOKEN_COMMA md_emitp_namelist",
 /* 166 */ "md_emitp_namelist ::= md_rhs",
 /* 167 */ "md_emitp_namelist ::= md_emitp_namelist MD_TOKEN_COMMA md_rhs",
 /* 168 */ "md_emit ::= MD_TOKEN_EMIT MD_TOKEN_ALL",
 /* 169 */ "md_emit ::= MD_TOKEN_EMIT MD_TOKEN_FULL_OOSVAR",
 /* 170 */ "md_emit ::= MD_TOKEN_EMIT MD_TOKEN_ALL MD_TOKEN_COMMA md_emit_namelist",
 /* 171 */ "md_emit ::= MD_TOKEN_EMIT MD_TOKEN_FULL_OOSVAR MD_TOKEN_COMMA md_emit_namelist",
 /* 172 */ "md_emit ::= MD_TOKEN_EMIT md_oosvar_keylist",
 /* 173 */ "md_emit ::= MD_TOKEN_EMIT md_oosvar_keylist MD_TOKEN_COMMA md_emit_namelist",
 /* 174 */ "md_emit_write ::= MD_TOKEN_EMIT MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_ALL",
 /* 175 */ "md_emit_write ::= MD_TOKEN_EMIT MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_ALL MD_TOKEN_COMMA md_emit_namelist",
 /* 176 */ "md_emit_write ::= MD_TOKEN_EMIT MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_FULL_OOSVAR",
 /* 177 */ "md_emit_write ::= MD_TOKEN_EMIT MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_FULL_OOSVAR MD_TOKEN_COMMA md_emit_namelist",
 /* 178 */ "md_emit_write ::= MD_TOKEN_EMIT MD_TOKEN_GT md_output_file MD_TOKEN_COMMA md_oosvar_keylist",
 /* 179 */ "md_emit_write ::= MD_TOKEN_EMIT MD_TOKEN_GT md_output_file MD_TOKEN_COMMA md_oosvar_keylist MD_TOKEN_COMMA md_emit_namelist",
 /* 180 */ "md_emit_append ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_ALL",
 /* 181 */ "md_emit_append ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_ALL MD_TOKEN_COMMA md_emit_namelist",
 /* 182 */ "md_emit_append ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_FULL_OOSVAR",
 /* 183 */ "md_emit_append ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_FULL_OOSVAR MD_TOKEN_COMMA md_emit_namelist",
 /* 184 */ "md_emit_append ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA md_oosvar_keylist",
 /* 185 */ "md_emit_append ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA md_oosvar_keylist MD_TOKEN_COMMA md_emit_namelist",
 /* 186 */ "md_emit_pipe ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_ALL",
 /* 187 */ "md_emit_pipe ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_ALL MD_TOKEN_COMMA md_emit_namelist",
 /* 188 */ "md_emit_pipe ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_FULL_OOSVAR",
 /* 189 */ "md_emit_pipe ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_FULL_OOSVAR MD_TOKEN_COMMA md_emit_namelist",
 /* 190 */ "md_emit_pipe ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA md_oosvar_keylist",
 /* 191 */ "md_emit_pipe ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA md_oosvar_keylist MD_TOKEN_COMMA md_emit_namelist",
 /* 192 */ "md_emit_namelist ::= md_rhs",
 /* 193 */ "md_emit_namelist ::= md_emit_namelist MD_TOKEN_COMMA md_rhs",
 /* 194 */ "md_emitp_lashed ::= MD_TOKEN_EMITP MD_TOKEN_LPAREN md_emitp_lashed_keylists MD_TOKEN_RPAREN",
 /* 195 */ "md_emitp_lashed ::= MD_TOKEN_EMITP MD_TOKEN_LPAREN md_emitp_lashed_keylists MD_TOKEN_RPAREN MD_TOKEN_COMMA md_emitp_lashed_namelist",
 /* 196 */ "md_emitp_lashed_write ::= MD_TOKEN_EMITP MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_LPAREN md_emitp_lashed_keylists MD_TOKEN_RPAREN",
 /* 197 */ "md_emitp_lashed_write ::= MD_TOKEN_EMITP MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_LPAREN md_emitp_lashed_keylists MD_TOKEN_RPAREN MD_TOKEN_COMMA md_emitp_lashed_namelist",
 /* 198 */ "md_emitp_lashed_append ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_LPAREN md_emitp_lashed_keylists MD_TOKEN_RPAREN",
 /* 199 */ "md_emitp_lashed_append ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_LPAREN md_emitp_lashed_keylists MD_TOKEN_RPAREN MD_TOKEN_COMMA md_emitp_lashed_namelist",
 /* 200 */ "md_emitp_lashed_pipe ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_LPAREN md_emitp_lashed_keylists MD_TOKEN_RPAREN",
 /* 201 */ "md_emitp_lashed_pipe ::= MD_TOKEN_EMITP MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_LPAREN md_emitp_lashed_keylists MD_TOKEN_RPAREN MD_TOKEN_COMMA md_emitp_lashed_namelist",
 /* 202 */ "md_emitp_lashed_keylists ::= md_oosvar_keylist",
 /* 203 */ "md_emitp_lashed_keylists ::= md_emitp_lashed_keylists MD_TOKEN_COMMA md_oosvar_keylist",
 /* 204 */ "md_emitp_lashed_namelist ::= md_rhs",
 /* 205 */ "md_emitp_lashed_namelist ::= md_emitp_lashed_namelist MD_TOKEN_COMMA md_rhs",
 /* 206 */ "md_emit_lashed ::= MD_TOKEN_EMIT MD_TOKEN_LPAREN md_emit_lashed_keylists MD_TOKEN_RPAREN",
 /* 207 */ "md_emit_lashed ::= MD_TOKEN_EMIT MD_TOKEN_LPAREN md_emit_lashed_keylists MD_TOKEN_RPAREN MD_TOKEN_COMMA md_emit_lashed_namelist",
 /* 208 */ "md_emit_lashed_write ::= MD_TOKEN_EMIT MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_LPAREN md_emit_lashed_keylists MD_TOKEN_RPAREN",
 /* 209 */ "md_emit_lashed_write ::= MD_TOKEN_EMIT MD_TOKEN_GT md_output_file MD_TOKEN_COMMA MD_TOKEN_LPAREN md_emit_lashed_keylists MD_TOKEN_RPAREN MD_TOKEN_COMMA md_emit_lashed_namelist",
 /* 210 */ "md_emit_lashed_append ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_LPAREN md_emit_lashed_keylists MD_TOKEN_RPAREN",
 /* 211 */ "md_emit_lashed_append ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA MD_TOKEN_LPAREN md_emit_lashed_keylists MD_TOKEN_RPAREN MD_TOKEN_COMMA md_emit_lashed_namelist",
 /* 212 */ "md_emit_lashed_pipe ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_LPAREN md_emit_lashed_keylists MD_TOKEN_RPAREN",
 /* 213 */ "md_emit_lashed_pipe ::= MD_TOKEN_EMIT MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA MD_TOKEN_LPAREN md_emit_lashed_keylists MD_TOKEN_RPAREN MD_TOKEN_COMMA md_emit_lashed_namelist",
 /* 214 */ "md_emit_lashed_keylists ::= md_oosvar_keylist",
 /* 215 */ "md_emit_lashed_keylists ::= md_emit_lashed_keylists MD_TOKEN_COMMA md_oosvar_keylist",
 /* 216 */ "md_emit_lashed_namelist ::= md_rhs",
 /* 217 */ "md_emit_lashed_namelist ::= md_emit_lashed_namelist MD_TOKEN_COMMA md_rhs",
 /* 218 */ "md_dump ::= MD_TOKEN_DUMP",
 /* 219 */ "md_edump ::= MD_TOKEN_EDUMP",
 /* 220 */ "md_dump_write ::= MD_TOKEN_DUMP MD_TOKEN_GT md_output_file",
 /* 221 */ "md_dump_append ::= MD_TOKEN_DUMP MD_TOKEN_BITWISE_RSH md_output_file",
 /* 222 */ "md_dump_pipe ::= MD_TOKEN_DUMP MD_TOKEN_BITWISE_OR md_rhs",
 /* 223 */ "md_print ::= MD_TOKEN_PRINT md_rhs",
 /* 224 */ "md_eprint ::= MD_TOKEN_EPRINT md_rhs",
 /* 225 */ "md_print_write ::= MD_TOKEN_PRINT MD_TOKEN_GT md_output_file MD_TOKEN_COMMA md_rhs",
 /* 226 */ "md_print_append ::= MD_TOKEN_PRINT MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA md_rhs",
 /* 227 */ "md_print_pipe ::= MD_TOKEN_PRINT MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA md_rhs",
 /* 228 */ "md_printn ::= MD_TOKEN_PRINTN md_rhs",
 /* 229 */ "md_eprintn ::= MD_TOKEN_EPRINTN md_rhs",
 /* 230 */ "md_printn_write ::= MD_TOKEN_PRINTN MD_TOKEN_GT md_output_file MD_TOKEN_COMMA md_rhs",
 /* 231 */ "md_printn_append ::= MD_TOKEN_PRINTN MD_TOKEN_BITWISE_RSH md_output_file MD_TOKEN_COMMA md_rhs",
 /* 232 */ "md_printn_pipe ::= MD_TOKEN_PRINTN MD_TOKEN_BITWISE_OR md_rhs MD_TOKEN_COMMA md_rhs",
 /* 233 */ "md_output_file ::= md_rhs",
 /* 234 */ "md_output_file ::= MD_TOKEN_STDOUT",
 /* 235 */ "md_output_file ::= MD_TOKEN_STDERR",
 /* 236 */ "md_rhs ::= md_ternary",
 /* 237 */ "md_ternary ::= md_logical_or_term MD_TOKEN_QUESTION_MARK md_ternary MD_TOKEN_COLON md_ternary",
 /* 238 */ "md_ternary ::= md_logical_or_term",
 /* 239 */ "md_logical_or_term ::= md_logical_xor_term",
 /* 240 */ "md_logical_or_term ::= md_logical_or_term MD_TOKEN_LOGICAL_OR md_logical_xor_term",
 /* 241 */ "md_logical_xor_term ::= md_logical_and_term",
 /* 242 */ "md_logical_xor_term ::= md_logical_xor_term MD_TOKEN_LOGICAL_XOR md_logical_and_term",
 /* 243 */ "md_logical_and_term ::= md_eqne_term",
 /* 244 */ "md_logical_and_term ::= md_logical_and_term MD_TOKEN_LOGICAL_AND md_eqne_term",
 /* 245 */ "md_eqne_term ::= md_cmp_term",
 /* 246 */ "md_eqne_term ::= md_eqne_term MD_TOKEN_MATCHES md_cmp_term",
 /* 247 */ "md_eqne_term ::= md_eqne_term MD_TOKEN_DOES_NOT_MATCH md_cmp_term",
 /* 248 */ "md_eqne_term ::= md_eqne_term MD_TOKEN_EQ md_cmp_term",
 /* 249 */ "md_eqne_term ::= md_eqne_term MD_TOKEN_NE md_cmp_term",
 /* 250 */ "md_cmp_term ::= md_bitwise_or_term",
 /* 251 */ "md_cmp_term ::= md_cmp_term MD_TOKEN_GT md_bitwise_or_term",
 /* 252 */ "md_cmp_term ::= md_cmp_term MD_TOKEN_GE md_bitwise_or_term",
 /* 253 */ "md_cmp_term ::= md_cmp_term MD_TOKEN_LT md_bitwise_or_term",
 /* 254 */ "md_cmp_term ::= md_cmp_term MD_TOKEN_LE md_bitwise_or_term",
 /* 255 */ "md_bitwise_or_term ::= md_bitwise_xor_term",
 /* 256 */ "md_bitwise_or_term ::= md_bitwise_or_term MD_TOKEN_BITWISE_OR md_bitwise_xor_term",
 /* 257 */ "md_bitwise_xor_term ::= md_bitwise_and_term",
 /* 258 */ "md_bitwise_xor_term ::= md_bitwise_xor_term MD_TOKEN_BITWISE_XOR md_bitwise_and_term",
 /* 259 */ "md_bitwise_and_term ::= md_bitwise_shift_term",
 /* 260 */ "md_bitwise_and_term ::= md_bitwise_and_term MD_TOKEN_BITWISE_AND md_bitwise_shift_term",
 /* 261 */ "md_bitwise_shift_term ::= md_addsubdot_term",
 /* 262 */ "md_bitwise_shift_term ::= md_bitwise_shift_term MD_TOKEN_BITWISE_LSH md_addsubdot_term",
 /* 263 */ "md_bitwise_shift_term ::= md_bitwise_shift_term MD_TOKEN_BITWISE_RSH md_addsubdot_term",
 /* 264 */ "md_addsubdot_term ::= md_muldiv_term",
 /* 265 */ "md_addsubdot_term ::= md_addsubdot_term MD_TOKEN_PLUS md_muldiv_term",
 /* 266 */ "md_addsubdot_term ::= md_addsubdot_term MD_TOKEN_MINUS md_muldiv_term",
 /* 267 */ "md_addsubdot_term ::= md_addsubdot_term MD_TOKEN_DOT md_muldiv_term",
 /* 268 */ "md_muldiv_term ::= md_unary_bitwise_op_term",
 /* 269 */ "md_muldiv_term ::= md_muldiv_term MD_TOKEN_TIMES md_unary_bitwise_op_term",
 /* 270 */ "md_muldiv_term ::= md_muldiv_term MD_TOKEN_DIVIDE md_unary_bitwise_op_term",
 /* 271 */ "md_muldiv_term ::= md_muldiv_term MD_TOKEN_INT_DIVIDE md_unary_bitwise_op_term",
 /* 272 */ "md_muldiv_term ::= md_muldiv_term MD_TOKEN_MOD md_unary_bitwise_op_term",
 /* 273 */ "md_unary_bitwise_op_term ::= md_pow_term",
 /* 274 */ "md_unary_bitwise_op_term ::= MD_TOKEN_PLUS md_unary_bitwise_op_term",
 /* 275 */ "md_unary_bitwise_op_term ::= MD_TOKEN_MINUS md_unary_bitwise_op_term",
 /* 276 */ "md_unary_bitwise_op_term ::= MD_TOKEN_LOGICAL_NOT md_unary_bitwise_op_term",
 /* 277 */ "md_unary_bitwise_op_term ::= MD_TOKEN_BITWISE_NOT md_unary_bitwise_op_term",
 /* 278 */ "md_pow_term ::= md_atom_or_fcn",
 /* 279 */ "md_pow_term ::= md_atom_or_fcn MD_TOKEN_POW md_pow_term",
 /* 280 */ "md_atom_or_fcn ::= md_field_name",
 /* 281 */ "md_field_name ::= MD_TOKEN_FIELD_NAME",
 /* 282 */ "md_field_name ::= MD_TOKEN_BRACED_FIELD_NAME",
 /* 283 */ "md_atom_or_fcn ::= md_indirect_field_name",
 /* 284 */ "md_indirect_field_name ::= MD_TOKEN_DOLLAR_SIGN MD_TOKEN_LEFT_BRACKET md_rhs MD_TOKEN_RIGHT_BRACKET",
 /* 285 */ "md_atom_or_fcn ::= md_oosvar_keylist",
 /* 286 */ "md_oosvar_keylist ::= md_oosvar_basename",
 /* 287 */ "md_oosvar_keylist ::= md_oosvar_keylist MD_TOKEN_LEFT_BRACKET md_rhs MD_TOKEN_RIGHT_BRACKET",
 /* 288 */ "md_oosvar_basename ::= MD_TOKEN_UNBRACED_OOSVAR_NAME",
 /* 289 */ "md_oosvar_basename ::= MD_TOKEN_BRACED_OOSVAR_NAME",
 /* 290 */ "md_oosvar_basename ::= MD_TOKEN_AT_SIGN MD_TOKEN_LEFT_BRACKET md_rhs MD_TOKEN_RIGHT_BRACKET",
 /* 291 */ "md_atom_or_fcn ::= MD_TOKEN_NUMBER",
 /* 292 */ "md_atom_or_fcn ::= MD_TOKEN_TRUE",
 /* 293 */ "md_atom_or_fcn ::= MD_TOKEN_FALSE",
 /* 294 */ "md_atom_or_fcn ::= md_string",
 /* 295 */ "md_atom_or_fcn ::= md_regexi",
 /* 296 */ "md_atom_or_fcn ::= md_bound_variable",
 /* 297 */ "md_bound_variable ::= MD_TOKEN_NON_SIGIL_NAME",
 /* 298 */ "md_string ::= MD_TOKEN_STRING",
 /* 299 */ "md_regexi ::= MD_TOKEN_REGEXI",
 /* 300 */ "md_atom_or_fcn ::= MD_TOKEN_CONTEXT_VARIABLE",
 /* 301 */ "md_atom_or_fcn ::= MD_TOKEN_ENV MD_TOKEN_LEFT_BRACKET md_rhs MD_TOKEN_RIGHT_BRACKET",
 /* 302 */ "md_atom_or_fcn ::= MD_TOKEN_LPAREN md_rhs MD_TOKEN_RPAREN",
 /* 303 */ "md_atom_or_fcn ::= MD_TOKEN_NON_SIGIL_NAME MD_TOKEN_LPAREN md_fcn_args MD_TOKEN_RPAREN",
 /* 304 */ "md_fcn_args ::=",
 /* 305 */ "md_fcn_args ::= md_rhs",
 /* 306 */ "md_fcn_args ::= md_fcn_args MD_TOKEN_COMMA md_rhs",
};
#endif /* NDEBUG */

/*
** This function returns the symbolic name associated with a token
** value.
*/
const char *mlr_dsl_ParseTokenName(int tokenType) {
#ifndef NDEBUG
	if (tokenType>0 && tokenType<(sizeof(yyTokenName)/sizeof(yyTokenName[0]))) {
		return yyTokenName[tokenType];
	} else {
		return "Unknown";
	}
#else
	return "";
#endif
}

/*
** This function allocates a new parser.
** The only argument is a pointer to a function which works like
** malloc.
**
** Inputs:
** A pointer to the function used to allocate memory.
**
** Outputs:
** A pointer to a parser.  This pointer is used in subsequent calls
** to Parse and mlr_dsl_lemon_parser_free.
*/
void *mlr_dsl_lemon_parser_alloc(void *(*mallocProc)(size_t)){
	lemon_parser_t *pParser;
	pParser = (lemon_parser_t*)(*mallocProc) ((size_t)sizeof(lemon_parser_t)) ;
	if (pParser) {
		pParser->yyidx = -1;
	}
	return pParser;
}

/* The following function deletes the value associated with a
** symbol.  The symbol can be either a terminal or nonterminal.
** "yymajor" is the symbol code, and "yypminor" is a pointer to
** the value.
*/
void mlr_dsl_yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
	switch (yymajor) {
		/* Here is inserted the actions which take place when a
		** terminal or non-terminal is destroyed.  This can happen
		** when the symbol is popped from the stack during a
		** reduce or during error processing or when a parser is
		** being destroyed before it is finished parsing.
		**
		** Note: during a reduce, the only symbols destroyed are those
		** which appear on the RHS of the rule, but which are not used
		** inside the C code.
		*/
		default:  break;   /* If no destructor action specified: do nothing */
	}
}

/*
** Pop the parser's stack once.
**
** If there is a destructor routine associated with the token which
** is popped from the stack, then call it.
**
** Return the major token number for the symbol popped.
*/
static int yy_pop_parser_stack(lemon_parser_t *pParser){
	YYCODETYPE yymajor;
	yyStackEntry *yytos = &pParser->yystack[pParser->yyidx];

	if (pParser->yyidx<0)  return 0;
#ifndef NDEBUG
	if (yyTraceFILE && pParser->yyidx>=0) {
		fprintf(yyTraceFILE,"%sPopping %s\n", yyTracePrompt, yyTokenName[yytos->major]);
	}
#endif
	yymajor = yytos->major;
	mlr_dsl_yy_destructor (yymajor, &yytos->minor);
	pParser->yyidx--;
	return yymajor;
}

/*
** Deallocate and destroy a parser.  Destructors are all called for
** all stack elements before shutting the parser down.
**
** Inputs:
** <ul>
** <li>  A pointer to the parser.  This should be a pointer
**       obtained from mlr_dsl_lemon_parser_alloc.
** <li>  A pointer to a function used to reclaim memory obtained
**       from malloc.
** </ul>
*/
void mlr_dsl_lemon_parser_free(
	void *pvparser,             /* The parser to be deleted */
	void (*freeProc)(void*))    /* Function used to reclaim memory */
{
	lemon_parser_t *pparser = (lemon_parser_t*)pvparser;
	if (pparser==0)  return;
	while (pparser->yyidx>=0)  yy_pop_parser_stack(pparser);
	(*freeProc)((void*)pparser);
}

/*
** Find the appropriate action for a parser given the terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_shift_action(
	lemon_parser_t *pparser,        /* The parser */
	int iLookAhead)           /* The look-ahead token */
{
	int i;
	int stateno = pparser->yystack[pparser->yyidx].stateno;

	/* if (pparser->yyidx<0)  return YY_NO_ACTION;  */
	i = yy_shift_ofst[stateno];
	if (i==YY_SHIFT_USE_DFLT) {
		return yy_default[stateno];
	}
	if (iLookAhead==YYNOCODE) {
		return YY_NO_ACTION;
	}
	i += iLookAhead;
	if (i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead) {
#ifdef YYFALLBACK
		int iFallback;            /* Fallback token */
		if (iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0])
					 && (iFallback = yyFallback[iLookAhead])!=0) {
#ifndef NDEBUG
			if (yyTraceFILE) {
				fprintf(yyTraceFILE, "%sFALLBACK %s => %s\n",
					 yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[iFallback]);
			}
#endif
			return yy_find_shift_action(pparser, iFallback);
		}
#endif
		return yy_default[stateno];
	} else {
		return yy_action[i];
	}
}

/*
** Find the appropriate action for a parser given the non-terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_reduce_action(
	lemon_parser_t *pparser,        /* The parser */
	int iLookAhead)           /* The look-ahead token */
{
	int i;
	int stateno = pparser->yystack[pparser->yyidx].stateno;

	i = yy_reduce_ofst[stateno];
	if (i==YY_REDUCE_USE_DFLT) {
		return yy_default[stateno];
	}
	if (iLookAhead==YYNOCODE) {
		return YY_NO_ACTION;
	}
	i += iLookAhead;
	if (i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead) {
		return yy_default[stateno];
	} else {
		return yy_action[i];
	}
}

/*
** Perform a shift action.
*/
static void yy_shift(
	lemon_parser_t *pparser,          /* The parser to be shifted */
	int yyNewState,               /* The new state to shift in */
	int yyMajor,                  /* The major token to shift in */
	YYMINORTYPE *yypMinor)        /* Pointer ot the minor token to shift in */
{
	yyStackEntry *yytos;
	pparser->yyidx++;
	if (pparser->yyidx>=YYSTACKDEPTH) {
		 ParseARG_FETCH;
		 pparser->yyidx--;
#ifndef NDEBUG
		 if (yyTraceFILE) {
			 fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
		 }
#endif
		 while (pparser->yyidx>=0)  yy_pop_parser_stack(pparser);
		 /* Here code is inserted which will execute if the parser
		 ** stack every overflows */
		 ParseARG_STORE; /* Suppress warning about unused %extra_argument var */
		 return;
	}
	yytos = &pparser->yystack[pparser->yyidx];
	yytos->stateno = yyNewState;
	yytos->major = yyMajor;
	yytos->minor = *yypMinor;
#ifndef NDEBUG
	if (yyTraceFILE && pparser->yyidx>0) {
		int i;
		fprintf(yyTraceFILE,"%sShift %d\n",yyTracePrompt,yyNewState);
		fprintf(yyTraceFILE,"%sStack:",yyTracePrompt);
		for(i=1; i<=pparser->yyidx; i++)
			fprintf(yyTraceFILE," %s", yyTokenName[pparser->yystack[i].major]);
		fprintf(yyTraceFILE,"\n");
	}
#endif
}

/* The following table contains information about every rule that
** is used during the reduce.
*/
static struct {
	YYCODETYPE lhs;         /* Symbol on the left-hand side of the rule */
	unsigned char nrhs;     /* Number of right-hand side symbols in the rule */
} yyRuleInfo[] = {
  { 97, 1 },
  { 98, 1 },
  { 98, 1 },
  { 98, 3 },
  { 98, 2 },
  { 99, 0 },
  { 100, 1 },
  { 100, 1 },
  { 100, 1 },
  { 100, 1 },
  { 100, 1 },
  { 100, 1 },
  { 100, 1 },
  { 100, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 99, 1 },
  { 101, 4 },
  { 102, 4 },
  { 103, 4 },
  { 104, 7 },
  { 113, 8 },
  { 105, 11 },
  { 106, 11 },
  { 106, 13 },
  { 107, 11 },
  { 107, 13 },
  { 157, 1 },
  { 158, 1 },
  { 158, 3 },
  { 108, 1 },
  { 108, 2 },
  { 160, 1 },
  { 160, 2 },
  { 162, 7 },
  { 163, 7 },
  { 161, 4 },
  { 114, 1 },
  { 116, 2 },
  { 109, 3 },
  { 110, 6 },
  { 115, 3 },
  { 111, 3 },
  { 112, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 109, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 115, 3 },
  { 117, 2 },
  { 117, 2 },
  { 117, 2 },
  { 165, 0 },
  { 165, 1 },
  { 165, 1 },
  { 165, 1 },
  { 165, 1 },
  { 165, 3 },
  { 165, 3 },
  { 118, 5 },
  { 119, 5 },
  { 120, 5 },
  { 121, 2 },
  { 168, 0 },
  { 168, 1 },
  { 168, 3 },
  { 122, 5 },
  { 123, 5 },
  { 124, 5 },
  { 125, 2 },
  { 125, 2 },
  { 125, 4 },
  { 125, 4 },
  { 125, 2 },
  { 125, 4 },
  { 126, 5 },
  { 126, 7 },
  { 126, 5 },
  { 126, 7 },
  { 126, 5 },
  { 126, 7 },
  { 127, 5 },
  { 127, 7 },
  { 127, 5 },
  { 127, 7 },
  { 127, 5 },
  { 127, 7 },
  { 128, 5 },
  { 128, 7 },
  { 128, 5 },
  { 128, 7 },
  { 128, 5 },
  { 128, 7 },
  { 169, 1 },
  { 169, 3 },
  { 129, 2 },
  { 129, 2 },
  { 129, 4 },
  { 129, 4 },
  { 129, 2 },
  { 129, 4 },
  { 130, 5 },
  { 130, 7 },
  { 130, 5 },
  { 130, 7 },
  { 130, 5 },
  { 130, 7 },
  { 131, 5 },
  { 131, 7 },
  { 131, 5 },
  { 131, 7 },
  { 131, 5 },
  { 131, 7 },
  { 132, 5 },
  { 132, 7 },
  { 132, 5 },
  { 132, 7 },
  { 132, 5 },
  { 132, 7 },
  { 170, 1 },
  { 170, 3 },
  { 133, 4 },
  { 133, 6 },
  { 134, 7 },
  { 134, 9 },
  { 135, 7 },
  { 135, 9 },
  { 136, 7 },
  { 136, 9 },
  { 171, 1 },
  { 171, 3 },
  { 172, 1 },
  { 172, 3 },
  { 137, 4 },
  { 137, 6 },
  { 138, 7 },
  { 138, 9 },
  { 139, 7 },
  { 139, 9 },
  { 140, 7 },
  { 140, 9 },
  { 173, 1 },
  { 173, 3 },
  { 174, 1 },
  { 174, 3 },
  { 141, 1 },
  { 145, 1 },
  { 142, 3 },
  { 143, 3 },
  { 144, 3 },
  { 146, 2 },
  { 147, 2 },
  { 148, 5 },
  { 149, 5 },
  { 150, 5 },
  { 151, 2 },
  { 152, 2 },
  { 153, 5 },
  { 154, 5 },
  { 155, 5 },
  { 167, 1 },
  { 167, 1 },
  { 167, 1 },
  { 156, 1 },
  { 175, 5 },
  { 175, 1 },
  { 176, 1 },
  { 176, 3 },
  { 177, 1 },
  { 177, 3 },
  { 178, 1 },
  { 178, 3 },
  { 179, 1 },
  { 179, 3 },
  { 179, 3 },
  { 179, 3 },
  { 179, 3 },
  { 180, 1 },
  { 180, 3 },
  { 180, 3 },
  { 180, 3 },
  { 180, 3 },
  { 181, 1 },
  { 181, 3 },
  { 182, 1 },
  { 182, 3 },
  { 183, 1 },
  { 183, 3 },
  { 184, 1 },
  { 184, 3 },
  { 184, 3 },
  { 185, 1 },
  { 185, 3 },
  { 185, 3 },
  { 185, 3 },
  { 186, 1 },
  { 186, 3 },
  { 186, 3 },
  { 186, 3 },
  { 186, 3 },
  { 187, 1 },
  { 187, 2 },
  { 187, 2 },
  { 187, 2 },
  { 187, 2 },
  { 188, 1 },
  { 188, 3 },
  { 189, 1 },
  { 164, 1 },
  { 164, 1 },
  { 189, 1 },
  { 166, 4 },
  { 189, 1 },
  { 159, 1 },
  { 159, 4 },
  { 190, 1 },
  { 190, 1 },
  { 190, 4 },
  { 189, 1 },
  { 189, 1 },
  { 189, 1 },
  { 189, 1 },
  { 189, 1 },
  { 189, 1 },
  { 193, 1 },
  { 191, 1 },
  { 192, 1 },
  { 189, 1 },
  { 189, 4 },
  { 189, 3 },
  { 189, 4 },
  { 194, 0 },
  { 194, 1 },
  { 194, 3 },
};

static void yy_accept(lemon_parser_t*);  /* Forward Declaration */

/*
** Perform a reduce action and the shift that must immediately
** follow the reduce.
*/
static void yy_reduce(
	lemon_parser_t *pparser,         /* The parser */
	int yyruleno)                /* Number of the rule by which to reduce */
{
	int yygoto = -999 ;                     /* The next state */
	int yyact = -999;                      /* The next action */
	YYMINORTYPE yygotominor;        /* The LHS of the rule reduced */
	yyStackEntry *yymsp;            /* The top of the parser's stack */
	int yysize;                     /* Amount to pop the stack */
	ParseARG_FETCH;
	yymsp = &pparser->yystack[pparser->yyidx];
#ifndef NDEBUG
	if (yyTraceFILE && yyruleno>=0
				&& yyruleno<sizeof(yyRuleName)/sizeof(yyRuleName[0])) {
		fprintf(yyTraceFILE, "%sReduce [%s].\n", yyTracePrompt,
			yyRuleName[yyruleno]);
	}
#endif /* NDEBUG */

	switch (yyruleno) {
	/* Beginning here are the reduction cases.  A typical example
	** follows:
	**   case 0:
	**  #line <lineno> <grammarfile>
	**     { ... }           // User supplied code
	**  #line <lineno> <thisfile>
	**     break;
	*/
      case 0:
#line 55 "mlr_dsl_parse.y"
{
	past->proot = yymsp[0].minor.yy121;
}
#line 2808 "mlr_dsl_parse.c"
        break;
      case 1:
      case 2:
#line 71 "mlr_dsl_parse.y"
{
	if (yymsp[0].minor.yy121->type == MD_AST_NODE_TYPE_NOP) {
		yygotominor.yy121 = mlr_dsl_ast_node_alloc_zary("list", MD_AST_NODE_TYPE_STATEMENT_LIST);
	} else {
		yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("list", MD_AST_NODE_TYPE_STATEMENT_LIST, yymsp[0].minor.yy121);
	}
}
#line 2820 "mlr_dsl_parse.c"
        break;
      case 3:
#line 90 "mlr_dsl_parse.y"
{
	if (yymsp[-2].minor.yy121->type == MD_AST_NODE_TYPE_NOP) {
		yygotominor.yy121 = yymsp[0].minor.yy121;
	} else {
		yygotominor.yy121 = mlr_dsl_ast_node_prepend_arg(yymsp[0].minor.yy121, yymsp[-2].minor.yy121);
	}
}
#line 2831 "mlr_dsl_parse.c"
        break;
      case 4:
#line 98 "mlr_dsl_parse.y"
{
	if (yymsp[-1].minor.yy121->type == MD_AST_NODE_TYPE_NOP) {
		yygotominor.yy121 = yymsp[0].minor.yy121;
	} else {
		yygotominor.yy121 = mlr_dsl_ast_node_prepend_arg(yymsp[0].minor.yy121, yymsp[-1].minor.yy121);
	}
}
#line 2842 "mlr_dsl_parse.c"
        break;
      case 5:
#line 107 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_zary("nop", MD_AST_NODE_TYPE_NOP);
}
#line 2849 "mlr_dsl_parse.c"
        break;
      case 6:
      case 7:
      case 8:
      case 9:
      case 10:
      case 11:
      case 12:
      case 13:
      case 14:
      case 15:
      case 16:
      case 17:
      case 18:
      case 19:
      case 20:
      case 21:
      case 22:
      case 23:
      case 24:
      case 25:
      case 26:
      case 27:
      case 28:
      case 29:
      case 30:
      case 31:
      case 32:
      case 33:
      case 34:
      case 35:
      case 36:
      case 37:
      case 38:
      case 39:
      case 40:
      case 41:
      case 42:
      case 43:
      case 44:
      case 45:
      case 46:
      case 47:
      case 48:
      case 49:
      case 50:
      case 51:
      case 52:
      case 53:
      case 54:
      case 55:
      case 56:
      case 57:
      case 58:
      case 59:
      case 60:
      case 233:
#line 112 "mlr_dsl_parse.y"
{ yygotominor.yy121 = yymsp[0].minor.yy121; }
#line 2909 "mlr_dsl_parse.c"
        break;
      case 61:
#line 178 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc(yymsp[0].minor.yy0->text, MD_AST_NODE_TYPE_BREAK);
}
#line 2916 "mlr_dsl_parse.c"
        break;
      case 62:
#line 181 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc(yymsp[0].minor.yy0->text, MD_AST_NODE_TYPE_CONTINUE);
}
#line 2923 "mlr_dsl_parse.c"
        break;
      case 63:
#line 186 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_BEGIN, yymsp[-1].minor.yy121);
}
#line 2930 "mlr_dsl_parse.c"
        break;
      case 64:
#line 189 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_END, yymsp[-1].minor.yy121);
}
#line 2937 "mlr_dsl_parse.c"
        break;
      case 65:
#line 194 "mlr_dsl_parse.y"
{
	//yygotominor.yy121 = mlr_dsl_ast_node_prepend_arg(yymsp[-1].minor.yy121, yymsp[-3].minor.yy121);
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("cond", MD_AST_NODE_TYPE_CONDITIONAL_BLOCK, yymsp[-3].minor.yy121, yymsp[-1].minor.yy121);
}
#line 2945 "mlr_dsl_parse.c"
        break;
      case 66:
#line 204 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_WHILE, yymsp[-4].minor.yy121, yymsp[-1].minor.yy121);
}
#line 2952 "mlr_dsl_parse.c"
        break;
      case 67:
#line 214 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-7].minor.yy0->text, MD_AST_NODE_TYPE_DO_WHILE, yymsp[-5].minor.yy121, yymsp[-1].minor.yy121);
}
#line 2959 "mlr_dsl_parse.c"
        break;
      case 68:
#line 228 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(
		yymsp[-10].minor.yy0->text,
		MD_AST_NODE_TYPE_FOR_SREC,
		mlr_dsl_ast_node_alloc_binary(
			"variables",
			MD_AST_NODE_TYPE_FOR_VARIABLES,
			yymsp[-8].minor.yy121,
			yymsp[-6].minor.yy121
		),
		yymsp[-1].minor.yy121
	);
}
#line 2976 "mlr_dsl_parse.c"
        break;
      case 69:
#line 251 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_ternary(
		yymsp[-10].minor.yy0->text,
		MD_AST_NODE_TYPE_FOR_OOSVAR,
		mlr_dsl_ast_node_alloc_binary(
			"key_and_value_variables",
			MD_AST_NODE_TYPE_FOR_VARIABLES,
			mlr_dsl_ast_node_alloc_unary(
				"key_variables",
				MD_AST_NODE_TYPE_FOR_VARIABLES,
				yymsp[-8].minor.yy121
			),
			yymsp[-6].minor.yy121
		),
		mlr_dsl_ast_node_alloc_zary("empty_keylist", MD_AST_NODE_TYPE_OOSVAR_KEYLIST),
		yymsp[-1].minor.yy121
	);
}
#line 2998 "mlr_dsl_parse.c"
        break;
      case 70:
#line 280 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_ternary(
		yymsp[-12].minor.yy0->text,
		MD_AST_NODE_TYPE_FOR_OOSVAR,
		mlr_dsl_ast_node_alloc_binary(
			"key_and_value_variables",
			MD_AST_NODE_TYPE_FOR_VARIABLES,
			yymsp[-9].minor.yy121,
			yymsp[-6].minor.yy121
		),
		mlr_dsl_ast_node_alloc_zary("empty_keylist", MD_AST_NODE_TYPE_OOSVAR_KEYLIST),
		yymsp[-1].minor.yy121
	);
}
#line 3016 "mlr_dsl_parse.c"
        break;
      case 71:
#line 304 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_ternary(
		yymsp[-10].minor.yy0->text,
		MD_AST_NODE_TYPE_FOR_OOSVAR,
		mlr_dsl_ast_node_alloc_binary(
			"key_and_value_variables",
			MD_AST_NODE_TYPE_FOR_VARIABLES,
			mlr_dsl_ast_node_alloc_unary(
				"key_variables",
				MD_AST_NODE_TYPE_FOR_VARIABLES,
				yymsp[-8].minor.yy121
			),
			yymsp[-6].minor.yy121
		),
		yymsp[-4].minor.yy121,
		yymsp[-1].minor.yy121
	);
}
#line 3038 "mlr_dsl_parse.c"
        break;
      case 72:
#line 333 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_ternary(
		yymsp[-12].minor.yy0->text,
		MD_AST_NODE_TYPE_FOR_OOSVAR,
		mlr_dsl_ast_node_alloc_binary(
			"key_and_value_variables",
			MD_AST_NODE_TYPE_FOR_VARIABLES,
			yymsp[-9].minor.yy121,
			yymsp[-6].minor.yy121
		),
		yymsp[-4].minor.yy121,
		yymsp[-1].minor.yy121
	);
}
#line 3056 "mlr_dsl_parse.c"
        break;
      case 73:
#line 348 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc(yymsp[0].minor.yy0->text, MD_AST_NODE_TYPE_NON_SIGIL_NAME);
}
#line 3063 "mlr_dsl_parse.c"
        break;
      case 74:
#line 351 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("key_variables", MD_AST_NODE_TYPE_FOR_VARIABLES, yymsp[0].minor.yy0);
}
#line 3070 "mlr_dsl_parse.c"
        break;
      case 75:
#line 354 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_append_arg(yymsp[-2].minor.yy121, yymsp[0].minor.yy0);
}
#line 3077 "mlr_dsl_parse.c"
        break;
      case 76:
      case 83:
      case 236:
      case 238:
      case 239:
      case 241:
      case 243:
      case 245:
      case 250:
      case 255:
      case 257:
      case 259:
      case 261:
      case 264:
      case 268:
      case 273:
      case 278:
      case 280:
      case 283:
      case 285:
      case 286:
      case 294:
      case 295:
      case 296:
#line 363 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = yymsp[0].minor.yy121;
}
#line 3107 "mlr_dsl_parse.c"
        break;
      case 77:
      case 79:
#line 366 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_append_arg(yymsp[-1].minor.yy121, yymsp[0].minor.yy121);
}
#line 3115 "mlr_dsl_parse.c"
        break;
      case 78:
#line 369 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("if_head", MD_AST_NODE_TYPE_IF_HEAD, yymsp[0].minor.yy121);
}
#line 3122 "mlr_dsl_parse.c"
        break;
      case 80:
      case 81:
#line 380 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_IF_ITEM, yymsp[-4].minor.yy121, yymsp[-1].minor.yy121);
}
#line 3130 "mlr_dsl_parse.c"
        break;
      case 82:
#line 395 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_IF_ITEM, yymsp[-1].minor.yy121);
}
#line 3137 "mlr_dsl_parse.c"
        break;
      case 84:
#line 405 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_FILTER, yymsp[0].minor.yy121);
}
#line 3144 "mlr_dsl_parse.c"
        break;
      case 85:
#line 410 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121, yymsp[0].minor.yy121);
}
#line 3151 "mlr_dsl_parse.c"
        break;
      case 86:
#line 416 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_INDIRECT_SREC_ASSIGNMENT, yymsp[-3].minor.yy121, yymsp[0].minor.yy121);
}
#line 3158 "mlr_dsl_parse.c"
        break;
      case 87:
#line 420 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121, yymsp[0].minor.yy121);
}
#line 3165 "mlr_dsl_parse.c"
        break;
      case 88:
#line 424 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_OOSVAR_FROM_FULL_SREC_ASSIGNMENT, yymsp[-2].minor.yy121, yymsp[0].minor.yy0);
}
#line 3172 "mlr_dsl_parse.c"
        break;
      case 89:
#line 428 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_FULL_SREC_FROM_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy0, yymsp[0].minor.yy121);
}
#line 3179 "mlr_dsl_parse.c"
        break;
      case 90:
#line 433 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("||", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3187 "mlr_dsl_parse.c"
        break;
      case 91:
#line 437 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("^^", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3195 "mlr_dsl_parse.c"
        break;
      case 92:
#line 441 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("&&", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3203 "mlr_dsl_parse.c"
        break;
      case 93:
#line 445 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("|", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3211 "mlr_dsl_parse.c"
        break;
      case 94:
#line 449 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("^", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3219 "mlr_dsl_parse.c"
        break;
      case 95:
#line 453 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("&", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3227 "mlr_dsl_parse.c"
        break;
      case 96:
#line 457 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("<<", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3235 "mlr_dsl_parse.c"
        break;
      case 97:
#line 461 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary(">>", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3243 "mlr_dsl_parse.c"
        break;
      case 98:
#line 465 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("+", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3251 "mlr_dsl_parse.c"
        break;
      case 99:
#line 469 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("-", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3259 "mlr_dsl_parse.c"
        break;
      case 100:
#line 473 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary(".", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3267 "mlr_dsl_parse.c"
        break;
      case 101:
#line 477 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("*", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3275 "mlr_dsl_parse.c"
        break;
      case 102:
#line 481 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("/", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3283 "mlr_dsl_parse.c"
        break;
      case 103:
#line 485 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("//", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3291 "mlr_dsl_parse.c"
        break;
      case 104:
#line 489 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("%", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3299 "mlr_dsl_parse.c"
        break;
      case 105:
#line 493 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_SREC_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("**", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3307 "mlr_dsl_parse.c"
        break;
      case 106:
#line 499 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("||", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3315 "mlr_dsl_parse.c"
        break;
      case 107:
#line 503 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("^^", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3323 "mlr_dsl_parse.c"
        break;
      case 108:
#line 507 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("&&", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3331 "mlr_dsl_parse.c"
        break;
      case 109:
#line 511 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("|", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3339 "mlr_dsl_parse.c"
        break;
      case 110:
#line 515 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("^", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3347 "mlr_dsl_parse.c"
        break;
      case 111:
#line 519 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("&", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3355 "mlr_dsl_parse.c"
        break;
      case 112:
#line 523 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("<<", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3363 "mlr_dsl_parse.c"
        break;
      case 113:
#line 527 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary(">>", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3371 "mlr_dsl_parse.c"
        break;
      case 114:
#line 531 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("+", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3379 "mlr_dsl_parse.c"
        break;
      case 115:
#line 535 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("-", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3387 "mlr_dsl_parse.c"
        break;
      case 116:
#line 539 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary(".", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3395 "mlr_dsl_parse.c"
        break;
      case 117:
#line 543 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("*", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3403 "mlr_dsl_parse.c"
        break;
      case 118:
#line 547 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("/", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3411 "mlr_dsl_parse.c"
        break;
      case 119:
#line 551 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("//", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3419 "mlr_dsl_parse.c"
        break;
      case 120:
#line 555 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("%", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3427 "mlr_dsl_parse.c"
        break;
      case 121:
#line 559 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("=", MD_AST_NODE_TYPE_OOSVAR_ASSIGNMENT, yymsp[-2].minor.yy121,
		mlr_dsl_ast_node_alloc_binary("**", MD_AST_NODE_TYPE_OPERATOR, mlr_dsl_ast_tree_copy(yymsp[-2].minor.yy121) , yymsp[0].minor.yy121));
}
#line 3435 "mlr_dsl_parse.c"
        break;
      case 122:
      case 123:
#line 565 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_UNSET, yymsp[0].minor.yy0);
}
#line 3443 "mlr_dsl_parse.c"
        break;
      case 124:
#line 571 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_set_function_name(yymsp[0].minor.yy121, yymsp[-1].minor.yy0->text);
}
#line 3450 "mlr_dsl_parse.c"
        break;
      case 125:
#line 575 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_zary("temp", MD_AST_NODE_TYPE_UNSET);
}
#line 3457 "mlr_dsl_parse.c"
        break;
      case 126:
      case 127:
      case 129:
#line 579 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("temp", MD_AST_NODE_TYPE_UNSET, yymsp[0].minor.yy121);
}
#line 3466 "mlr_dsl_parse.c"
        break;
      case 128:
#line 585 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("temp", MD_AST_NODE_TYPE_UNSET, yymsp[0].minor.yy0);
}
#line 3473 "mlr_dsl_parse.c"
        break;
      case 130:
      case 131:
      case 138:
      case 167:
      case 193:
      case 203:
      case 205:
      case 215:
      case 217:
      case 306:
#line 592 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_append_arg(yymsp[-2].minor.yy121, yymsp[0].minor.yy121);
}
#line 3489 "mlr_dsl_parse.c"
        break;
      case 132:
#line 600 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_TEE,
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE, yymsp[-2].minor.yy121));
}
#line 3497 "mlr_dsl_parse.c"
        break;
      case 133:
#line 605 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_TEE,
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND, yymsp[-2].minor.yy121));
}
#line 3505 "mlr_dsl_parse.c"
        break;
      case 134:
#line 610 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_TEE,
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE, yymsp[-2].minor.yy121));
}
#line 3513 "mlr_dsl_parse.c"
        break;
      case 135:
#line 623 "mlr_dsl_parse.y"
{
	yymsp[0].minor.yy121 = mlr_dsl_ast_node_set_function_name(yymsp[0].minor.yy121, yymsp[-1].minor.yy0->text);
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_EMITF, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 3522 "mlr_dsl_parse.c"
        break;
      case 136:
#line 629 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_zary("temp", MD_AST_NODE_TYPE_EMITF);
}
#line 3529 "mlr_dsl_parse.c"
        break;
      case 137:
#line 632 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("temp", MD_AST_NODE_TYPE_EMITF, yymsp[0].minor.yy121);
}
#line 3536 "mlr_dsl_parse.c"
        break;
      case 139:
#line 639 "mlr_dsl_parse.y"
{
	yymsp[0].minor.yy121 = mlr_dsl_ast_node_set_function_name(yymsp[0].minor.yy121, yymsp[-4].minor.yy0->text);
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITF, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE, yymsp[-2].minor.yy121));
}
#line 3545 "mlr_dsl_parse.c"
        break;
      case 140:
#line 645 "mlr_dsl_parse.y"
{
	yymsp[0].minor.yy121 = mlr_dsl_ast_node_set_function_name(yymsp[0].minor.yy121, yymsp[-4].minor.yy0->text);
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITF, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND, yymsp[-2].minor.yy121));
}
#line 3554 "mlr_dsl_parse.c"
        break;
      case 141:
#line 651 "mlr_dsl_parse.y"
{
	yymsp[0].minor.yy121 = mlr_dsl_ast_node_set_function_name(yymsp[0].minor.yy121, yymsp[-4].minor.yy0->text);
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITF, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE, yymsp[-2].minor.yy121));
}
#line 3563 "mlr_dsl_parse.c"
        break;
      case 142:
      case 143:
#line 659 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_unary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[0].minor.yy0),
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 3573 "mlr_dsl_parse.c"
        break;
      case 144:
      case 145:
#line 670 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_binary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[-2].minor.yy0, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 3583 "mlr_dsl_parse.c"
        break;
      case 146:
#line 681 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_unary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 3592 "mlr_dsl_parse.c"
        break;
      case 147:
#line 686 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_binary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[-2].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 3601 "mlr_dsl_parse.c"
        break;
      case 148:
      case 150:
#line 694 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[0].minor.yy0),
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
		yymsp[-2].minor.yy121));
}
#line 3612 "mlr_dsl_parse.c"
        break;
      case 149:
      case 151:
#line 702 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[-2].minor.yy0, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
		yymsp[-4].minor.yy121));
}
#line 3623 "mlr_dsl_parse.c"
        break;
      case 152:
#line 726 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
		yymsp[-2].minor.yy121));
}
#line 3633 "mlr_dsl_parse.c"
        break;
      case 153:
#line 734 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[-2].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
		yymsp[-4].minor.yy121));
}
#line 3643 "mlr_dsl_parse.c"
        break;
      case 154:
      case 156:
#line 744 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[0].minor.yy0),
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND,
		yymsp[-2].minor.yy121));
}
#line 3654 "mlr_dsl_parse.c"
        break;
      case 155:
      case 157:
#line 752 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[-2].minor.yy0, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND,
		yymsp[-4].minor.yy121));
}
#line 3665 "mlr_dsl_parse.c"
        break;
      case 158:
#line 776 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND,
		yymsp[-2].minor.yy121));
}
#line 3675 "mlr_dsl_parse.c"
        break;
      case 159:
#line 784 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[-2].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND,
		yymsp[-4].minor.yy121));
}
#line 3685 "mlr_dsl_parse.c"
        break;
      case 160:
      case 162:
#line 794 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[0].minor.yy0),
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE,
		yymsp[-2].minor.yy121));
}
#line 3696 "mlr_dsl_parse.c"
        break;
      case 161:
      case 163:
#line 802 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[-2].minor.yy0, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE,
		yymsp[-4].minor.yy121));
}
#line 3707 "mlr_dsl_parse.c"
        break;
      case 164:
#line 826 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE,
		yymsp[-2].minor.yy121));
}
#line 3717 "mlr_dsl_parse.c"
        break;
      case 165:
#line 834 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP,
		mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP, yymsp[-2].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE,
		yymsp[-4].minor.yy121));
}
#line 3727 "mlr_dsl_parse.c"
        break;
      case 166:
#line 841 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("emitp_namelist", MD_AST_NODE_TYPE_EMITP, yymsp[0].minor.yy121);
}
#line 3734 "mlr_dsl_parse.c"
        break;
      case 168:
      case 169:
#line 849 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_unary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[0].minor.yy0),
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 3744 "mlr_dsl_parse.c"
        break;
      case 170:
      case 171:
#line 860 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_binary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[-2].minor.yy0, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 3754 "mlr_dsl_parse.c"
        break;
      case 172:
#line 871 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_unary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 3763 "mlr_dsl_parse.c"
        break;
      case 173:
#line 876 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_binary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[-2].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 3772 "mlr_dsl_parse.c"
        break;
      case 174:
      case 176:
#line 885 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[0].minor.yy0),
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
		yymsp[-2].minor.yy121));
}
#line 3783 "mlr_dsl_parse.c"
        break;
      case 175:
      case 177:
#line 893 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[-2].minor.yy0, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
		yymsp[-4].minor.yy121));
}
#line 3794 "mlr_dsl_parse.c"
        break;
      case 178:
#line 917 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
		yymsp[-2].minor.yy121));
}
#line 3804 "mlr_dsl_parse.c"
        break;
      case 179:
#line 925 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[-2].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
		yymsp[-4].minor.yy121));
}
#line 3814 "mlr_dsl_parse.c"
        break;
      case 180:
      case 182:
#line 935 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[0].minor.yy0),
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND,
		yymsp[-2].minor.yy121));
}
#line 3825 "mlr_dsl_parse.c"
        break;
      case 181:
      case 183:
#line 943 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[-2].minor.yy0, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND,
		yymsp[-4].minor.yy121));
}
#line 3836 "mlr_dsl_parse.c"
        break;
      case 184:
#line 967 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND,
		yymsp[-2].minor.yy121));
}
#line 3846 "mlr_dsl_parse.c"
        break;
      case 185:
#line 975 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[-2].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND,
		yymsp[-4].minor.yy121));
}
#line 3856 "mlr_dsl_parse.c"
        break;
      case 186:
      case 188:
#line 985 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[0].minor.yy0),
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE,
		yymsp[-2].minor.yy121));
}
#line 3867 "mlr_dsl_parse.c"
        break;
      case 187:
      case 189:
#line 993 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[-2].minor.yy0, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE,
		yymsp[-4].minor.yy121));
}
#line 3878 "mlr_dsl_parse.c"
        break;
      case 190:
#line 1017 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_unary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE,
		yymsp[-2].minor.yy121));
}
#line 3888 "mlr_dsl_parse.c"
        break;
      case 191:
#line 1025 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT,
		mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT, yymsp[-2].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE,
		yymsp[-4].minor.yy121));
}
#line 3898 "mlr_dsl_parse.c"
        break;
      case 192:
#line 1033 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("emit_namelist", MD_AST_NODE_TYPE_EMIT, yymsp[0].minor.yy121);
}
#line 3905 "mlr_dsl_parse.c"
        break;
      case 194:
#line 1043 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED,
		mlr_dsl_ast_node_alloc_unary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED, yymsp[-1].minor.yy121),
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 3914 "mlr_dsl_parse.c"
        break;
      case 195:
#line 1051 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-5].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED,
		mlr_dsl_ast_node_alloc_binary(yymsp[-5].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED, yymsp[-3].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 3923 "mlr_dsl_parse.c"
        break;
      case 196:
#line 1059 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED,
		mlr_dsl_ast_node_alloc_unary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED, yymsp[-1].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE, yymsp[-4].minor.yy121));
}
#line 3932 "mlr_dsl_parse.c"
        break;
      case 197:
#line 1067 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-8].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED,
		mlr_dsl_ast_node_alloc_binary(yymsp[-8].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED, yymsp[-3].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE, yymsp[-6].minor.yy121));
}
#line 3941 "mlr_dsl_parse.c"
        break;
      case 198:
#line 1075 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED,
		mlr_dsl_ast_node_alloc_unary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED, yymsp[-1].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND, yymsp[-4].minor.yy121));
}
#line 3950 "mlr_dsl_parse.c"
        break;
      case 199:
#line 1083 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-8].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED,
		mlr_dsl_ast_node_alloc_binary(yymsp[-8].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED, yymsp[-3].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND, yymsp[-6].minor.yy121));
}
#line 3959 "mlr_dsl_parse.c"
        break;
      case 200:
#line 1091 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED,
		mlr_dsl_ast_node_alloc_unary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED, yymsp[-1].minor.yy121),
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE, yymsp[-4].minor.yy121));
}
#line 3968 "mlr_dsl_parse.c"
        break;
      case 201:
#line 1099 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-8].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED,
		mlr_dsl_ast_node_alloc_binary(yymsp[-8].minor.yy0->text, MD_AST_NODE_TYPE_EMITP_LASHED, yymsp[-3].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE, yymsp[-6].minor.yy121));
}
#line 3977 "mlr_dsl_parse.c"
        break;
      case 202:
#line 1105 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("lashed_keylists", MD_AST_NODE_TYPE_EMITP_LASHED, yymsp[0].minor.yy121);
}
#line 3984 "mlr_dsl_parse.c"
        break;
      case 204:
#line 1112 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("lashed_namelist", MD_AST_NODE_TYPE_EMITP_LASHED, yymsp[0].minor.yy121);
}
#line 3991 "mlr_dsl_parse.c"
        break;
      case 206:
#line 1122 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED,
		mlr_dsl_ast_node_alloc_unary(yymsp[-3].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED, yymsp[-1].minor.yy121),
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 4000 "mlr_dsl_parse.c"
        break;
      case 207:
#line 1130 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-5].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED,
		mlr_dsl_ast_node_alloc_binary(yymsp[-5].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED, yymsp[-3].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_zary("stream", MD_AST_NODE_TYPE_STREAM));
}
#line 4009 "mlr_dsl_parse.c"
        break;
      case 208:
#line 1138 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED,
		mlr_dsl_ast_node_alloc_unary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED, yymsp[-1].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE, yymsp[-4].minor.yy121));
}
#line 4018 "mlr_dsl_parse.c"
        break;
      case 209:
#line 1146 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-8].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED,
		mlr_dsl_ast_node_alloc_binary(yymsp[-8].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED, yymsp[-3].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE, yymsp[-6].minor.yy121));
}
#line 4027 "mlr_dsl_parse.c"
        break;
      case 210:
#line 1154 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED,
		mlr_dsl_ast_node_alloc_unary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED, yymsp[-1].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND, yymsp[-4].minor.yy121));
}
#line 4036 "mlr_dsl_parse.c"
        break;
      case 211:
#line 1162 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-8].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED,
		mlr_dsl_ast_node_alloc_binary(yymsp[-8].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED, yymsp[-3].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND, yymsp[-6].minor.yy121));
}
#line 4045 "mlr_dsl_parse.c"
        break;
      case 212:
#line 1170 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED,
		mlr_dsl_ast_node_alloc_unary(yymsp[-6].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED, yymsp[-1].minor.yy121),
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE, yymsp[-4].minor.yy121));
}
#line 4054 "mlr_dsl_parse.c"
        break;
      case 213:
#line 1178 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-8].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED,
		mlr_dsl_ast_node_alloc_binary(yymsp[-8].minor.yy0->text, MD_AST_NODE_TYPE_EMIT_LASHED, yymsp[-3].minor.yy121, yymsp[0].minor.yy121),
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE, yymsp[-6].minor.yy121));
}
#line 4063 "mlr_dsl_parse.c"
        break;
      case 214:
#line 1185 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("lashed_keylists", MD_AST_NODE_TYPE_EMIT_LASHED, yymsp[0].minor.yy121);
}
#line 4070 "mlr_dsl_parse.c"
        break;
      case 216:
#line 1192 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("lashed_namelist", MD_AST_NODE_TYPE_EMIT_LASHED, yymsp[0].minor.yy121);
}
#line 4077 "mlr_dsl_parse.c"
        break;
      case 218:
#line 1200 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[0].minor.yy0->text, MD_AST_NODE_TYPE_DUMP,
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
			mlr_dsl_ast_node_alloc_zary("stdout", MD_AST_NODE_TYPE_STDOUT)));
}
#line 4086 "mlr_dsl_parse.c"
        break;
      case 219:
#line 1205 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[0].minor.yy0->text, MD_AST_NODE_TYPE_DUMP,
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
			mlr_dsl_ast_node_alloc_zary("stdout", MD_AST_NODE_TYPE_STDERR)));
}
#line 4095 "mlr_dsl_parse.c"
        break;
      case 220:
#line 1210 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[-2].minor.yy0->text, MD_AST_NODE_TYPE_DUMP,
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
		yymsp[0].minor.yy121));
}
#line 4104 "mlr_dsl_parse.c"
        break;
      case 221:
#line 1215 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[-2].minor.yy0->text, MD_AST_NODE_TYPE_DUMP,
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND,
		yymsp[0].minor.yy121));
}
#line 4113 "mlr_dsl_parse.c"
        break;
      case 222:
#line 1220 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[-2].minor.yy0->text, MD_AST_NODE_TYPE_DUMP,
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE,
		yymsp[0].minor.yy121));
}
#line 4122 "mlr_dsl_parse.c"
        break;
      case 223:
#line 1226 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_PRINT, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
			mlr_dsl_ast_node_alloc_zary("stdout", MD_AST_NODE_TYPE_STDOUT)));
}
#line 4131 "mlr_dsl_parse.c"
        break;
      case 224:
#line 1231 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_PRINT, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
			mlr_dsl_ast_node_alloc_zary("stdout", MD_AST_NODE_TYPE_STDERR)));
}
#line 4140 "mlr_dsl_parse.c"
        break;
      case 225:
#line 1236 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_PRINT, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
		yymsp[-2].minor.yy121));
}
#line 4149 "mlr_dsl_parse.c"
        break;
      case 226:
#line 1241 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_PRINT, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND,
		yymsp[-2].minor.yy121));
}
#line 4158 "mlr_dsl_parse.c"
        break;
      case 227:
#line 1246 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_PRINT, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE,
		yymsp[-2].minor.yy121));
}
#line 4167 "mlr_dsl_parse.c"
        break;
      case 228:
#line 1252 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_PRINTN, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
			mlr_dsl_ast_node_alloc_zary("stdout", MD_AST_NODE_TYPE_STDOUT)));
}
#line 4176 "mlr_dsl_parse.c"
        break;
      case 229:
#line 1257 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_PRINTN, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
			mlr_dsl_ast_node_alloc_zary("stdout", MD_AST_NODE_TYPE_STDERR)));
}
#line 4185 "mlr_dsl_parse.c"
        break;
      case 230:
#line 1262 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_PRINTN, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary(">", MD_AST_NODE_TYPE_FILE_WRITE,
		yymsp[-2].minor.yy121));
}
#line 4194 "mlr_dsl_parse.c"
        break;
      case 231:
#line 1267 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_PRINTN, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary(">>", MD_AST_NODE_TYPE_FILE_APPEND,
		yymsp[-2].minor.yy121));
}
#line 4203 "mlr_dsl_parse.c"
        break;
      case 232:
#line 1272 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-4].minor.yy0->text, MD_AST_NODE_TYPE_PRINTN, yymsp[0].minor.yy121,
		mlr_dsl_ast_node_alloc_unary("|", MD_AST_NODE_TYPE_PIPE,
		yymsp[-2].minor.yy121));
}
#line 4212 "mlr_dsl_parse.c"
        break;
      case 234:
      case 235:
#line 1280 "mlr_dsl_parse.y"
{ yygotominor.yy121 = yymsp[0].minor.yy0; }
#line 4218 "mlr_dsl_parse.c"
        break;
      case 237:
#line 1290 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_ternary("? :", MD_AST_NODE_TYPE_OPERATOR, yymsp[-4].minor.yy121, yymsp[-2].minor.yy121, yymsp[0].minor.yy121);
}
#line 4225 "mlr_dsl_parse.c"
        break;
      case 240:
      case 242:
      case 244:
      case 246:
      case 247:
      case 248:
      case 249:
      case 251:
      case 252:
      case 253:
      case 254:
      case 256:
      case 258:
      case 260:
      case 262:
      case 263:
      case 265:
      case 266:
      case 267:
      case 269:
      case 270:
      case 271:
      case 272:
      case 279:
#line 1302 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_OPERATOR, yymsp[-2].minor.yy121, yymsp[0].minor.yy121);
}
#line 4255 "mlr_dsl_parse.c"
        break;
      case 274:
      case 275:
      case 276:
      case 277:
#line 1426 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary(yymsp[-1].minor.yy0->text, MD_AST_NODE_TYPE_OPERATOR, yymsp[0].minor.yy121);
}
#line 4265 "mlr_dsl_parse.c"
        break;
      case 281:
#line 1457 "mlr_dsl_parse.y"
{
	char* dollar_name = yymsp[0].minor.yy0->text;
	char* no_dollar_name = &dollar_name[1];
	yygotominor.yy121 = mlr_dsl_ast_node_alloc(no_dollar_name, yymsp[0].minor.yy0->type);
}
#line 4274 "mlr_dsl_parse.c"
        break;
      case 282:
#line 1462 "mlr_dsl_parse.y"
{
	// Replace "${field.name}" with just "field.name"
	char* dollar_name = yymsp[0].minor.yy0->text;
	char* no_dollar_name = &dollar_name[2];
	int len = strlen(no_dollar_name);
	if (len > 0)
		no_dollar_name[len-1] = 0;
	yygotominor.yy121 = mlr_dsl_ast_node_alloc(no_dollar_name, yymsp[0].minor.yy0->type);
}
#line 4287 "mlr_dsl_parse.c"
        break;
      case 284:
#line 1475 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("indirect_field_name", MD_AST_NODE_TYPE_INDIRECT_FIELD_NAME, yymsp[-1].minor.yy121);
}
#line 4294 "mlr_dsl_parse.c"
        break;
      case 287:
#line 1487 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_append_arg(yymsp[-3].minor.yy121, yymsp[-1].minor.yy121);
}
#line 4301 "mlr_dsl_parse.c"
        break;
      case 288:
#line 1492 "mlr_dsl_parse.y"
{
	char* at_name = yymsp[0].minor.yy0->text;
	char* no_at_name = &at_name[1];
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("oosvar_keylist", MD_AST_NODE_TYPE_OOSVAR_KEYLIST,
		mlr_dsl_ast_node_alloc(no_at_name, yymsp[0].minor.yy0->type));
}
#line 4311 "mlr_dsl_parse.c"
        break;
      case 289:
#line 1500 "mlr_dsl_parse.y"
{
	// Replace "@%{field.name}" with just "field.name"
	char* at_name = yymsp[0].minor.yy0->text;
	char* no_at_name = &at_name[2];
	int len = strlen(no_at_name);
	if (len > 0)
		no_at_name[len-1] = 0;
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("oosvar_keylist", MD_AST_NODE_TYPE_OOSVAR_KEYLIST,
		mlr_dsl_ast_node_alloc(no_at_name, yymsp[0].minor.yy0->type));
}
#line 4325 "mlr_dsl_parse.c"
        break;
      case 290:
#line 1512 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("oosvar_keylist", MD_AST_NODE_TYPE_OOSVAR_KEYLIST, yymsp[-1].minor.yy121);
}
#line 4332 "mlr_dsl_parse.c"
        break;
      case 291:
      case 292:
      case 293:
      case 300:
#line 1517 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = yymsp[0].minor.yy0;
}
#line 4342 "mlr_dsl_parse.c"
        break;
      case 297:
#line 1537 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc(yymsp[0].minor.yy0->text, MD_AST_NODE_TYPE_BOUND_VARIABLE);
}
#line 4349 "mlr_dsl_parse.c"
        break;
      case 298:
#line 1541 "mlr_dsl_parse.y"
{
	char* input = yymsp[0].minor.yy0->text;
	char* stripped = &input[1];
	int len = strlen(input);
	stripped[len-2] = 0;
	yygotominor.yy121 = mlr_dsl_ast_node_alloc(mlr_alloc_unbackslash(stripped), yymsp[0].minor.yy0->type);
}
#line 4360 "mlr_dsl_parse.c"
        break;
      case 299:
#line 1548 "mlr_dsl_parse.y"
{
	char* input = yymsp[0].minor.yy0->text;
	char* stripped = &input[1];
	int len = strlen(input);
	stripped[len-3] = 0;
	yygotominor.yy121 = mlr_dsl_ast_node_alloc(mlr_alloc_unbackslash(stripped), yymsp[0].minor.yy0->type);
}
#line 4371 "mlr_dsl_parse.c"
        break;
      case 301:
#line 1559 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_binary("env", MD_AST_NODE_TYPE_ENV, yymsp[-3].minor.yy0, yymsp[-1].minor.yy121);
}
#line 4378 "mlr_dsl_parse.c"
        break;
      case 302:
#line 1563 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = yymsp[-1].minor.yy121;
}
#line 4385 "mlr_dsl_parse.c"
        break;
      case 303:
#line 1574 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_set_function_name(yymsp[-1].minor.yy121, yymsp[-3].minor.yy0->text);
}
#line 4392 "mlr_dsl_parse.c"
        break;
      case 304:
#line 1578 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_zary("anon", MD_AST_NODE_TYPE_NON_SIGIL_NAME);
}
#line 4399 "mlr_dsl_parse.c"
        break;
      case 305:
#line 1581 "mlr_dsl_parse.y"
{
	yygotominor.yy121 = mlr_dsl_ast_node_alloc_unary("anon", MD_AST_NODE_TYPE_NON_SIGIL_NAME, yymsp[0].minor.yy121);
}
#line 4406 "mlr_dsl_parse.c"
        break;
	};
	yygoto = yyRuleInfo[yyruleno].lhs;
	yysize = yyRuleInfo[yyruleno].nrhs;
	pparser->yyidx -= yysize;
	yyact = yy_find_reduce_action(pparser,yygoto);
	if (yyact < YYNSTATE) {
		yy_shift(pparser,yyact,yygoto,&yygotominor);
	} else if (yyact == YYNSTATE + YYNRULE + 1) {
		yy_accept(pparser);
	}
}

/*
** The following code executes when the parse fails
*/
static void yy_parse_failed(lemon_parser_t *pparser) {
	ParseARG_FETCH;
#ifndef NDEBUG
	if (yyTraceFILE) {
		fprintf(yyTraceFILE,"%sFail!\n",yyTracePrompt);
	}
#endif
	while (pparser->yyidx >= 0)
		yy_pop_parser_stack(pparser);
	/* Here code is inserted which will be executed whenever the
	** parser fails */
	ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/*
** The following code executes when a syntax error first occurs.
*/
static void yy_syntax_error(
	lemon_parser_t *pparser,           /* The parser */
	int yymajor,                   /* The major type of the error token */
	YYMINORTYPE yyminor            /* The minor type of the error token */
){
	ParseARG_FETCH;
#define TOKEN (yyminor.yy0)
#line 50 "mlr_dsl_parse.y"

	fprintf(stderr, "mlr DSL: syntax error.\n");

#line 4451 "mlr_dsl_parse.c"
	ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/*
** The following is executed when the parser accepts
*/
static void yy_accept(lemon_parser_t *pparser) {
	ParseARG_FETCH;
#ifndef NDEBUG
	if (yyTraceFILE) {
		fprintf(yyTraceFILE,"%sAccept!\n", yyTracePrompt);
	}
#endif
	while (pparser->yyidx>=0)
		yy_pop_parser_stack(pparser);
	/* Here code is inserted which will be executed whenever the
	** parser accepts */
#line 46 "mlr_dsl_parse.y"


#line 4472 "mlr_dsl_parse.c"
	ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/* The main parser program.
** The first argument is a pointer to a structure obtained from
** "mlr_dsl_lemon_parser_alloc" which describes the current state of the parser.
** The second argument is the major token number.  The third is
** the minor token.  The fourth optional argument is whatever the
** user wants (and specified in the grammar) and is available for
** use by the action routines.
**
** Inputs:
** <ul>
** <li> A pointer to the parser (an opaque structure).
** <li> The major token number.
** <li> The minor token number.
** <li> An option argument of a grammar-specified type.
** </ul>
**
** Outputs:
** None.
*/

int mlr_dsl_lemon_parser_parse_token(
	void *pvparser,              /* The parser */
	int yymajor,                 /* The major token code number */
	ParseTOKENTYPE yyminor       /* The value for the token */
	ParseARG_PDECL)              /* Optional %extra_argument parameter */
{
	YYMINORTYPE yyminorunion;
	int yyact;            /* The parser action. */
	int yyendofinput;     /* True if we are at the end of input */
	int yyerrorhit = 0;   /* True if yymajor has invoked an error */
	lemon_parser_t *pparser;  /* The parser */

	int parse_code = 1;

	/* (re)initialize the parser, if necessary */
	pparser = (lemon_parser_t*)pvparser;
	if (pparser->yyidx<0) {
		if (yymajor==0)  return 1;
		pparser->yyidx = 0;
		pparser->yyerrcnt = -1;
		pparser->yystack[0].stateno = 0;
		pparser->yystack[0].major = 0;
	}
	yyminorunion.yy0 = yyminor;
	yyendofinput = (yymajor==0);
	ParseARG_STORE;

#ifndef NDEBUG
	if (yyTraceFILE) {
		fprintf(yyTraceFILE,"%sInput %s\n", yyTracePrompt, yyTokenName[yymajor]);
	}
#endif

	do {
		yyact = yy_find_shift_action(pparser,yymajor);
		if (yyact<YYNSTATE) {
			yy_shift(pparser,yyact,yymajor,&yyminorunion);
			pparser->yyerrcnt--;
			if (yyendofinput && pparser->yyidx>=0) {
				yymajor = 0;
			} else {
				yymajor = YYNOCODE;
			}
		} else if (yyact < YYNSTATE + YYNRULE) {
			yy_reduce(pparser,yyact-YYNSTATE);
		} else if (yyact == YY_ERROR_ACTION) {
			int yymx;
#ifndef NDEBUG
			if (yyTraceFILE) {
				fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
			}
#endif
#ifdef YYERRORSYMBOL
			/* A syntax error has occurred.
			** The response to an error depends upon whether or not the
			** grammar defines an error token "ERROR".
			**
			** This is what we do if the grammar does define ERROR:
			**
			**  * Call the %syntax_error function.
			**
			**  * Begin popping the stack until we enter a state where
			**    it is legal to shift the error symbol, then shift
			**    the error symbol.
			**
			**  * Set the error count to three.
			**
			**  * Begin accepting and shifting new tokens.  No new error
			**    processing will occur until three tokens have been
			**    shifted successfully.
			**
			*/
			if (pparser->yyerrcnt<0) {
				yy_syntax_error(pparser, yymajor, yyminorunion);
				return 0;
			}
			yymx = pparser->yystack[pparser->yyidx].major;
			if (yymx==YYERRORSYMBOL || yyerrorhit) {
#ifndef NDEBUG
				if (yyTraceFILE) {
					fprintf(yyTraceFILE,"%sDiscard input token %s\n",
						 yyTracePrompt, yyTokenName[yymajor]);
				}
#endif
				mlr_dsl_yy_destructor(yymajor,&yyminorunion);
				yymajor = YYNOCODE;
			} else {
				 while(
					pparser->yyidx >= 0 &&
					yymx != YYERRORSYMBOL &&
					(yyact = yy_find_shift_action(pparser,YYERRORSYMBOL)) >= YYNSTATE
				){
					yy_pop_parser_stack(pparser);
				}
				if (pparser->yyidx < 0 || yymajor==0) {
					mlr_dsl_yy_destructor(yymajor,&yyminorunion);
					yy_parse_failed(pparser);
					parse_code = 0;
					yymajor = YYNOCODE;
				} else if (yymx!=YYERRORSYMBOL) {
					YYMINORTYPE u2;
					u2.YYERRSYMDT = 0;
					yy_shift(pparser,yyact,YYERRORSYMBOL,&u2);
				}
			}
			pparser->yyerrcnt = 3;
			yyerrorhit = 1;
#else  /* YYERRORSYMBOL is not defined */
			/* This is what we do if the grammar does not define ERROR:
			**
			**  * Report an error message, and throw away the input token.
			**
			**  * If the input token is $, then fail the parse.
			**
			** As before, subsequent error messages are suppressed until
			** three input tokens have been successfully shifted.
			*/
			if (pparser->yyerrcnt<=0) {
				yy_syntax_error(pparser, yymajor, yyminorunion);
				return 0;
			}
			pparser->yyerrcnt = 3;
			mlr_dsl_yy_destructor(yymajor,&yyminorunion);
			if (yyendofinput) {
				yy_parse_failed(pparser);
				parse_code = 0;
			}
			yymajor = YYNOCODE;
#endif
		} else {
			yy_accept(pparser);
			yymajor = YYNOCODE;
		}
	} while (yymajor!=YYNOCODE && pparser->yyidx>=0) ;
	return parse_code;
}
